@@ -13,7 +13,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -31,7 +31,7 @@
 #include <math.h>
 #include <errno.h>
 
-float fabsf(const float x) _FLOAT_FUNC_REENTRANT
+float fabsf(float x) _FLOAT_FUNC_REENTRANT
 {
     union float_long fl;
 
