@@ -1,7 +1,7 @@
 /*-------------------------------------------------------------------------
    logf.c - Computes the natural log of a 32 bit float as outlined in [1].
 
-   Copyright (C) 2001,2002, Jesus Calvino-Fraga, jesusc@ieee.org 
+   Copyright (C) 2001,2002, Jesus Calvino-Fraga, jesusc@ieee.org
 
    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -13,7 +13,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -43,7 +43,7 @@
 #define C1  0.693359375 /*355.0/512.0*/
 #define C2 -2.121944400546905827679E-4
 
-float logf(const float x) _MATH_REENTRANT
+float logf(float x) _MATH_REENTRANT
 {
     float Rz;
     float f, z, w, znum, zden, xn;
