@@ -1,7 +1,7 @@
 /*-------------------------------------------------------------------------
    sqrtf.c - Computes square root of a 32-bit float as outlined in [1]
 
-   Copyright (C) 2001,2002, Jesus Calvino-Fraga, jesusc@ieee.org 
+   Copyright (C) 2001,2002, Jesus Calvino-Fraga, jesusc@ieee.org
 
    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -13,7 +13,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -33,7 +33,7 @@
 #include <math.h>
 #include <errno.h>
 
-float sqrtf(const float x) _MATH_REENTRANT
+float sqrtf(float x) _MATH_REENTRANT
 {
     float f, y;
     FS_STATIC int n;
