@@ -1,7 +1,7 @@
 /*-------------------------------------------------------------------------
    tanhf.c - Computes tanh(x) where x is a 32-bit float as outlined in [1].
 
-   Copyright (C) 2001,2002, Jesus Calvino-Fraga, jesusc@ieee.org 
+   Copyright (C) 2001,2002, Jesus Calvino-Fraga, jesusc@ieee.org
 
    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -13,7 +13,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -45,7 +45,7 @@
 #define P(g) ((P1*g+P0)*g)
 #define Q(g) (Q1*g+Q0)
 
-float tanhf(const float x) _MATH_REENTRANT
+float tanhf(float x) _MATH_REENTRANT
 {
     float f, g, r;
 
@@ -65,4 +65,3 @@ float tanhf(const float x) _MATH_REENTRANT
     if(x<0.0) r=-r;
     return r;
 }
-
