@@ -160,11 +160,25 @@
  #if defined(__BORLANDC__) || defined(_MSC_VER)
   #include <stdlib.h>
   #define PATH_MAX      _MAX_PATH
+ #elif defined(__x86_64__)
+  #define PATH_MAX      4096
  #else
   #define PATH_MAX      255     /* define a reasonable value */
  #endif
 #endif
 
+#ifdef _WIN32           /* WIN32 native */
+#  define NATIVE_WIN32          1
+#  ifdef __MINGW32__    /* GCC MINGW32 depends on configure */
+#    include "../../sdccconf.h"
+#  else
+#    include "../../sdcc_vc.h"
+#    define PATH_MAX    _MAX_PATH
+#  endif
+#else                   /* Assume *nix style system */
+#  include "../../sdccconf.h"
+#endif
+
 #define LKOBJEXT        "rel"
 
 /*
@@ -1113,7 +1127,7 @@ extern  char *sdld_output;      /*      output file name, --output argument
 extern char *optsdcc;
 extern char *optsdcc_module;
 /* sdld 8015 specific */
-extern  char idatamap[];        /* 0:not used, 1:used
+extern  char idatamap[256];     /*      space is unused
                                  */
 /* end sdld 8051 specific */
 /* end sdld specific */
