@@ -25,6 +25,7 @@ enum sdld_target_e {
   TARGET_ID_Z180,
   TARGET_ID_8051,
   TARGET_ID_6808,
+  TARGET_ID_STM8,
 };
 
 void sdld_init (char *path);
@@ -37,6 +38,7 @@ int is_sdld_target_8051_like(void);
 #define TARGET_IS_Z80   (get_sdld_target() == TARGET_ID_Z80)
 #define TARGET_IS_Z180  (get_sdld_target() == TARGET_ID_Z180)
 #define TARGET_IS_8051  (get_sdld_target() == TARGET_ID_8051)
-#define TARGET_IS_6808 	(get_sdld_target() == TARGET_ID_6808)
+#define TARGET_IS_6808  (get_sdld_target() == TARGET_ID_6808)
+#define TARGET_IS_STM8  (get_sdld_target() == TARGET_ID_STM8)
 
 #endif  /* __SDLD_H */
