@@ -1,7 +1,7 @@
 /*
  * Simulator of microcontrollers (appcl.h)
  *
- * Copyright (C) 2001,01 Drotos Daniel, Talker Bt.
+ * Copyright (C) 1997,16 Drotos Daniel, Talker Bt.
  * 
  * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
  *
@@ -39,17 +39,23 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "simcl.h"
 
 
+extern bool jaj;
+
 /* Options */
-
-#define OPT_GENERAL	0x0001
-#define OPT_SIM		0x0002
-#define OPT_UC		0x0004
-#define OPT_PRG_OPT	(OPT_GENERAL|OPT_SIM|OPT_UC)
-#define OPT_51		0x0010
-#define OPT_AVR		0x0020
-#define OPT_Z80		0x0040
-#define OPT_TARGET	(OPT_51|OPT_AVR|OPT_Z80)
-
+/*
+enum opt_types {
+  OPT_GENERAL	= 0x0001,
+  OPT_SIM	= 0x0002,
+  OPT_UC	= 0x0004,
+  OPT_PRG_OPT	= (OPT_GENERAL|OPT_SIM|OPT_UC),
+  OPT_51	= 0x0010,
+  OPT_AVR	= 0x0020,
+  OPT_Z80	= 0x0040,
+  OPT_HC08	= 0x0080,
+  OPT_XA	= 0x0100,
+  OPT_TARGET	= (OPT_51|OPT_AVR|OPT_Z80|OPT_HC08|OPT_XA)
+};
+*/
 /*class cl_option: public cl_base
 {
 public:
@@ -82,10 +88,10 @@ protected:
 public:
   class cl_sim *sim;
   class cl_ustrings *in_files;
-  //class cl_arguments *args;
   class cl_options *options;
   int going;
-
+  long expr_result;
+  
 public:
   cl_app(void);
   virtual ~cl_app(void);
@@ -102,8 +108,10 @@ public:
   class cl_sim *get_sim(void) { return(sim); }
   class cl_uc *get_uc(void);
   class cl_commander_base *get_commander(void) { return(commander); }
-  virtual class cl_cmd *get_cmd(class cl_cmdline *cmdline);
-
+  //virtual class cl_cmd *get_cmd(class cl_cmdline *cmdline);
+  virtual long eval(chars expr);
+  virtual void exec(chars line);
+  
 public: // messages to broadcast
   //virtual void mem_cell_changed(class cl_m *mem, t_addr addr);
 
