@@ -28,12 +28,17 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "ddconfig.h"
 
 #include <stdlib.h>
+#include <stdarg.h>
 #include "i_string.h"
 
 // prj
+#include "fiocl.h"
+#include "utils.h"
+#include "appcl.h"
 
 // local, cmd
 #include "commandcl.h"
+#include "argcl.h"
 
 
 /*
@@ -42,7 +47,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  */
 
 cl_cmdline::cl_cmdline(class cl_app *the_app,
-                       char *acmd, class cl_console_base *acon):
+		       char *acmd, class cl_console_base *acon):
   cl_base()
 {
   app= the_app;
@@ -65,6 +70,9 @@ cl_cmdline::~cl_cmdline(void)
 int
 cl_cmdline::init(void)
 {
+  rest= NULL;
+  params->free_all();
+  tokens->free_all();
   split();
   return(0);
 }
@@ -73,7 +81,7 @@ char *
 cl_cmdline::skip_delims(char *start)
 {
   while (*start &&
-         strchr(" \t\v\r,", *start))
+	 strchr(" \t\v\r,", *start))
     start++;
   return(start);
 }
@@ -83,71 +91,91 @@ cl_cmdline::split(void)
 {
   //class cl_sim *sim;
   char *start= cmd;
-  int i;
+  int i;//, j;
   class cl_cmd_arg *arg;
 
-  //sim= app->get_sim();
   set_name("\n");
   if (!cmd ||
       !*cmd)
     return(0);
   start+= strspn(start, " \t\v\r,");
-  if (start &&
-      *start == '\n')
+  if (!start)
+    return 0;
+  set_name(0);
+  if (*start == '\n')
     {
-      char *n= (char*)malloc(2);
-      strcpy(n, "\n");
-      set_name(n);
+      // never, as \n stripped by readline
+      set_name("\n");
       return(0);
     }
+  else if (*start == '#')
+    return *start= 0;
+  else if (*start == ';')
+    {
+      rest= start+1;
+      *start= 0;
+      return 0;
+    }
   if (!*start)
     return(0);
-  i= strcspn(start, " \t\v\r,");
+  // start now points to first word
+  i= strcspn(start, " \t\v\r,;#");
+  // i should be at end of it
   if (i)
     {
+      if (*start == '#')
+	return set_name("\n"), *start= 0;
       char *n= (char*)malloc(i+1);
       strncpy(n, start, i);
       n[i]= '\0';
       set_name(n);
+      free(n);
     }
   start+= i;
   start= skip_delims(start);
-  // skip delimiters
   while (*start)
     {
       char *end= start, *param_str;
-      if (*start == '"')
-        split_out_string(&start, &end);
+      if (*start == '#')
+	return *start= '\0';
+      else if (*start == ';')
+	{
+	  rest= start+1;
+	  *start= 0;
+	  return 0;
+	}
+      else if (*start == '"')
+	split_out_string(&start, &end);
       else if (*start == '>')
-        split_out_output_redirection(&start, &end);
+	split_out_output_redirection(&start, &end);
       else
-        {
-          char *dot;
-          i= strcspn(start, " \t\v\r,");
+	{
+	  char *dot;
+          i= strcspn(start, " \t\v\r,#;");
           end= start+i;
           param_str= (char *)malloc(i+1);
           strncpy(param_str, start, i);
-          param_str[i]= '\0';
-          tokens->add(strdup(param_str));
-          if ((dot= strchr(param_str, '.')) != NULL)
-            split_out_bit(dot, param_str);
-          else if ((dot= strchr(param_str, '[')) != NULL)
-            split_out_array(dot, param_str);
-          else if (strchr("0123456789-+", *param_str) != NULL)
-            {
-              // number
-              params->add(arg= new cl_cmd_int_arg((long)
-                                                  strtol(param_str, 0, 0)));
-              arg->init();
-            }
-          else
-            {
-              // symbol
-              params->add(arg= new cl_cmd_sym_arg(param_str));
-              arg->init();
-            }
-          free(param_str);
-        }
+	  param_str[i]= '\0';
+	  tokens->add(strdup(param_str));
+	  if ((dot= strchr(param_str, '.')) != NULL)
+	    split_out_bit(dot, param_str);
+	  else if ((dot= strchr(param_str, '[')) != NULL)
+	    split_out_array(dot, param_str);
+	  else if (strchr("0123456789-+", *param_str) != NULL)
+	    {
+	      // number
+	      params->add(arg= new cl_cmd_int_arg((long)
+						  strtol(param_str, 0, 0)));
+	      arg->init();
+	    }
+	  else
+	    {
+	      // symbol
+	      params->add(arg= new cl_cmd_sym_arg(param_str));
+	      arg->init();
+	    }
+	  free(param_str);
+	}
       start= end;
       start= skip_delims(start);
     }
@@ -325,21 +353,23 @@ cl_cmdline::shift(void)
 {
   char *s= skip_delims(cmd);
 
+  params->free_all();
+  tokens->free_all();
   set_name(0);
   if (s && *s)
     {
       while (*s &&
-             strchr(" \t\v\r,", *s) == NULL)
-        s++;
+	     strchr(" \t\v\r,;#", *s) == NULL)
+	s++;
       s= skip_delims(s);
-      char *p= strdup(s);
+      char *p= strdup(s), *r= rest?strdup(rest):NULL;
       free(cmd);
       cmd= p;
-      delete params;
-      params= new cl_list(2, 2, "params");
+      rest= r;
+      //params= new cl_list(2, 2, "params");
       split();
       if (strcmp(get_name(), "\n") == 0)
-        set_name(0);
+	set_name(0);
     }
   return(have_real_name());
 }
@@ -349,7 +379,7 @@ cl_cmdline::repeat(void)
 {
   const char *n;
   return((n= get_name()) &&
-         *n == '\n');
+	 *n == '\n');
 }
 
 class cl_cmd_arg *
@@ -373,81 +403,89 @@ bool
 cl_cmdline::syntax_match(class cl_uc *uc, const char *syntax)
 {
   if (!syntax)
-    return(DD_FALSE);
+    return(false);
   if (!*syntax &&
       !params->count)
     {
       matched_syntax= syntax;
-      return(DD_TRUE);
+      return(true);
     }
   if (!params->count)
-    return(DD_FALSE);
+    return(false);
   //printf("syntax %s?\n",syntax);
   const char *p= syntax;
   int iparam= 0;
   class cl_cmd_arg *parm= (class cl_cmd_arg *)(params->at(iparam));
   while (*p &&
-         parm)
+	 parm)
     {
       //printf("***Checking %s as %c\n",parm->get_svalue(),*p);
       if (uc)
-        switch (*p)
-          {
-          case SY_ADDR:
-            if (!parm->as_address(uc))
-              return(DD_FALSE);
-            //printf("ADDRESS match %lx\n",parm->value.address);
-            break;
-          case SY_MEMORY:
-            if (!parm->as_memory(uc))
-              return(DD_FALSE);
-            //printf("MEMORY match %s\n",parm->value.memory->class_name);
-            break;
-          case SY_BIT:
-            if (!parm->as_bit(uc))
-              return(DD_FALSE);
-            break;
-          }
-      switch (*p)
-        {
-        case SY_ADDR: case SY_MEMORY: case SY_BIT: break;
-        case SY_NUMBER:
-          if (!parm->as_number())
-            return(DD_FALSE);
-          break;
-        case SY_DATA:
-          if (!parm->as_data())
-            return(DD_FALSE);
-          break;
-        case SY_HW:
-          if (!parm->as_hw(uc))
-            return(DD_FALSE);
-          break;
-        case SY_STRING:
-          if (!parm->as_string())
-            return(DD_FALSE);
-          break;
-        case SY_DATALIST:
-          if (!set_data_list(parm, &iparam))
-            return(DD_FALSE);
-          break;
-        default:
-          return(DD_FALSE);
-        }
+	{
+	  switch (*p)
+	    {
+	    case SY_ADDR:
+	      if (!parm->as_address(uc))
+		return(false);
+	      //printf("ADDRESS match %lx\n",parm->value.address);
+	      break;
+	    case SY_MEMORY:
+	      if (!parm->as_memory(uc))
+		return(false);
+	      //printf("MEMORY match %s\n",parm->value.memory->class_name);
+	      break;
+	    case SY_BIT:
+	      if (!parm->as_bit(uc))
+		return(false);
+	      break;
+	    case SY_HW:
+	      if (!parm->as_hw(uc))
+		return(false);
+	      break;
+	    case SY_CELL:
+	      if (!parm->as_cell(uc))
+		return false;
+	      break;
+	    }
+	}
+      //else
+	{
+	  switch (*p)
+	    {
+	    case SY_ADDR: case SY_MEMORY: case SY_BIT: break;
+	    case SY_NUMBER:
+	      if (!parm->as_number())
+		return(false);
+	      break;
+	    case SY_DATA:
+	      if (!parm->as_data())
+		return(false);
+	      break;
+	    case SY_STRING:
+	      if (!parm->as_string())
+		return(false);
+	      break;
+	    case SY_DATALIST:
+	      if (!set_data_list(parm, &iparam))
+		return(false);
+	      break;
+	      //default: return(false);
+	    }
+	}
       p++;
       iparam++;
       if (iparam < params->count)
-        parm= (class cl_cmd_arg *)(params->at(iparam));
+	parm= (class cl_cmd_arg *)(params->at(iparam));
       else
-        parm= 0;
+	parm= 0;
     }
   if (!*p &&
       !parm)
     {
       matched_syntax= syntax;
-      return(DD_TRUE);
+      return(true);
     }
-  return(DD_FALSE);
+  return(false);
 }
 
 bool
@@ -462,59 +500,75 @@ cl_cmdline::set_data_list(class cl_cmd_arg *parm, int *iparm)
   for (i= *iparm, next_parm= param(i); next_parm; i++, next_parm= param(i))
     {
       if (next_parm->is_string())
-        {
-          int l;
-          char *s;
-          //s= proc_escape(next_parm->get_svalue(), &l);
-          if (!next_parm->as_string())
-            continue;
-          s= next_parm->value.string.string;
-          l= next_parm->value.string.len;
-          if (!array)
-            array= (t_mem*)malloc(sizeof(t_mem)*l);
-          else
-            array= (t_mem*)realloc(array, sizeof(t_mem)*(l+len));
-          for (j= 0; j < l; j++)
-            {
-              array[len]= s[j];
-              len++;
-            }
-          //if (s)
-          //free(s);
-        }
+	{
+	  int l;
+	  char *s;
+	  //s= proc_escape(next_parm->get_svalue(), &l);
+	  if (!next_parm->as_string())
+	    continue;
+	  s= next_parm->value.string.string;
+	  l= next_parm->value.string.len;
+	  if (!array)
+	    array= (t_mem*)malloc(sizeof(t_mem)*l);
+	  else
+	    array= (t_mem*)realloc(array, sizeof(t_mem)*(l+len));
+	  for (j= 0; j < l; j++)
+	    {
+	      array[len]= s[j];
+	      len++;
+	    }
+	  //if (s)
+	  //free(s);
+	}
       else
-        {
-          if (!next_parm->as_data())
-            {
-              if (array)
-                free(array);
-              return(DD_FALSE);
-            }
-          if (!array)
-            array= (t_mem*)malloc(sizeof(t_mem));
-          else
-            array= (t_mem*)realloc(array, sizeof(t_mem)*(1+len));
-          array[len]= next_parm->value.data;
-          len++;
-        }
+	{
+	  if (!next_parm->as_data())
+	    {
+	      if (array)
+		free(array);
+	      return(false);
+	    }
+	  if (!array)
+	    array= (t_mem*)malloc(sizeof(t_mem));
+	  else
+	    array= (t_mem*)realloc(array, sizeof(t_mem)*(1+len));
+	  array[len]= next_parm->value.data;
+	  len++;
+	}
     }
   *iparm= i;
   parm->value.data_list.array= array;
   parm->value.data_list.len= len;
-  return(DD_TRUE);
+  return(true);
 }
 
+bool
+cl_cmdline::restart_at_rest(void)
+{
+  char *newcmd;
+  if ((rest == NULL) ||
+      (*rest == 0))
+    {
+      return false;
+    }
+  newcmd= strdup(rest);
+  if (cmd)
+    free(cmd);
+  cmd= newcmd;
+  return true;
+}
 
+  
 /*
  * Command
  *____________________________________________________________________________
  */
 
 cl_cmd::cl_cmd(enum cmd_operate_on op_on,
-               const char *aname,
-               int can_rep,
-               const char *short_hlp,
-               const char *long_hlp):
+	       const char *aname,
+	       int can_rep,
+	       const char *short_hlp,
+	       const char *long_hlp):
   cl_base()
 {
   operate_on= op_on;
@@ -536,6 +590,10 @@ cl_cmd::cl_cmd(enum cmd_operate_on op_on,
 cl_cmd::~cl_cmd(void)
 {
   delete names;
+  if (short_help)
+    free((void*)short_help);
+  if (long_help)
+    free((void*)long_help);
 }
 
 void
@@ -558,20 +616,20 @@ cl_cmd::name_match(const char *aname, int strict)
   if (strict)
     {
       for (i= 0; i < names->count; i++)
-        {
-          char *n= (char*)(names->at(i));
-          if (strcmp(aname, n) == 0)
-            return(1);
-        }
+	{
+	  char *n= (char*)(names->at(i));
+	  if (strcmp(aname, n) == 0)
+	    return(1);
+	}
     }
   else
     {
       for (i= 0; i < names->count; i++)
-        {
-          char *n= (char*)(names->at(i));
-          if (strstr(n, aname) == n)
-            return(1);
-        }
+	{
+	  char *n= (char*)(names->at(i));
+	  if (strstr(n, aname) == n)
+	    return(1);
+	}
     }
   return(0);
 }
@@ -590,7 +648,7 @@ cl_cmd::syntax_ok(class cl_cmdline *cmdline)
 
 int
 cl_cmd::work(class cl_app *app,
-             class cl_cmdline *cmdline, class cl_console_base *con)
+	     class cl_cmdline *cmdline, class cl_console_base *con)
 {
   if (!syntax_ok(cmdline))
     return(0);
@@ -602,24 +660,24 @@ cl_cmd::work(class cl_app *app,
     {
     case operate_on_app:
       if (!app)
-        {
-          con->dd_printf("There is no application to work on!\n");
-          return(DD_TRUE);
-        }
+	{
+	  con->dd_printf("There is no application to work on!\n");
+	  return(false);
+	}
       return(do_work(app, cmdline, con));
     case operate_on_sim:
       if (!sim)
-        {
-          con->dd_printf("There is no simulator to work on!\n");
-          return(DD_TRUE);
-        }
+	{
+	  con->dd_printf("There is no simulator to work on!\n");
+	  return(false);
+	}
       return(do_work(sim, cmdline, con));
     case operate_on_uc:
       if (!sim)
-        {
-          con->dd_printf("There is no microcontroller to work on!\n");
-          return(DD_TRUE);
-        }
+	{
+	  con->dd_printf("There is no microcontroller to work on!\n");
+	  return(false);
+	}
       return(do_work(uc, cmdline, con));
     default:
       return(do_work(cmdline, con));
@@ -630,34 +688,34 @@ int
 cl_cmd::do_work(class cl_cmdline *cmdline, class cl_console_base *con)
 {
   con->dd_printf("Command \"%s\" does nothing.\n",
-                 (char*)(names->at(0)));
+		 (char*)(names->at(0)));
   return(0);
 }
 
 int
 cl_cmd::do_work(class cl_app *app,
-                class cl_cmdline *cmdline, class cl_console_base *con)
+		class cl_cmdline *cmdline, class cl_console_base *con)
 {
   con->dd_printf("Command \"%s\" does nothing on application.\n",
-                 (char*)(names->at(0)));
+		 (char*)(names->at(0)));
   return(0);
 }
 
 int
 cl_cmd::do_work(class cl_sim *sim,
-                class cl_cmdline *cmdline, class cl_console_base *con)
+		class cl_cmdline *cmdline, class cl_console_base *con)
 {
   con->dd_printf("Command \"%s\" does nothing on simulator.\n",
-                 (char*)(names->at(0)));
+		 (char*)(names->at(0)));
   return(0);
 }
 
 int
 cl_cmd::do_work(class cl_uc *uc,
-                class cl_cmdline *cmdline, class cl_console_base *con)
+		class cl_cmdline *cmdline, class cl_console_base *con)
 {
   con->dd_printf("Command \"%s\" does nothing on microcontroller.\n",
-                 (char*)(names->at(0)));
+		 (char*)(names->at(0)));
   return(0);
 }
 
@@ -691,7 +749,7 @@ cl_cmdset::get_cmd(class cl_cmdline *cmdline, bool accept_last)
     {
       class cl_cmd *c= (class cl_cmd *)at(i);
       if (c->name_match(cmdline, 1))
-        return(c);
+	return(c);
     }
   // not exact match
   class cl_cmd *c_matched= 0;
@@ -699,12 +757,12 @@ cl_cmdset::get_cmd(class cl_cmdline *cmdline, bool accept_last)
     {
       class cl_cmd *c= (class cl_cmd *)at(i);
       if (c->name_match(cmdline, 0))
-        {
-          if (!c_matched)
-            c_matched= c;
-          else
-            return(0);
-        }
+	{
+	  if (!c_matched)
+	    c_matched= c;
+	  else
+	    return(0);
+	}
     }
   return(c_matched);
   //return(0);
@@ -719,7 +777,7 @@ cl_cmdset::get_cmd(const char *cmd_name)
     {
       class cl_cmd *c= (class cl_cmd *)at(i);
       if (c->name_match(cmd_name, 1))
-        return(c);
+	return(c);
     }
   return(0);
 }
@@ -735,7 +793,7 @@ cl_cmdset::del(char *nam)
     {
       class cl_cmd *cmd= (class cl_cmd *)(at(i));
       if (cmd->name_match(nam, 1))
-        free_at(i);
+	free_at(i);
     }
 }
 
@@ -750,10 +808,10 @@ cl_cmdset::replace(char *nam, class cl_cmd *cmd)
     {
       class cl_cmd *c= (class cl_cmd *)(at(i));
       if (c->name_match(nam, 1))
-        {
-          delete c;
-          put_at(i, cmd);
-        }
+	{
+	  delete c;
+	  put_at(i, cmd);
+	}
     }
 }
 
@@ -764,10 +822,10 @@ cl_cmdset::replace(char *nam, class cl_cmd *cmd)
  */
 
 cl_super_cmd::cl_super_cmd(const char *aname,
-                           int  can_rep,
-                           const char *short_hlp,
-                           const char *long_hlp,
-                           class cl_cmdset *acommands):
+			   int  can_rep,
+			   const char *short_hlp,
+			   const char *long_hlp,
+			   class cl_cmdset *acommands):
   cl_cmd(operate_on_none, aname, can_rep, short_hlp, long_hlp)
 {
   commands= acommands;
@@ -781,7 +839,7 @@ cl_super_cmd::~cl_super_cmd(void)
 
 int
 cl_super_cmd::work(class cl_app *app,
-                   class cl_cmdline *cmdline, class cl_console_base *con)
+		   class cl_cmdline *cmdline, class cl_console_base *con)
 {
   class cl_cmd *cmd= 0;
 
@@ -791,21 +849,21 @@ cl_super_cmd::work(class cl_app *app,
   if (!cmdline->shift())
     {
       if ((cmd= commands->get_cmd("_no_parameters_")) != 0)
-        return(cmd->work(app, cmdline, con));
+	return(cmd->work(app, cmdline, con));
       int i;
       con->dd_printf("\"%s\" must be followed by the name of a subcommand\n"
-                     "List of subcommands:\n", (char*)(names->at(0)));
+		     "List of subcommands:\n", (char*)(names->at(0)));
       for (i= 0; i < commands->count; i++)
-        {
-          cmd= (class cl_cmd *)(commands->at(i));
-          con->dd_printf("%s\n", cmd->short_help);
-        }
+	{
+	  cmd= (class cl_cmd *)(commands->at(i));
+	  con->dd_printf("%s\n", cmd->short_help);
+	}
       return(0);
     }
-  if ((cmd= commands->get_cmd(cmdline, con->accept_last())) == NULL)
+  if ((cmd= commands->get_cmd(cmdline, con->is_interactive())) == NULL)
     {
       con->dd_printf("Undefined subcommand: \"%s\". Try \"help %s\".\n",
-                     cmdline->get_name(), (char*)(names->at(0)));
+		     cmdline->get_name(), (char*)(names->at(0)));
       return(0);
     }
   return(cmd->work(app, cmdline, con));
