@@ -1,7 +1,7 @@
 /*
  * Simulator of microcontrollers (pobjt.h)
  *
- * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
+ * Copyright (C) 1997,16 Drotos Daniel, Talker Bt.
  * 
  * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
  *
@@ -30,19 +30,15 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 #include "ddconfig.h"
 
-#ifndef MyTypesHeader
-#define DD_TRUE	 1
-#define DD_FALSE 0
-#define bool	 int
-#endif
 
 typedef int	t_index;
-typedef int	(*match_func)(void *, const void *);
+typedef int	(*match_func)(void *, void *);
 typedef void	(*iterator_func)(void *, void *);
 
 #define max_list_size (0x10000/sizeof(void *))
 #define ccNotFound -1
 
+
 #endif
 
 /* End of pobjt.h */
