@@ -36,10 +36,9 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  * Making an 251 CPU object
  */
 
-cl_uc251::cl_uc251(int Itype, int Itech, class cl_sim *asim):
-  cl_uc89c51r(Itype, Itech, asim)
+cl_uc251::cl_uc251(struct cpu_entry *Itype, class cl_sim *asim):
+  cl_uc89c51r(Itype, asim)
 {
 }
 
-
 /* End of s51.src/uc251.cc */
