@@ -60,7 +60,8 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 cl_st7::cl_st7(class cl_sim *asim):
   cl_uc(asim)
 {
-  type= CPU_ST7;
+  type= (struct cpu_entry *)malloc(sizeof(struct cpu_entry));
+  type->type= CPU_ST7;
 }
 
 int
@@ -70,12 +71,12 @@ cl_st7::init(void)
 
   xtal = 8000000;
 
-  rom = address_space(MEM_ROM_ID);
+  //rom = address_space(MEM_ROM_ID);
 //  ram = mem(MEM_XRAM);
-  ram = rom;
+  //ram = rom;
 
   // zero out ram(this is assumed in regression tests)
-  printf("******************** leave the RAM dirty now \n");
+  //printf("******************** leave the RAM dirty now \n");
 //  for (int i=0x0; i<0x8000; i++) {
 //    ram->set((t_addr) i, 0);
 //  }
@@ -99,10 +100,10 @@ cl_st7::reset(void)
 }
 
 
-const char *
+char *
 cl_st7::id_string(void)
 {
-  return("unspecified ST7");
+  return((char*)"unspecified ST7");
 }
 
 
@@ -126,7 +127,7 @@ void
 cl_st7::mk_hw_elements(void)
 {
   //class cl_base *o;
-  /* t_uc::mk_hw() does nothing */
+  cl_uc::mk_hw_elements();
 }
 
 void
@@ -134,7 +135,7 @@ cl_st7::make_memories(void)
 {
   class cl_address_space *as;
 
-  as= new cl_address_space("rom", 0, 0x10000, 8);
+  rom= ram= as= new cl_address_space("rom", 0, 0x10000, 8);
   as->init();
   address_spaces->add(as);
 
@@ -148,6 +149,34 @@ cl_st7::make_memories(void)
   ad->init();
   as->decoders->add(ad);
   ad->activate(0);
+
+  regs8= new cl_address_space("regs8", 0, 2, 8);
+  regs8->init();
+  regs8->get_cell(0)->decode((t_mem*)&regs.A);
+  regs8->get_cell(1)->decode((t_mem*)&regs.CC);
+
+  regs16= new cl_address_space("regs16", 0, 3, 16);
+  regs16->init();
+
+  regs16->get_cell(0)->decode((t_mem*)&regs.X);
+  regs16->get_cell(1)->decode((t_mem*)&regs.Y);
+  regs16->get_cell(2)->decode((t_mem*)&regs.SP);
+
+  address_spaces->add(regs8);
+  address_spaces->add(regs16);
+
+  class cl_var *v;
+  vars->add(v= new cl_var(cchars("A"), regs8, 0, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("CC"), regs8, 1, ""));
+  v->init();
+  
+  vars->add(v= new cl_var(cchars("X"), regs16, 0, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("Y"), regs16, 1, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("SP"), regs16, 2, ""));
+  v->init();
 }
 
 
@@ -179,21 +208,30 @@ cl_st7::inst_length(t_addr addr)
 {
   int len = 0;
 
-  get_disasm_info(addr, &len, NULL, NULL);
+  get_disasm_info(addr, &len, NULL, NULL, NULL);
 
   return len;
 }
-
 int
 cl_st7::inst_branch(t_addr addr)
 {
   int b;
 
-  get_disasm_info(addr, NULL, &b, NULL);
+  get_disasm_info(addr, NULL, &b, NULL, NULL);
 
   return b;
 }
 
+bool
+cl_st7::is_call(t_addr addr)
+{
+  struct dis_entry *e;
+
+  get_disasm_info(addr, NULL, NULL, NULL, &e);
+
+  return e?(e->is_call):false;
+}
+
 int
 cl_st7::longest_inst(void)
 {
@@ -205,7 +243,8 @@ const char *
 cl_st7::get_disasm_info(t_addr addr,
                         int *ret_len,
                         int *ret_branch,
-                        int *immed_offset)
+                        int *immed_offset,
+			struct dis_entry **dentry)
 {
   const char *b = NULL;
   uint code;
@@ -215,13 +254,13 @@ cl_st7::get_disasm_info(t_addr addr,
   int start_addr = addr;
   struct dis_entry *dis_e;
 
-  code= get_mem(MEM_ROM_ID, addr++);
+  code= rom->get(addr++);
   dis_e = NULL;
 
   switch(code) {
 	/* here will be all the prefixes for ST7 */
 	case 0x90 :
-	  code= get_mem(MEM_ROM_ID, addr++);
+	  code= rom->get(addr++);
       i= 0;
       while ((code & disass_st7_90[i].mask) != disass_st7_90[i].code &&
         disass_st7_90[i].mnemonic)
@@ -233,7 +272,7 @@ cl_st7::get_disasm_info(t_addr addr,
     break;
 	  
 	case 0x91 :
-	  code= get_mem(MEM_ROM_ID, addr++);
+	  code= rom->get(addr++);
       i= 0;
       while ((code & disass_st7_91[i].mask) != disass_st7_91[i].code &&
         disass_st7_91[i].mnemonic)
@@ -245,7 +284,7 @@ cl_st7::get_disasm_info(t_addr addr,
     break;
 	  
 	case 0x92 :
-	  code= get_mem(MEM_ROM_ID, addr++);
+	  code= rom->get(addr++);
       i= 0;
       while ((code & disass_st7_92[i].mask) != disass_st7_92[i].code &&
         disass_st7_92[i].mnemonic)
@@ -285,10 +324,13 @@ cl_st7::get_disasm_info(t_addr addr,
   if (ret_len)
     *ret_len = len;
 
+  if (dentry)
+    *dentry= dis_e;
+  
   return b;
 }
 
-const char *
+char *
 cl_st7::disass(t_addr addr, const char *sep)
 {
   char work[256], temp[20];
@@ -297,9 +339,10 @@ cl_st7::disass(t_addr addr, const char *sep)
   int len = 0;
   int immed_offset = 0;
 
+
   p= work;
 
-  b = get_disasm_info(addr, &len, NULL, &immed_offset);
+  b = get_disasm_info(addr, &len, NULL, &immed_offset, NULL);
 
   if (b == NULL) {
     buf= (char*)malloc(30);
@@ -315,64 +358,64 @@ cl_st7::disass(t_addr addr, const char *sep)
           switch (*(b++))
             {
             //case 's': // s    signed byte immediate
-            //  sprintf(temp, "#%d", (char)get_mem(MEM_ROM_ID, addr+immed_offset));
+            //  sprintf(temp, "#%d", (char)rom->get(addr+immed_offset));
             //  ++immed_offset;
             //  break;
             //case 'e': // e    extended 24bit immediate operand
             //  sprintf(temp, "#0x%06lx",
-            //     (ulong)((get_mem(MEM_ROM_ID, addr+immed_offset)<<16) |
-            //            (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8) |
-            //            (get_mem(MEM_ROM_ID, addr+immed_offset+2))) );
+            //     (ulong)((rom->get(addr+immed_offset)<<16) |
+            //            (rom->get(addr+immed_offset+1)<<8) |
+            //            (rom->get(addr+immed_offset+2))) );
             //  ++immed_offset;
             //  ++immed_offset;
             //  ++immed_offset;
             //  break;
             //case 'w': // w    word immediate operand
             //  sprintf(temp, "#0x%04x",
-            //     (uint)((get_mem(MEM_ROM_ID, addr+immed_offset)<<8) |
-            //            (get_mem(MEM_ROM_ID, addr+immed_offset+1))) );
+            //     (uint)((rom->get(addr+immed_offset)<<8) |
+            //            (rom->get(addr+immed_offset+1))) );
             //  ++immed_offset;
             //  ++immed_offset;
             //  break;
             case 'b': // b    byte immediate operand
-              sprintf(temp, "#0x%02x", (uint)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "#0x%02x", (uint)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             case 'd': // d    short direct addressing
-              sprintf(temp, "$0x%02x", (uint)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "$0x%02x", (uint)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             case 'x': // x    long direct
               sprintf(temp, "$0x%04x",
-                 (uint)((get_mem(MEM_ROM_ID, addr+immed_offset)<<8) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+1))) );
+                 (uint)((rom->get(addr+immed_offset)<<8) |
+                        (rom->get(addr+immed_offset+1))) );
               ++immed_offset;
               ++immed_offset;
               break;
             //case '3': // 3    24bit index offset
             //  sprintf(temp, "0x%06lx",
-            //     (ulong)((get_mem(MEM_ROM_ID, addr+immed_offset)<<16) |
-            //            (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8) |
-            //            (get_mem(MEM_ROM_ID, addr+immed_offset+2))) );
+            //     (ulong)((rom->get(addr+immed_offset)<<16) |
+            //            (rom->get(addr+immed_offset+1)<<8) |
+            //            (rom->get(addr+immed_offset+2))) );
             //  ++immed_offset;
             //  ++immed_offset;
             //  ++immed_offset;
             // break;
             case '2': // 2    word index offset
               sprintf(temp, "0x%04x",
-                 (uint)((get_mem(MEM_ROM_ID, addr+immed_offset)<<8) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+1))) );
+                 (uint)((rom->get(addr+immed_offset)<<8) |
+                        (rom->get(addr+immed_offset+1))) );
               ++immed_offset;
               ++immed_offset;
               break;
             case '1': // b    byte index offset
-              sprintf(temp, "0x%02x", (uint)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "0x%02x", (uint)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             case 'p': // b    byte index offset
-              sprintf(temp, "0x%04x",
-                 addr+immed_offset+1
-                 +(char)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "0x%04lx",
+		      (long int)(addr+immed_offset+1
+				 +(char)rom->get(addr+immed_offset)));
               ++immed_offset;
               break;
             default:
@@ -456,7 +499,7 @@ cl_st7::exec_inst(void)
 
 
   if (fetch(&code)) {
-	  printf("******************** break \n");
+    //printf("******************** break \n");
 	  return(resBREAKPOINT);
   }
   tick(1);
@@ -473,7 +516,7 @@ cl_st7::exec_inst(void)
 		break;
   }
 	
-  printf("********************  switch; pc=0x%x, prefix = 0x%x, code = 0x%x\n",PC, cprefix, code);
+  //printf("********************  switch; pc=0x%lx, prefix = 0x%x, code = 0x%x\n",(long int)(PC), cprefix, code);
   switch (code & 0xf) {
    int mulres;
    
@@ -512,7 +555,7 @@ cl_st7::exec_inst(void)
             break;
          case 0x90: // this is prefix, do not be mad ???
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 		
@@ -546,7 +589,7 @@ cl_st7::exec_inst(void)
          case 0x70: // 
          case 0x90: // this is prefix, do not be mad ???
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -590,7 +633,7 @@ cl_st7::exec_inst(void)
          case 0x80: // 
          case 0x90: // this is prefix, do not be mad ???
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -613,7 +656,7 @@ cl_st7::exec_inst(void)
             return( inst_cpl( code, cprefix));
             break;
          case 0x80: // trap
-            printf("************* TRAP instruction unimplemented !!!!\n");
+	   //printf("************* TRAP instruction unimplemented !!!!\n");
             return(resINV_INST);
             break;
          case 0x90: // 
@@ -635,7 +678,7 @@ cl_st7::exec_inst(void)
             return( inst_cpxy( code, cprefix));
             break;
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -680,7 +723,7 @@ cl_st7::exec_inst(void)
             return( inst_and( code, cprefix));
             break;
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -727,7 +770,7 @@ cl_st7::exec_inst(void)
          case 0x60:
          case 0x70: // 
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -772,7 +815,7 @@ cl_st7::exec_inst(void)
             return( inst_lda( code, cprefix));
             break;
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -815,7 +858,7 @@ cl_st7::exec_inst(void)
             break;
          case 0x80: // 
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -854,7 +897,7 @@ cl_st7::exec_inst(void)
             return( inst_xor( code, cprefix));
             break;
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -899,7 +942,7 @@ cl_st7::exec_inst(void)
             return( inst_adc( code, cprefix));
             break;
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -938,7 +981,7 @@ cl_st7::exec_inst(void)
             return( inst_or( code, cprefix));
             break;
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -972,7 +1015,7 @@ cl_st7::exec_inst(void)
          case 0x70: // 
          case 0x80: // 
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -1010,7 +1053,7 @@ cl_st7::exec_inst(void)
             break;
          case 0x80: // 
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -1047,7 +1090,7 @@ cl_st7::exec_inst(void)
             break;
          case 0x80: // 
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -1091,7 +1134,7 @@ cl_st7::exec_inst(void)
             return( inst_ldxy( code, cprefix));
             break;
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
@@ -1137,13 +1180,13 @@ cl_st7::exec_inst(void)
             break;
          case 0xA0:
          default: 
-            printf("************* bad code !!!!\n");
+	   //printf("************* bad code !!!!\n");
             return(resINV_INST);
 		}
 
 
     default:
-		printf("************* bad code !!!!\n");
+      //printf("************* bad code !!!!\n");
 		return(resINV_INST);
 	}
 	  
@@ -1157,5 +1200,27 @@ cl_st7::exec_inst(void)
   return(resINV_INST);
 }
 
+t_mem
+cl_st7::get_1(t_addr addr)
+{
+  vc.rd++;
+  return ram->read(addr);
+}
+
+t_mem
+cl_st7::get_2(t_addr addr)
+{
+  vc.rd+= 2;
+  return (ram->read(addr) << 8) | ram->read(addr+1);
+}
+
+t_mem
+cl_st7::get_3(t_addr addr)
+{
+  vc.rd+= 3;
+  return (ram->read(addr) << 16) |
+    (ram->read(addr+1) << 8) |
+    (ram->read(addr+2));
+}
 
 /* End of st7.src/st7.cc */
