@@ -28,6 +28,8 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA. */
 /*@1@*/
 
+/* $Id: stm8.cc 776 2017-07-09 11:45:22Z drdani $ */
+
 #include "ddconfig.h"
 
 #include <stdarg.h> /* for va_list */
@@ -38,6 +40,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 // prj
 #include "pobjcl.h"
+#include "globals.h"
 
 // sim
 #include "simcl.h"
@@ -47,9 +50,15 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "glob.h"
 #include "regsstm8.h"
 #include "stm8mac.h"
-
-#define uint32 t_addr
-#define uint8 unsigned char
+#include "itccl.h"
+#include "serialcl.h"
+#include "rstcl.h"
+#include "timercl.h"
+#include "portcl.h"
+#include "clkcl.h"
+#include "uidcl.h"
+#include "bl.h"
+#include "flashcl.h"
 
 /*******************************************************************/
 
@@ -58,10 +67,11 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  * Base type of STM8 controllers
  */
 
-cl_stm8::cl_stm8(class cl_sim *asim):
+cl_stm8::cl_stm8(struct cpu_entry *IType, class cl_sim *asim):
   cl_uc(asim)
 {
-  type= CPU_STM8;
+  type= IType;
+  flash_ctrl= NULL;
 }
 
 int
@@ -71,16 +81,23 @@ cl_stm8::init(void)
 
   xtal = 8000000;
 
-  rom = address_space(MEM_ROM_ID);
-//  ram = mem(MEM_XRAM);
-  ram = rom;
+  //rom = address_space(MEM_ROM_ID);
+  //ram = mem(MEM_XRAM);
+  //ram = rom;
 
-  printf("******************** leave the RAM dirty now \n");
+  //printf("******************** leave the RAM dirty now \n");
   // zero out ram(this is assumed in regression tests)
   //for (int i=0x0; i<0x8000; i++) {
   //  ram->set((t_addr) i, 0);
   //}
 
+  trap_src= new cl_it_src(this, -1,
+			  (cl_memory_cell*)NULL, (t_mem)0,
+			  (cl_memory_cell*)NULL, (t_mem)0,
+			  (t_addr)0x8004,
+			  false, false,
+			  "trap", 0);
+  trap_src->init();
   return(0);
 }
 
@@ -94,15 +111,26 @@ cl_stm8::reset(void)
   regs.A = 0;
   regs.X = 0;
   regs.Y = 0;
-  regs.CC = 0x00;
-  regs.VECTOR = 1;
+  regs.CC = 0x28;
+  //regs.VECTOR = 1;
+  PC= 0x8000;
 }
 
 
-const char *
+char *
 cl_stm8::id_string(void)
 {
-  return("unspecified STM8");
+  switch (type->type)
+    {
+    case CPU_STM8S:
+      return((char*)"STM8 S,AF");
+    case CPU_STM8L:
+      return((char*)"STM8 AL,L");
+    case CPU_STM8L101:
+      return((char*)"STM8 L101");
+    default:
+      return((char*)"STM8");
+    }
 }
 
 
@@ -122,32 +150,494 @@ cl_stm8::get_mem_size(enum mem_class type)
  return(cl_uc::get_mem_size(type));
 }
 */
+
+ /*
+   L15x46 uid: 0x4926 00 5b 00 16 11 47 30 31 38 35 35 36
+ */
+
+static class cl_port_ui *d= NULL;
+static int puix= 1;
+static int puiy= 4;
+static int puik= 0;
+static int puis= 1;
+static const char *puiks= keysets[puik];
+static class cl_port_data pd;
+
+void
+cl_stm8::mk_port(t_addr base, chars n)
+{
+  class cl_port *p;
+  add_hw(p= new cl_port(this, base, n));
+  p->init();
+
+  pd.set_name(n);
+  pd.cell_p  = p->cell_p;
+  pd.cell_in = p->cell_in;
+  pd.cell_dir= p->cell_dir;
+  pd.keyset  = chars(puiks);
+  pd.basx    = puix;
+  pd.basy    = puiy;
+  d->add_port(&pd, puis++);
+  
+  if ((puix+= 20) > 80)
+    {
+      puix= 1;
+      if ((puiy+= 7) > 20)
+	;
+    }
+  if ((puik+= 1) > 6)
+    puiks= NULL;
+  else
+    puiks= keysets[puik];
+}
+
 void
 cl_stm8::mk_hw_elements(void)
 {
-  //class cl_base *o;
-  /* t_uc::mk_hw() does nothing */
+  class cl_hw *h;
+  cl_uc::mk_hw_elements();
+  class cl_option *o;
+
+  if ((o= application->options->get_option("serial1_in_file")) == NULL)
+    {
+      o= new cl_string_option(this, "serial1_in_file",
+			      "Input file for serial line uart1 (-S)");
+      application->options->new_option(o);
+      o->init();
+      o->hide();
+    }
+  if ((o= application->options->get_option("serial1_out_file")) == NULL)
+    {
+      o= new cl_string_option(this, "serial1_out_file",
+			      "Output file for serial line uart1 (-S)");
+      application->options->new_option(o);
+      o->init();
+      o->hide();
+    }
+  
+  if ((o= application->options->get_option("serial2_in_file")) == NULL)
+    {
+      o= new cl_string_option(this, "serial2_in_file",
+			      "Input file for serial line uart2 (-S)");
+      application->options->new_option(o);
+      o->init();
+      o->hide();
+    }
+  if ((o= application->options->get_option("serial2_out_file")) == NULL)
+    {
+      o= new cl_string_option(this, "serial2_out_file",
+			      "Output file for serial line uart2 (-S)");
+      application->options->new_option(o);
+      o->init();
+      o->hide();
+    }
+  
+  if ((o= application->options->get_option("serial3_in_file")) == NULL)
+    {
+      o= new cl_string_option(this, "serial3_in_file",
+			      "Input file for serial line uart3 (-S)");
+      application->options->new_option(o);
+      o->init();
+      o->hide();
+    }
+  if ((o= application->options->get_option("serial3_out_file")) == NULL)
+    {
+      o= new cl_string_option(this, "serial3_out_file",
+			      "Output file for serial line uart3 (-S)");
+      application->options->new_option(o);
+      o->init();
+      o->hide();
+    }
+  
+  if ((o= application->options->get_option("serial4_in_file")) == NULL)
+    {
+      o= new cl_string_option(this, "serial4_in_file",
+			      "Input file for serial line uart4 (-S)");
+      application->options->new_option(o);
+      o->init();
+      o->hide();
+    }
+  if ((o= application->options->get_option("serial4_out_file")) == NULL)
+    {
+      o= new cl_string_option(this, "serial4_out_file",
+			      "Output file for serial line uart4 (-S)");
+      application->options->new_option(o);
+      o->init();
+      o->hide();
+    }
+  
+  add_hw(d= new cl_port_ui(this, 0, "dport"));
+  d->init();
+  pd.init();
+  
+  add_hw(h= new cl_stm8_cpu(this));
+  h->init();
+  
+  if (type->type == CPU_STM8S)
+    {
+      add_hw(h= new cl_clk_saf(this));
+      h->init();
+      if (type->subtype & (DEV_STM8S003|
+			   DEV_STM8S007|
+			   DEV_STM8S103|
+			   DEV_STM8S207|
+			   DEV_STM8S208|
+			   DEV_STM8S903|
+			   DEV_STM8AF52))
+	{
+	  add_hw(h= new cl_serial(this, 0x5230, 1, 17, 18));
+	  h->init();
+	}
+      if (type->subtype & (DEV_STM8S005|
+			   DEV_STM8S105|
+			   DEV_STM8AF62_46))
+	{
+	  add_hw(h= new cl_serial(this, 0x5240, 2, 20, 21));
+	  h->init();
+	}
+      if (type->subtype & (DEV_STM8S007|
+			   DEV_STM8S207|
+			   DEV_STM8S208|
+			   DEV_STM8AF52))
+	{
+	  add_hw(h= new cl_serial(this, 0x5240, 3, 20, 21));
+	  h->init();
+	}
+      if (type->subtype & (DEV_STM8AF62_12))
+	{
+	  add_hw(h= new cl_serial(this, 0x5230, 4, 17, 18));
+	  h->init();
+	}
+    }
+  if (type->type == CPU_STM8L)
+    {
+      add_hw(h= new cl_clk_all(this));
+      h->init();
+      add_hw(h= new cl_serial(this, 0x5230, 1, 27, 28));
+      h->init();
+      if (type->subtype & (DEV_STM8AL3xE|
+			   DEV_STM8AL3x8|
+			   DEV_STM8L052R|
+			   DEV_STM8L15x8|
+			   DEV_STM8L162))
+	{
+	  add_hw(h= new cl_serial(this, 0x53e0, 2, 19, 20));
+	  h->init();
+	}
+      if (type->subtype & (DEV_STM8AL3xE|
+			   DEV_STM8AL3x8|
+			   DEV_STM8L052R|
+			   DEV_STM8L15x8|
+			   DEV_STM8L162))
+	{
+	  add_hw(h= new cl_serial(this, 0x53f0, 3, 21, 22));
+	  h->init();
+	}
+    }
+  if (type->type == CPU_STM8L101)
+    {
+      add_hw(h= new cl_clk_l101(this));
+      h->init();
+      add_hw(h= new cl_serial(this, 0x5230, 1, 27, 28));
+      h->init();
+    }
+
+  add_hw(itc= new cl_itc(this));
+  itc->init();
+
+  {
+    mk_port(0x5000, "pa");
+    mk_port(0x5005, "pb");
+    mk_port(0x500a, "pc");
+    mk_port(0x500f, "pd");
+  }
+  
+  if (type->type == CPU_STM8S)
+    {
+      // all S and AF
+      mk_port(0x5014, "pe");
+      mk_port(0x5019, "pf");
+      if (type->subtype & (DEV_STM8S005|
+			   DEV_STM8S007|
+			   DEV_STM8S105|
+			   DEV_STM8S207|
+			   DEV_STM8S208|
+			   DEV_STM8AF52|
+			   DEV_STM8AF62_46))
+	{
+	  mk_port(0x501e, "pg");
+	  if (type->subtype != DEV_STM8AF62_46)
+	    {
+	      mk_port(0x5023, "ph");
+	      mk_port(0x5028, "pi");
+	    }
+	}
+      add_hw(h= new cl_rst(this, 0x50b3, 0x1f));
+      h->init();
+      add_hw(h= new cl_tim1_saf(this, 1, 0x5250));
+      h->init();
+      // some S, some AF
+      if (type->subtype & (DEV_STM8S005|
+			   DEV_STM8S007|
+			   DEV_STM8S105|
+			   DEV_STM8S207|
+			   DEV_STM8S208|
+			   DEV_STM8AF52|
+			   DEV_STM8AF62_46))
+	{
+	  add_hw(h= new cl_tim2_saf_a(this, 2, 0x5300));
+	  h->init();
+	  add_hw(h= new cl_tim3_saf(this, 3, 0x5320));
+	  h->init();
+	  add_hw(h= new cl_tim4_saf_a(this, 4, 0x5340));
+	  h->init();
+	}
+      if (type->subtype & (DEV_STM8S903|
+			   DEV_STM8AF62_12))
+	{
+	  add_hw(h= new cl_tim5_saf(this, 5, 0x5300));
+	  h->init();
+	  add_hw(h= new cl_tim6_saf(this, 6, 0x5340));
+	  h->init();
+	}
+      if (type->subtype & (DEV_STM8S003|
+			   DEV_STM8S103))
+	{
+	  add_hw(h= new cl_tim2_saf_b(this, 2, 0x5300));
+	  h->init();
+	  // tim4 B
+	  add_hw(h= new cl_tim4_saf_b(this, 4, 0x5340));
+	  h->init();
+	}
+    }
+  else if (type->type == CPU_STM8L)
+    {
+      if (type->subtype != DEV_STM8L051)
+	{
+	  mk_port(0x5014, "pe");
+	  mk_port(0x5019, "pf");
+	}
+      if (type->subtype & (DEV_STM8AL3xE|
+			   DEV_STM8AL3x8|
+			   DEV_STM8L052R|
+			   DEV_STM8L15x8|
+			   DEV_STM8L162))
+	{
+	  mk_port(0x501e, "pg");
+	  if (type->subtype != DEV_STM8L052R)
+	    {
+	      mk_port(0x5023, "ph");
+	      mk_port(0x5028, "pi");
+	    }
+	}
+      add_hw(h= new cl_rst(this, 0x50b0+1, 0x3f));
+      h->init();
+      add_hw(h= new cl_tim2_all(this, 2, 0x5250));
+      h->init();
+      add_hw(h= new cl_tim3_all(this, 3, 0x5280));
+      h->init();
+      add_hw(h= new cl_tim4_all(this, 4, 0x52E0));
+      h->init();
+      // all AL
+      if (type->subtype & DEV_STM8AL)
+	{
+	  add_hw(h= new cl_tim1_all(this, 1, 0x52b0));
+	  h->init();
+	}
+      // some L
+      if (type->subtype & (DEV_STM8L052C |
+			   DEV_STM8L052R |
+			   DEV_STM8L15x46 |
+			   DEV_STM8L15x8 |
+			   DEV_STM8L162))
+	{
+	  add_hw(h= new cl_tim1_all(this, 1, 0x52b0));
+	  h->init();
+	}
+      if (type->subtype & (DEV_STM8AL3xE |
+			   DEV_STM8AL3x8 |
+			   DEV_STM8L052R |
+			   DEV_STM8L15x8 |
+			   DEV_STM8L162))
+	{
+	  add_hw(h= new cl_tim5_all(this, 5, 0x5300));
+	  h->init();
+	}
+    }
+  else if (type->type == CPU_STM8L101)
+    {
+      add_hw(h= new cl_rst(this, 0x50b0+1, 0x0f));
+      h->init();
+      add_hw(h= new cl_tim2_l101(this, 2, 0x5250));
+      h->init();
+      add_hw(h= new cl_tim3_l101(this, 2, 0x5280));
+      h->init();
+      add_hw(h= new cl_tim4_l101(this, 4, 0x52E0));
+      h->init();
+    }
+
+  // UID
+  if (type->subtype & (DEV_STM8S103 |
+		       DEV_STM8S903 |
+		       DEV_STM8AF62_12))
+    {
+      add_hw(h= new cl_uid(this, 0x4865));
+      h->init();
+    }
+  else if (type->subtype & (DEV_STM8AL |
+			    DEV_STM8L151x23 |
+			    DEV_STM8L15x46 |
+			    DEV_STM8L15x8 |
+			    DEV_STM8L162))
+    {
+      add_hw(h= new cl_uid(this, 0x4926));
+      h->init();
+    }
+  else if (type->subtype & (DEV_STM8L101))
+    {
+      add_hw(h= new cl_uid(this, 0x4925));
+      h->init();
+    }
+
+  // FLASH
+  if (type->subtype & (DEV_STM8SAF))
+    {
+      add_hw(flash_ctrl= new cl_saf_flash(this, 0x505a));
+      flash_ctrl->init();
+    }
+  else if (type->subtype & (DEV_STM8ALL |
+			    DEV_STM8L101))
+    {
+      add_hw(flash_ctrl= new cl_l_flash(this, 0x5050));
+      flash_ctrl->init();
+    }
+  //add_hw(h= new cl_tim235(this, 3, 0x5320));
+  //h->init();
+  //add_hw(h= new cl_tim46(this, 4, 0x5340));
+  //h->init();
 }
 
+class cl_memory_chip *c;
+
 void
 cl_stm8::make_memories(void)
 {
   class cl_address_space *as;
 
-  as= new cl_address_space("rom", 0, 0x10000, 8);
+  rom= ram= as= new cl_flash_as/*address_space*/("rom", 0, 0x28000/*, 8*/);
   as->init();
   address_spaces->add(as);
 
   class cl_address_decoder *ad;
-  class cl_memory_chip *chip;
+  class cl_memory_chip /* *chip,*/ *rom_chip;
 
-  chip= new cl_memory_chip("rom_chip", 0x10000, 8);
-  chip->init();
-  memchips->add(chip);
-  ad= new cl_address_decoder(as= address_space("rom"), chip, 0, 0xffff, 0);
+  c= rom_chip= NULL;/*new cl_memory_chip("rom_chip", 0x20000, 8, 0);
+  rom_chip->init();
+  memchips->add(rom_chip);*/
+
+  ram_chip= new cl_memory_chip("ram_chip", 0x1800, 8);
+  ram_chip->init();
+  memchips->add(ram_chip);
+  eeprom_chip= new cl_memory_chip("eeprom_chip", 0x0800, 8, 0);
+  eeprom_chip->init();
+  memchips->add(eeprom_chip);
+  option_chip= new cl_memory_chip("option_chip", 0x0800, 8, 0);
+  option_chip->init();
+  memchips->add(option_chip);
+  io_chip= new cl_memory_chip("io_chip", 0x0800, 8);
+  io_chip->init();
+  memchips->add(io_chip);
+  if (type->subtype & DEV_STM8S105)
+    boot_chip= new cl_memory_chip("boot_chip_s105", bl_s105_length, 8, bl_s105);
+  else if (type->subtype & DEV_STM8L15x46)
+    boot_chip= new cl_memory_chip("boot_chip_l15x46", bl_l15x46_length, 8, bl_l15x46);
+  /*else if (type->subtype & DEV_STM8L101)
+    boot_chip= new cl_memory_chip("boot_chip_l101", bl_l15x46_length, 8, bl_l15x46);*/
+  else
+    boot_chip= new cl_memory_chip("boot_chip", 0x0800, 8);
+  boot_chip->init();
+  memchips->add(boot_chip);
+  cpu_chip= new cl_memory_chip("cpu_chip", 0x0100, 8);
+  cpu_chip->init();
+  memchips->add(cpu_chip);
+  flash_chip= new cl_memory_chip("flash_chip", 0x20000, 8, 0);
+  flash_chip->init();
+  memchips->add(flash_chip);
+  /*
+  ad= new cl_address_decoder(as= address_space("rom"), rom_chip, 0, 0x1ffff, 0);
   ad->init();
   as->decoders->add(ad);
   ad->activate(0);
+  */
+  ad= new cl_address_decoder(as= rom, ram_chip, 0, 0x17ff, 0);
+  ad->init();
+  as->decoders->add(ad);
+  ad->activate(0);
+
+  ad= new cl_address_decoder(as= rom, eeprom_chip, 0x4000, 0x47ff, 0);
+  ad->init();
+  as->decoders->add(ad);
+  ad->activate(0);
+
+  ad= new cl_address_decoder(as= rom, option_chip, 0x4800, 0x4fff, 0);
+  ad->init();
+  as->decoders->add(ad);
+  ad->activate(0);
+
+  ad= new cl_address_decoder(as= rom, io_chip, 0x5000, 0x57ff, 0);
+  ad->init();
+  as->decoders->add(ad);
+  ad->activate(0);
+
+  ad= new cl_address_decoder(as= rom, boot_chip, 0x6000, 0x67ff, 0);
+  ad->init();
+  as->decoders->add(ad);
+  ad->activate(0);
+  rom->set_cell_flag(0x6000, 0x67ff, true, CELL_READ_ONLY);
+
+  ad= new cl_address_decoder(as= rom, cpu_chip, 0x7f00, 0x7fff, 0);
+  ad->init();
+  as->decoders->add(ad);
+  ad->activate(0);
+
+  ad= new cl_address_decoder(as= rom, flash_chip, 0x8000, 0x27fff, 0);
+  ad->init();
+  as->decoders->add(ad);
+  ad->activate(0);
+  class cl_option *o= application->options->get_option("writable_flash");
+  bool wv= false;
+  if (o)
+    o->get_value(&wv);
+  if (!wv)
+    rom->set_cell_flag(0x8000, 0x27fff, true, CELL_READ_ONLY);
+  
+  regs8= new cl_address_space("regs8", 0, 2, 8);
+  regs8->init();
+  regs8->get_cell(0)->decode((t_mem*)&regs.A);
+  regs8->get_cell(1)->decode((t_mem*)&regs.CC);
+
+  regs16= new cl_address_space("regs16", 0, 3, 16);
+  regs16->init();
+
+  regs16->get_cell(0)->decode((t_mem*)&regs.X);
+  regs16->get_cell(1)->decode((t_mem*)&regs.Y);
+  regs16->get_cell(2)->decode((t_mem*)&regs.SP);
+
+  address_spaces->add(regs8);
+  address_spaces->add(regs16);
+
+  class cl_var *v;
+  vars->add(v= new cl_var(cchars("A"), regs8, 0, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("CC"), regs8, 1, ""));
+  v->init();
+  
+  vars->add(v= new cl_var(cchars("X"), regs16, 0, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("Y"), regs16, 1, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("SP"), regs16, 2, ""));
+  v->init();
 }
 
 
@@ -179,21 +669,30 @@ cl_stm8::inst_length(t_addr addr)
 {
   int len = 0;
 
-  get_disasm_info(addr, &len, NULL, NULL);
+  get_disasm_info(addr, &len, NULL, NULL, NULL);
 
   return len;
 }
-
 int
 cl_stm8::inst_branch(t_addr addr)
 {
   int b;
 
-  get_disasm_info(addr, NULL, &b, NULL);
+  get_disasm_info(addr, NULL, &b, NULL, NULL);
 
   return b;
 }
 
+bool
+cl_stm8::is_call(t_addr addr)
+{
+  struct dis_entry *e;
+
+  get_disasm_info(addr, NULL, NULL, NULL, &e);
+
+  return e?(e->is_call):false;
+}
+
 int
 cl_stm8::longest_inst(void)
 {
@@ -201,11 +700,13 @@ cl_stm8::longest_inst(void)
 }
 
 
+
 const char *
 cl_stm8::get_disasm_info(t_addr addr,
-                        int *ret_len,
-                        int *ret_branch,
-                        int *immed_offset)
+			 int *ret_len,
+			 int *ret_branch,
+			 int *immed_offset,
+			 struct dis_entry **dentry)
 {
   const char *b = NULL;
   uint code;
@@ -215,13 +716,13 @@ cl_stm8::get_disasm_info(t_addr addr,
   int start_addr = addr;
   struct dis_entry *dis_e;
 
-  code= get_mem(MEM_ROM_ID, addr++);
+  code= rom->get(addr++);
   dis_e = NULL;
 
   switch(code) {
 	/* here will be all the prefixes for STM8 */
 	case 0x72 :
-	  code= get_mem(MEM_ROM_ID, addr++);
+	  code= rom->get(addr++);
       i= 0;
       while ((code & disass_stm8_72[i].mask) != disass_stm8_72[i].code &&
         disass_stm8_72[i].mnemonic)
@@ -231,9 +732,21 @@ cl_stm8::get_disasm_info(t_addr addr,
       if (b != NULL)
         len += (disass_stm8_72[i].length + 1);
     break;
-	  
+
+    	case 0x71 :
+	  code= rom->get(addr++);
+      i= 0;
+      while ((code & disass_stm8_71[i].mask) != disass_stm8_71[i].code &&
+        disass_stm8_71[i].mnemonic)
+        i++;
+      dis_e = &disass_stm8_71[i];
+      b= disass_stm8_71[i].mnemonic;
+      if (b != NULL)
+        len += (disass_stm8_71[i].length + 1);
+    break;
+
 	case 0x90 :
-	  code= get_mem(MEM_ROM_ID, addr++);
+	  code= rom->get(addr++);
       i= 0;
       while ((code & disass_stm8_90[i].mask) != disass_stm8_90[i].code &&
         disass_stm8_90[i].mnemonic)
@@ -245,7 +758,7 @@ cl_stm8::get_disasm_info(t_addr addr,
     break;
 	  
 	case 0x91 :
-	  code= get_mem(MEM_ROM_ID, addr++);
+	  code= rom->get(addr++);
       i= 0;
       while ((code & disass_stm8_91[i].mask) != disass_stm8_91[i].code &&
         disass_stm8_91[i].mnemonic)
@@ -257,7 +770,7 @@ cl_stm8::get_disasm_info(t_addr addr,
     break;
 	  
 	case 0x92 :
-	  code= get_mem(MEM_ROM_ID, addr++);
+	  code= rom->get(addr++);
       i= 0;
       while ((code & disass_stm8_92[i].mask) != disass_stm8_92[i].code &&
         disass_stm8_92[i].mnemonic)
@@ -297,10 +810,13 @@ cl_stm8::get_disasm_info(t_addr addr,
   if (ret_len)
     *ret_len = len;
 
+  if (dentry)
+    *dentry= dis_e;
+  
   return b;
 }
 
-const char *
+char *
 cl_stm8::disass(t_addr addr, const char *sep)
 {
   char work[256], temp[20];
@@ -309,9 +825,10 @@ cl_stm8::disass(t_addr addr, const char *sep)
   int len = 0;
   int immed_offset = 0;
 
+
   p= work;
 
-  b = get_disasm_info(addr, &len, NULL, &immed_offset);
+  b = get_disasm_info(addr, &len, NULL, &immed_offset, NULL);
 
   if (b == NULL) {
     buf= (char*)malloc(30);
@@ -327,65 +844,74 @@ cl_stm8::disass(t_addr addr, const char *sep)
           switch (*(b++))
             {
             case 's': // s    signed byte immediate
-              sprintf(temp, "#%d", (char)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "#%d", (char)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             case 'e': // e    extended 24bit immediate operand
               sprintf(temp, "#0x%06lx",
-                 (ulong)((get_mem(MEM_ROM_ID, addr+immed_offset)<<16) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+2))) );
+                 (ulong)((rom->get(addr+immed_offset)<<16) |
+                        (rom->get(addr+immed_offset+1)<<8) |
+                        (rom->get(addr+immed_offset+2))) );
               ++immed_offset;
               ++immed_offset;
               ++immed_offset;
               break;
             case 'w': // w    word immediate operand
               sprintf(temp, "#0x%04x",
-                 (uint)((get_mem(MEM_ROM_ID, addr+immed_offset)<<8) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+1))) );
+                 (uint)((rom->get(addr+immed_offset)<<8) |
+                        (rom->get(addr+immed_offset+1))) );
               ++immed_offset;
               ++immed_offset;
               break;
             case 'b': // b    byte immediate operand
-              sprintf(temp, "#0x%02x", (uint)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "#0x%02x", (uint)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             case 'x': // x    extended addressing
               sprintf(temp, "0x%04x",
-                 (uint)((get_mem(MEM_ROM_ID, addr+immed_offset)<<8) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+1))) );
+                 (uint)((rom->get(addr+immed_offset)<<8) |
+                        (rom->get(addr+immed_offset+1))) );
               ++immed_offset;
               ++immed_offset;
               break;
             case 'd': // d    direct addressing
-              sprintf(temp, "0x%02x", (uint)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "0x%02x", (uint)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             case '3': // 3    24bit index offset
               sprintf(temp, "0x%06lx",
-                 (ulong)((get_mem(MEM_ROM_ID, addr+immed_offset)<<16) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+2))) );
+                 (ulong)((rom->get(addr+immed_offset)<<16) |
+                        (rom->get(addr+immed_offset+1)<<8) |
+                        (rom->get(addr+immed_offset+2))) );
               ++immed_offset;
               ++immed_offset;
               ++immed_offset;
              break;
             case '2': // 2    word index offset
               sprintf(temp, "0x%04x",
-                 (uint)((get_mem(MEM_ROM_ID, addr+immed_offset)<<8) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+1))) );
+                 (uint)((rom->get(addr+immed_offset)<<8) |
+                        (rom->get(addr+immed_offset+1))) );
               ++immed_offset;
               ++immed_offset;
               break;
             case '1': // b    byte index offset
-              sprintf(temp, "0x%02x", (uint)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "0x%02x", (uint)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             case 'p': // b    byte index offset
-              sprintf(temp, "0x%04x",
-                 addr+immed_offset+1
-                 +(char)get_mem(MEM_ROM_ID, addr+immed_offset));
-              ++immed_offset;
+	      {
+		long int base;
+		i8_t offs;
+		base= addr+immed_offset+1;
+		offs= rom->get(addr+immed_offset);
+		long int res= base+offs;
+		sprintf(temp, "0x%04lx",
+			/*(long int)(addr+immed_offset+1
+			  +(int)rom->get(addr+immed_offset))*/
+			res
+			);
+		++immed_offset;
+	      }
               break;
             default:
               strcpy(temp, "?");
@@ -461,7 +987,7 @@ cl_stm8::exec_inst(void)
 {
   t_mem code;
   unsigned char cprefix; // prefix used for processing in functions
-
+  /*
   if (regs.VECTOR) {
     PC = get1(0x8000);
 	if (PC == 0x82) { // this is reserved opcode for vector table
@@ -473,25 +999,35 @@ cl_stm8::exec_inst(void)
 		return( resERROR);
 	}
   }
-
+  */
   if (fetch(&code)) {
-	  printf("******************** break \n");
+    //printf("******************** break \n");
 	  return(resBREAKPOINT);
   }
   tick(1);
 
-  switch (code) { // get prefix
-	case 0x72:
-	case 0x90:
-	case 0x91:
-	case 0x92:
-		cprefix = code;
-		fetch(&code);
-		break;
-	default:
-		cprefix = 0x00;
-		break;
-  }
+  switch (code)
+    { // get prefix
+    case 0x72:
+    case 0x90:
+    case 0x91:
+    case 0x92:
+      cprefix = code;
+      fetch(&code);
+      break;
+    case 0x82:
+      {
+	int ce= fetch();
+	int ch= fetch();
+	int cl= fetch();
+	PC= ce*0x10000 + ch*0x100 + cl;
+	return resGO;
+      }
+    case 0x8b: return resSTOP; // BREAK instruction
+    default:
+      cprefix = 0x00;
+      break;
+    }
 
    // exceptions
    if((cprefix==0x90)&&((code&0xf0)==0x10)) {
@@ -550,6 +1086,15 @@ cl_stm8::exec_inst(void)
                pop1( tempi);
                pop2( PC);
                PC += (tempi <<16); //Add PCE to PC
+	       {
+		 class it_level *il= (class it_level *)(it_levels->top());
+		 if (il &&
+		     il->level >= 0)
+		   {
+		     il= (class it_level *)(it_levels->pop());
+		     delete il;
+		   }
+	       }
                return(resGO);
             case 0x10: 
             case 0xA0:
@@ -560,7 +1105,7 @@ cl_stm8::exec_inst(void)
             case 0xF0: // SUB
                return( inst_sub( code, cprefix));
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -613,7 +1158,7 @@ cl_stm8::exec_inst(void)
                   case 0xEC: return(resHALT);
                   case 0xED: putchar(regs.A); fflush(stdout); return(resGO);
                   default:
-                     printf("************* bad code !!!!\n");
+		    //printf("************* bad code !!!!\n");
                      return(resINV_INST);
                }
             case 0x80: // ret
@@ -629,7 +1174,7 @@ cl_stm8::exec_inst(void)
                return( inst_cp( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -657,7 +1202,8 @@ cl_stm8::exec_inst(void)
                break;
             case 0x30: // POP longmem
                opaddr = fetch2();
-               pop1( opaddr);
+               pop1(tempi);
+               store1(opaddr, tempi);
                return(resGO);
             case 0x40: //mul
                if(cprefix==0x90) {
@@ -688,7 +1234,7 @@ cl_stm8::exec_inst(void)
                return( inst_sbc( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -703,26 +1249,11 @@ cl_stm8::exec_inst(void)
                return( inst_cpl( code, cprefix));
                break;
             case 0x80: // TRAP
-               // store to stack
-               PC++;
-               push2( PC & 0xffff);
-               push1( PC >> 16); //extended PC
-               push2( regs.Y);
-               push2( regs.X);
-               push1( regs.A);
-               push1( regs.CC);
-               // set I1 and I0 status bits
-               FLAG_SET(BIT_I1);
-               FLAG_SET(BIT_I0);
-               // get TRAP address
-               PC = get1(0x8004);
-               if (PC == 0x82) { // this is reserved opcode for vector table
-                  regs.VECTOR = 0;
-                  PC = get1(0x8005) << 16;
-                  PC |= get2(0x8006);
-                  return(resGO);
-               }
-               return(resHALT);
+	       {
+		 class it_level *il= new it_level(3, 0x8004, PC, trap_src);
+		 accept_it(il);
+	       }
+               return(/*resHALT*/resGO);
             case 0x90:
                if(cprefix==0x90) {
                   regs.Y = regs.X;
@@ -743,7 +1274,7 @@ cl_stm8::exec_inst(void)
                return( inst_cpw( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -780,7 +1311,7 @@ cl_stm8::exec_inst(void)
                return( inst_and( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -837,7 +1368,7 @@ cl_stm8::exec_inst(void)
                return( inst_bcp( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -876,7 +1407,7 @@ cl_stm8::exec_inst(void)
                return( inst_lda( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -935,7 +1466,7 @@ cl_stm8::exec_inst(void)
                return( inst_lddst( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -965,7 +1496,7 @@ cl_stm8::exec_inst(void)
                return( inst_xor( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -1002,7 +1533,7 @@ cl_stm8::exec_inst(void)
                return( inst_adc( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -1033,7 +1564,7 @@ cl_stm8::exec_inst(void)
                return( inst_or( code, cprefix));
                break;
              default:
-               printf("************* bad code !!!!\n");
+               //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -1073,7 +1604,7 @@ cl_stm8::exec_inst(void)
                return( inst_add( code, cprefix));
                break;
             default:
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -1123,7 +1654,7 @@ cl_stm8::exec_inst(void)
                return( inst_jp( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -1159,7 +1690,7 @@ cl_stm8::exec_inst(void)
                break;
             case 0xA0: // CALLR
              {
-               char c = (char) fetch1();
+               signed char c = (signed char) fetch1();
                push2(PC);
                PC += c;
                return(resGO);
@@ -1182,7 +1713,7 @@ cl_stm8::exec_inst(void)
                return( inst_call( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -1197,7 +1728,7 @@ cl_stm8::exec_inst(void)
                return( inst_swap( code, cprefix));
                break;
             case 0x80: 
-               printf("************* HALT instruction reached !!!!\n");
+	      //printf("************* HALT instruction reached !!!!\n");
                return(resHALT);
             case 0x90: // LD A, YH / XH
                if(cprefix==0x90) {
@@ -1219,7 +1750,7 @@ cl_stm8::exec_inst(void)
                return( inst_ldxy( code, cprefix));
                break;
             default:
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
@@ -1238,7 +1769,7 @@ cl_stm8::exec_inst(void)
                return( inst_clr( code, cprefix));
                break;
             case 0x80: 
-               printf("************* WFI/WFE instruction not implemented !!!!\n");
+	      //printf("************* WFI/WFE instruction not implemented !!!!\n");
                return(resINV_INST);
             case 0x90:
                if(cprefix==0x90) {
@@ -1272,26 +1803,248 @@ cl_stm8::exec_inst(void)
                return( inst_ldxydst( code, cprefix));
                break;
             default: 
-               printf("************* bad code !!!!\n");
+	      //printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
       default:
-         printf("************* bad code !!!!\n");
+	//printf("************* bad code !!!!\n");
          return(resINV_INST);
       
    }
 
-   printf("************* bad code !!!!\n");
+   //printf("************* bad code !!!!\n");
   /*if (PC)
     PC--;
   else
   PC= get_mem_size(MEM_ROM_ID)-1;*/
-  PC= rom->inc_address(PC, -1);
+  //PC= rom->inc_address(PC, -1);
 
-  sim->stop(resINV_INST);
+  //sim->stop(resINV_INST);
   return(resINV_INST);
 }
 
 
+/*
+ * Checking for interrupt requests and accept one if needed
+ */
+
+int
+cl_stm8::do_interrupt(void)
+{
+  return cl_uc::do_interrupt();
+}
+
+int
+cl_stm8::priority_of(uchar nuof_it)
+{
+  t_addr ra= 0x7f70;
+  int idx= nuof_it / 4;
+  u8_t i1_mask, i0_mask, i0, i1;
+  cl_memory_cell *c;
+  t_mem cv;
+  int levels[4]= { 2, 1, 0, 3 };
+
+  if (nuof_it > 31)
+    return 0;
+  i1_mask= 2 << ((nuof_it % 4) * 2);
+  i0_mask= 1 << ((nuof_it % 4) * 2);
+  c= ram->get_cell(ra + idx);
+  cv= c->read();
+  i0= (cv & i0_mask)?1:0;
+  i1= (cv & i1_mask)?2:0;
+  return levels[i1+i0];
+}
+
+int
+cl_stm8::priority_main(void)
+{
+  t_mem cv= regs.CC;
+  u8_t i1, i0;
+  int levels[4]= { 2, 1, 0, 3 };
+  i0= (cv & BIT_I0)?1:0;
+  i1= (cv & BIT_I1)?2:0;
+  return levels[i1+i0];
+}
+
+
+/*
+ * Accept an interrupt
+ */
+
+int
+cl_stm8::accept_it(class it_level *il)
+{
+  //class cl_it_src *is= il->source;
+  push2( PC & 0xffff);
+  push1( PC >> 16); //extended PC
+  push2( regs.Y);
+  push2( regs.X);
+  push1( regs.A);
+  push1( regs.CC);
+  // set I1 and I0 status bits
+  if (il->level == 0)
+    { FLAG_SET(BIT_I1) FLAG_CLEAR(BIT_I0) }
+  else if (il->level == 1)
+    { FLAG_CLEAR(BIT_I1) FLAG_SET(BIT_I0) }
+  else if (il->level == 2)
+    { FLAG_CLEAR(BIT_I1) FLAG_CLEAR(BIT_I0) }
+  else // 3
+    { FLAG_SET(BIT_I1) FLAG_SET(BIT_I0) }
+  PC = il->addr;
+
+  it_levels->push(il);
+  return resGO;//resINTERRUPT;
+}
+
+
+/* check if interrupts are enabled (globally)
+ */
+
+bool
+cl_stm8::it_enabled(void)
+{
+  return !(regs.CC & BIT_I0) || !(regs.CC & BIT_I1);
+}
+
+
+cl_stm8_cpu::cl_stm8_cpu(class cl_uc *auc):
+  cl_hw(auc, HW_DUMMY, 0, "cpu")
+{
+}
+
+int
+cl_stm8_cpu::init(void)
+{
+  int i;
+  cl_hw::init();
+  for (i= 0; i < 11; i++)
+    {
+      regs[i]= register_cell(uc->rom, 0x7f00+i);
+    }
+  return 0;
+}
+
+void
+cl_stm8_cpu::write(class cl_memory_cell *cell, t_mem *val)
+{
+  t_addr a;
+  cl_stm8 *u= (cl_stm8*)uc;
+
+  if (conf(cell, val))
+    return;
+  
+  if (conf(cell, NULL))
+    return;
+
+  *val&= 0xff;
+  if (!uc->rom->is_owned(cell, &a))
+    return;  
+  if ((a < 0x7f00) ||
+      (a > 0x7f0a))
+    return;
+
+  a-= 0x7f00;
+  switch (a)
+    {
+    case 0:
+      u->regs.A= *val;
+      break;
+    case 1:
+      u->PC= (u->PC & 0xffff) + (*val << 16);
+      break;
+    case 2:
+      u->PC= (u->PC & 0xff00ff) | (*val << 8);
+      break;
+    case 3:
+      u->PC= (u->PC & 0xffff00) | (*val);
+      break;
+    case 4:
+      u->regs.X= (u->regs.X & 0xff) | (*val << 8);
+      break;
+    case 5:
+      u->regs.X= (u->regs.X & 0xff00) | (*val);
+      break;
+    case 6:
+      u->regs.Y= (u->regs.Y & 0xff) | (*val << 8);
+      break;
+    case 7:
+      u->regs.Y= (u->regs.Y & 0xff00) | (*val);
+      break;
+    case 8:
+      u->regs.SP= (u->regs.SP & 0xff) | (*val << 8);
+      break;
+    case 9:
+      u->regs.SP= (u->regs.SP & 0xff00) | (*val);
+      break;
+    case 0xa:
+      u->regs.CC= (u->regs.CC & 0xff00) | (*val);
+      break;
+    }
+}
+
+t_mem
+cl_stm8_cpu::read(class cl_memory_cell *cell)
+{
+  t_mem v= cell->get();
+  t_addr a;
+  cl_stm8 *u= (cl_stm8*)uc;
+  
+  if (conf(cell, NULL))
+    return v;
+  if (!uc->rom->is_owned(cell, &a))
+    return v;
+  if ((a < 0x7f00) ||
+      (a > 0x7f0a))
+    return v;
+
+  a-= 0x7f00;
+  switch (a)
+    {
+    case 0:
+      v= u->regs.A;
+      break;
+    case 1:
+      v= (u->PC >> 16) & 0xff;
+      break;
+    case 2:
+      v= (u->PC >> 8) & 0xff;
+      break;
+    case 3:
+      v= u->PC & 0xff;
+      break;
+    case 4:
+      v= (u->regs.X >> 8) & 0xff;
+      break;
+    case 5:
+      v= u->regs.X & 0xff;
+      break;
+    case 6:
+      v= (u->regs.Y >> 8) & 0xff;
+      break;
+    case 7:
+      v= u->regs.Y & 0xff;
+      break;
+    case 8:
+      v= (u->regs.SP >> 8) & 0xff;
+      break;
+    case 9:
+      v= u->regs.SP & 0xff;
+      break;
+    case 0xa:
+      v= u->regs.CC;
+      break;
+    }
+  return v;
+}
+
+t_mem
+cl_stm8_cpu::conf_op(cl_memory_cell *cell, t_addr addr, t_mem *val)
+{
+  if (val)
+    cell->set(*val);
+  return cell->get();
+}
+
+
 /* End of stm8.src/stm8.cc */
