@@ -45,6 +45,184 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "glob.h"
 #include "regsxa.h"
 
+
+/*
+ * Names of SFRs
+ */
+
+static struct name_entry sfr_tabXA51[]=
+{
+  //#include "xa_sfr.cc"
+  {CPU_XA, 0x400, "PSW"}, /* Program status word */
+  {CPU_XA, 0x400, "PSWL"}, /* Program status word (low byte) */
+  {CPU_XA, 0x401, "PSWH"}, /* Program status word (high byte) */
+  {CPU_XA, 0x402, "PSW51"}, /* 80C51 compatible PSW */
+  {CPU_XA, 0x403, "SSEL"}, /* Segment selection register */
+  {CPU_XA, 0x404, "PCON"}, /* Power control register */
+  {CPU_XA, 0x410, "TCON"}, /* Timer 0 and 1 control register */
+  {CPU_XA, 0x411, "TSTAT"}, /* Timer 0 and 1 extended status */
+  {CPU_XA, 0x418, "T2CON"}, /* Timer 2 control register */
+  {CPU_XA, 0x419, "T2MOD"}, /* Timer 2 mode control */
+  {CPU_XA, 0x41F, "WDCON"}, /* Watchdog control register */
+  {CPU_XA, 0x420, "S0CON"}, /* Serial port 0 control register */
+  {CPU_XA, 0x421, "S0STAT"}, /* Serial port 0 extended status */
+  {CPU_XA, 0x424, "S1CON"}, /* Serial port 1 control register */
+  {CPU_XA, 0x425, "S1STAT"}, /* Serial port 1 extended status */
+  {CPU_XA, 0x426, "IEL"}, /* Interrupt enable low byte */
+  {CPU_XA, 0x427, "IEH"}, /* Interrupt enable high byte */
+  {CPU_XA, 0x42A, "SWR"}, /* Software Interrupt Request */
+  {CPU_XA, 0x430, "P0"}, /* Port 0 */
+  {CPU_XA, 0x431, "P1"}, /* Port 1 */
+  {CPU_XA, 0x432, "P2"}, /* Port 2 */
+  {CPU_XA, 0x433, "P3"}, /* Port3 */
+  {CPU_XA, 0x440, "SCR"}, /* System configuration register */
+  {CPU_XA, 0x441, "DS"}, /* Data segment */
+  {CPU_XA, 0x442, "ES"}, /* Extra segment */
+  {CPU_XA, 0x443, "CS"}, /* Code segment */
+  {CPU_XA, 0x450, "TL0"}, /* Timer 0 low byte */
+  {CPU_XA, 0x451, "TH0"}, /* Timer 0 high byte */
+  {CPU_XA, 0x452, "TL1"}, /* Timer 1 low byte */
+  {CPU_XA, 0x453, "TH1"}, /* Timer 1 high byte */
+  {CPU_XA, 0x454, "RTL0"}, /* Timer 0 extended reload, low byte */
+  {CPU_XA, 0x455, "RTH0"}, /* Timer 0 extended reload, high byte */
+  {CPU_XA, 0x456, "RTL1"}, /* Timer 1 extended reload, low byte */
+  {CPU_XA, 0x457, "RTH1"}, /* Timer 1 extended reload, high byte */
+  {CPU_XA, 0x458, "TL2"}, /* Timer 2 low byte */
+  {CPU_XA, 0x459, "TH2"}, /* Timer 2 high byte */
+  {CPU_XA, 0x45A, "T2CAPL"}, /* Timer 2 capture register, low byte */
+  {CPU_XA, 0x45B, "T2CAPH"}, /* Timer 2 capture register, high byte */
+  {CPU_XA, 0x45C, "TMOD"}, /* Timer 0 and 1 mode register */
+  {CPU_XA, 0x45D, "WFEED1"}, /* Watchdog feed 1 */
+  {CPU_XA, 0x45E, "WFEED2"}, /* Watchdog feed 2 */
+  {CPU_XA, 0x45F, "WDL"}, /* Watchdog timer reload */
+  {CPU_XA, 0x460, "S0BUF"}, /* Serial port 0 buffer register */
+  {CPU_XA, 0x461, "S0ADDR"}, /* Serial port 0 address register */
+  {CPU_XA, 0x462, "S0ADEN"}, /* Serial port 0 address enable register */
+  {CPU_XA, 0x464, "S1BUF"}, /* Serial port 1 buffer register */
+  {CPU_XA, 0x465, "S1ADDR"}, /* Serial port 1 address register */
+  {CPU_XA, 0x466, "S1ADEN"}, /* Serial port 1 address enable register */
+  {CPU_XA, 0x468, "BTRL"}, /* Bus timing register high byte */
+  {CPU_XA, 0x469, "BTRH"}, /* Bus timing register low byte */
+  {CPU_XA, 0x46A, "BCR"}, /* Bus configuration register */
+  {CPU_XA, 0x470, "P0CFGA"}, /* Port 0 configuration A */
+  {CPU_XA, 0x471, "P1CFGA"}, /* Port 1 configuration A */
+  {CPU_XA, 0x472, "P2CFGA"}, /* Port 2 configuration A */
+  {CPU_XA, 0x473, "P3CFGA"}, /* Port 3 configuration A */
+  {CPU_XA, 0x47A, "SWE"}, /* Software Interrupt Enable */
+  {CPU_XA, 0x4A0, "IPA0"}, /* Interrupt priority 0 */
+  {CPU_XA, 0x4A1, "IPA1"}, /* Interrupt priority 1 */
+  {CPU_XA, 0x4A2, "IPA2"}, /* Interrupt priority 2 */
+  {CPU_XA, 0x4A4, "IPA4"}, /* Interrupt priority 4 */
+  {CPU_XA, 0x4A5, "IPA5"}, /* Interrupt priority 5 */
+  {CPU_XA, 0x4F0, "P0CFGB"}, /* Port 0 configuration B */
+  {CPU_XA, 0x4F1, "P1CFGB"}, /* Port 1 configuration B */
+  {CPU_XA, 0x4F2, "P2CFGB"}, /* Port 2 configuration B */
+  {CPU_XA, 0x4F3, "P3CFGB"}, /* Port 3 configuration B */
+  { 0, 0, NULL }
+};
+
+/*
+ * Names of SBITs
+ */
+
+static struct name_entry bit_tabXA51[]=
+{
+  //#include "xa_bit.cc"
+  {CPU_XA, 0x33B, "ETI1"}, /* TX interrupt enable 1 */
+  {CPU_XA, 0x33A, "ERI1"}, /* RX interrupt enable 1 */
+  {CPU_XA, 0x339, "ETI0"}, /* TX interrupt enable 0 */
+  {CPU_XA, 0x338, "ERI0"}, /* RX interrupt enable 0 */
+  {CPU_XA, 0x337, "EA"}, /* global int. enable */
+  {CPU_XA, 0x334, "ET2"}, /* timer 2 interrupt */
+  {CPU_XA, 0x333, "ET1"}, /* timer 1 interrupt */
+  {CPU_XA, 0x332, "EX1"}, /* external interrupt 1 */
+  {CPU_XA, 0x331, "ET0"}, /* timer 0 interrupt */
+  {CPU_XA, 0x330, "EX0"}, /* external interrupt 0 */
+  {CPU_XA, 0x221, "PD"}, /* power down */
+  {CPU_XA, 0x220, "IDL"},
+  {CPU_XA, 0x20F, "SM"},
+  {CPU_XA, 0x20E, "TM"},
+  {CPU_XA, 0x20D, "RS1"},
+  {CPU_XA, 0x20C, "RS0"},
+  {CPU_XA, 0x20B, "IM3"},
+  {CPU_XA, 0x20A, "IM2"},
+  {CPU_XA, 0x209, "IM1"},
+  {CPU_XA, 0x208, "IM0"},
+  {CPU_XA, 0x307, "S0M0"},
+  {CPU_XA, 0x306, "S0M1"},
+  {CPU_XA, 0x305, "S0M2"},
+  {CPU_XA, 0x304, "R0EN"},
+  {CPU_XA, 0x303, "T0B8"},
+  {CPU_XA, 0x302, "R0B8"},
+  {CPU_XA, 0x301, "TI0"}, /* serial port 0 tx ready */
+  {CPU_XA, 0x300, "RI0"}, /* serial port 0 rx ready */
+  {CPU_XA, 0x30B, "FE0"},
+  {CPU_XA, 0x30A, "BR0"},
+  {CPU_XA, 0x309, "OE0"},
+  {CPU_XA, 0x308, "STINT0"},
+  {CPU_XA, 0x327, "S1M0"},
+  {CPU_XA, 0x326, "S1M1"},
+  {CPU_XA, 0x325, "S1M2"},
+  {CPU_XA, 0x324, "R1EN"},
+  {CPU_XA, 0x323, "T1B8"},
+  {CPU_XA, 0x322, "R1B8"},
+  {CPU_XA, 0x321, "TI1"}, /* serial port 0 tx ready */
+  {CPU_XA, 0x320, "RI1"}, /* serial port 0 rx ready */
+  {CPU_XA, 0x32B, "FE1"},
+  {CPU_XA, 0x32A, "BR1"},
+  {CPU_XA, 0x329, "OE1"},
+  {CPU_XA, 0x328, "STINT1"},
+  {CPU_XA, 0x356, "SWR7"},
+  {CPU_XA, 0x355, "SWR6"},
+  {CPU_XA, 0x354, "SWR5"},
+  {CPU_XA, 0x353, "SWR4"},
+  {CPU_XA, 0x352, "SWR3"},
+  {CPU_XA, 0x351, "SWR2"},
+  {CPU_XA, 0x350, "SWR1"},
+  {CPU_XA, 0x2C7, "TF2"},
+  {CPU_XA, 0x2C6, "EXF2"},
+  {CPU_XA, 0x2C5, "RCLK0"},
+  {CPU_XA, 0x2C4, "TCLK0"},
+  {CPU_XA, 0x2CD, "RCLK1"},
+  {CPU_XA, 0x2CC, "TCLK1"},
+  {CPU_XA, 0x2C3, "EXEN2"},
+  {CPU_XA, 0x2C2, "TR2"},
+  {CPU_XA, 0x2C1, "CT2"},
+  {CPU_XA, 0x2C0, "CPRL2"},
+  {CPU_XA, 0x2C9, "T2OE"},
+  {CPU_XA, 0x2C8, "DCEN"},
+  {CPU_XA, 0x287, "TF1"},
+  {CPU_XA, 0x286, "TR1"},
+  {CPU_XA, 0x285, "TF0"},
+  {CPU_XA, 0x284, "TR0"},
+  {CPU_XA, 0x283, "IE1"},
+  {CPU_XA, 0x282, "IT1"},
+  {CPU_XA, 0x281, "IE0"},
+  {CPU_XA, 0x280, "IT0"},
+  {CPU_XA, 0x28A, "T1OE"},
+  {CPU_XA, 0x288, "T0OE"},
+  {CPU_XA, 0x2FF, "PRE2"},
+  {CPU_XA, 0x2FE, "PRE1"},
+  {CPU_XA, 0x2FD, "PRE0"},
+  {CPU_XA, 0x2FA, "WDRUN"},
+  {CPU_XA, 0x2F9, "WDTOF"},
+  {CPU_XA, 0x2F8, "WDMOD"},
+  {CPU_XA, 0x388, "WR1"},
+  {CPU_XA, 0x38F, "T2EX"},
+  {CPU_XA, 0x38C, "RXD1"},
+  {CPU_XA, 0x38D, "TXD1"},
+  {CPU_XA, 0x398, "RXD0"},
+  {CPU_XA, 0x399, "TXD0"},
+  {CPU_XA, 0x39A, "INT0"},
+  {CPU_XA, 0x39B, "INT1"},
+  {CPU_XA, 0x39C, "T0"},
+  {CPU_XA, 0x39D, "T1"},
+  {CPU_XA, 0x39E, "WR"},
+  {CPU_XA, 0x39F, "RD"},
+  { 0, 0, NULL }
+};
+
+
 /*
  * Base type of xa controllers
  */
@@ -52,15 +230,14 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 cl_xa::cl_xa(class cl_sim *asim):
   cl_uc(asim)
 {
-  type= CPU_XA;
+  type= (struct cpu_entry *)malloc(sizeof(struct cpu_entry));
+  type->type= CPU_XA;
 }
 
 int
 cl_xa::init(void)
 {
   cl_uc::init(); /* Memories now exist */
-  ram= address_space(MEM_XRAM_ID);
-  rom= address_space(MEM_ROM_ID);
 
   /* set SCR to osc/4, native XA mode, flat 24 */
   set_scr(0);
@@ -73,53 +250,58 @@ cl_xa::init(void)
 
   printf("The XA Simulator is in development, UNSTABLE, DEVELOPERS ONLY!\n");
 
+  int i;
+  for (i= 0; sfr_tabXA51[i].name != NULL; i++)
+    {
+      if (type->type & sfr_tabXA51[i].cpu_type)
+	{
+	  class cl_var *v;
+	  vars->add(v= new cl_var(chars(sfr_tabXA51[i].name),
+				  sfr,
+				  sfr_tabXA51[i].addr, ""));
+	  v->init();
+	}
+    }
+  for (i= 0; bit_tabXA51[i].name != NULL; i++)
+    {
+      if (type->type & bit_tabXA51[i].cpu_type)
+	{
+	  class cl_var *v;
+	  t_addr a= bit_tabXA51[i].addr;
+	  int bitnr, offset= 0;
+	  if (a >= 0x200)
+	    {
+	      a-= 0x200;
+	      offset= 0x400;
+	    }
+	  bitnr= a%8;
+	  a= offset + a/8;
+	  vars->add(v= new cl_var(chars(bit_tabXA51[i].name),
+				  sfr,
+				  a, "", bitnr));
+	  v->init();
+	}
+    }
   return(0);
 }
 
-/*
-class cl_m *
-cl_xa::mk_mem(enum mem_class type, char *class_name)
-{
-  class cl_m *m= cl_uc::mk_mem(type, class_name);
-  if (type == MEM_SFR)
-    sfr= m;
-  if (type == MEM_IRAM)
-    iram= m;
-  return(m);
-}
-*/
 
 char *
 cl_xa::id_string(void)
 {
-  return("unspecified XA");
+  return((char*)"unspecified XA");
 }
 
 
 /*
  * Making elements of the controller
  */
-/*
-t_addr
-cl_xa::get_mem_size(enum mem_class type)
-{
-  switch(type)
-    {
-    case MEM_IRAM: return(0x2000);
-    case MEM_SFR:  return(0x2000);
-    case MEM_ROM:  return(0x10000);
-    case MEM_XRAM: return(0x10000);
-    default: return(0);
-    }
- return(cl_uc::get_mem_size(type));
-}
-*/
 
 void
 cl_xa::mk_hw_elements(void)
 {
   //class cl_base *o;
-  /* t_uc::mk_hw() does nothing */
+  cl_uc::mk_hw_elements();
 }
 
 void
@@ -228,7 +410,7 @@ cl_xa::longest_inst(void)
 
 static char dir_name[64];
 char *cl_xa::get_dir_name(short addr) {
-  if (!get_name(addr, sfr_tbl(), dir_name)) {
+  if (!/*get*/addr_name(addr, sfr/*_tbl()*/, dir_name)) {
     sprintf (dir_name, "0x%03x", addr);
   }
   return dir_name;
@@ -236,7 +418,10 @@ char *cl_xa::get_dir_name(short addr) {
 
 static char bit_name[64];
 char *cl_xa::get_bit_name(short addr) {
-  if (!get_name(addr, bit_tbl(), bit_name)) {
+  t_addr a= addr; int offset= 0, bitnr= addr%8;
+  if (a >= 0x200) { a-= 0x200; offset= 0x400; }
+  a= offset+a/8;
+  if (!/*get*/addr_name(a/*ddr*/, sfr/*bit_tbl()*/, bitnr, bit_name)) {
     sprintf (bit_name, "0x%03x", addr);
   }
   return bit_name;
@@ -272,19 +457,19 @@ cl_xa::get_disasm_info(t_addr addr,
                        int *mnemonic)
 {
   uint code;
-  int len = 0;
+  //int len = 0;
   int immed_n = 0;
   int i;
   int start_addr = addr;
 
-  code= get_mem(MEM_ROM_ID, addr++);
+  code= rom->get(addr++);
   if (code == 0x00) {
     i= 0;
     while (disass_xa[i].mnemonic != NOP)
       i++;
   } else {
-    len = 2;
-    code = (code << 8) | get_mem(MEM_ROM_ID, addr++);
+    //len = 2;
+    code = (code << 8) | rom->get(addr++);
     i= 0;
     while ((code & disass_xa[i].mask) != disass_xa[i].code &&
            disass_xa[i].mnemonic != BAD_OPCODE)
@@ -310,7 +495,7 @@ cl_xa::get_disasm_info(t_addr addr,
   return code;
 }
 
-static char *w_reg_strs[] = {
+static const char *w_reg_strs[] = {
  "R0", "R1",
  "R2", "R3",
  "R4", "R5",
@@ -320,7 +505,7 @@ static char *w_reg_strs[] = {
  "R12", "R13",
  "R14", "R15"};
 
-static char *b_reg_strs[] = {
+static const char *b_reg_strs[] = {
  "R0l", "R0h",
  "R1l", "R1h",
  "R2l", "R2h",
@@ -336,7 +521,7 @@ disass - Disassemble an opcode.
     sep - optionally points to string(tab) to use as separator.
 |--------------------------------------------------------------------*/
 char *
-cl_xa::disass(t_addr addr, char *sep)
+cl_xa::disass(t_addr addr, const char *sep)
 {
   char work[256], parm_str[40];
   char *buf, *p, *b;
@@ -345,7 +530,7 @@ cl_xa::disass(t_addr addr, char *sep)
   int immed_offset = 0;
   int operands;
   int mnemonic;
-  char **reg_strs;
+  const char **reg_strs;
 
   p= work;
 
@@ -383,13 +568,13 @@ cl_xa::disass(t_addr addr, char *sep)
       sprintf(parm_str, "%s,[%s+%02x]",
               reg_strs[((code >> 4) & 0xf)],
               w_reg_strs[(code & 0x7)],
-              get_mem(MEM_ROM_ID, addr+immed_offset));
+              rom->get(addr+immed_offset));
       ++immed_offset;
     break;
     case IREGOFF8_REG :
       sprintf(parm_str, "[%s+%02x],%s",
               w_reg_strs[(code & 0x7)],
-              get_mem(MEM_ROM_ID, addr+immed_offset),
+              rom->get(addr+immed_offset),
               reg_strs[((code >> 4) & 0xf)] );
       ++immed_offset;
     break;
@@ -397,16 +582,16 @@ cl_xa::disass(t_addr addr, char *sep)
       sprintf(parm_str, "%s,[%s+%04x]",
               reg_strs[((code >> 4) & 0xf)],
               w_reg_strs[(code & 0x7)],
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+1)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset)<<8)) );
+              (short)((rom->get(addr+immed_offset+1)) |
+                     (rom->get(addr+immed_offset)<<8)) );
       ++immed_offset;
       ++immed_offset;
     break;
     case IREGOFF16_REG :
       sprintf(parm_str, "[%s+%04x],%s",
               w_reg_strs[(code & 0x7)],
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+1)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset)<<8)),
+              (short)((rom->get(addr+immed_offset+1)) |
+                     (rom->get(addr+immed_offset)<<8)),
               reg_strs[((code >> 4) & 0xf)] );
       ++immed_offset;
       ++immed_offset;
@@ -423,105 +608,105 @@ cl_xa::disass(t_addr addr, char *sep)
     break;
     case DIRECT_REG :
       sprintf(parm_str, "%s,%s",
-              get_dir_name(((code & 0x7) << 8) | 
-                           get_mem(MEM_ROM_ID, addr+immed_offset)),
+              get_dir_name(((code & 0x7) << 8) |
+                           rom->get(addr+immed_offset)),
               reg_strs[((code >> 4) & 0xf)] );
       ++immed_offset;
     break;
     case REG_DIRECT :
       sprintf(parm_str, "%s,%s",
-              reg_strs[((code >> 4) & 0xf)], 
-              get_dir_name(((code & 0x7) << 8) | 
-                           get_mem(MEM_ROM_ID, addr+immed_offset)));
+              reg_strs[((code >> 4) & 0xf)],
+              get_dir_name(((code & 0x7) << 8) |
+                           rom->get(addr+immed_offset)));
       ++immed_offset;
     break;
     case REG_DATA8 :
       sprintf(parm_str, "%s,#0x%02x",
               b_reg_strs[((code >> 4) & 0xf)],
-              get_mem(MEM_ROM_ID, addr+immed_offset) );
+              rom->get(addr+immed_offset) );
       ++immed_offset;
     break;
     case REG_DATA16 :
       sprintf(parm_str, "%s,#0x%04x",
               reg_strs[((code >> 4) & 0xf)],
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+1)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset)<<8)) );
+              (short)((rom->get(addr+immed_offset+1)) |
+                     (rom->get(addr+immed_offset)<<8)) );
       ++immed_offset;
       ++immed_offset;
     break;
     case IREG_DATA8 :
       sprintf(parm_str, "[%s], 0x%02x",
               w_reg_strs[((code >> 4) & 0x7)],
-              get_mem(MEM_ROM_ID, addr+immed_offset) );
+              rom->get(addr+immed_offset) );
       ++immed_offset;
     break;
     case IREG_DATA16 :
       sprintf(parm_str, "[%s], 0x%04x",
               w_reg_strs[((code >> 4) & 0x7)],
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+1)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset)<<8)) );
+              (short)((rom->get(addr+immed_offset+1)) |
+                     (rom->get(addr+immed_offset)<<8)) );
       ++immed_offset;
       ++immed_offset;
     break;
     case IREGINC_DATA8 :
       sprintf(parm_str, "[%s+], 0x%02x",
               w_reg_strs[((code >> 4) & 0x7)],
-              get_mem(MEM_ROM_ID, addr+immed_offset) );
+              rom->get(addr+immed_offset) );
       ++immed_offset;
     break;
     case IREGINC_DATA16 :
       sprintf(parm_str, "[%s+], 0x%04x",
               w_reg_strs[((code >> 4) & 0x7)],
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+1)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset)<<8)) );
+              (short)((rom->get(addr+immed_offset+1)) |
+                     (rom->get(addr+immed_offset)<<8)) );
       ++immed_offset;
       ++immed_offset;
     break;
     case IREGOFF8_DATA8 :
       sprintf(parm_str, "[%s+%02x], 0x%02x",
               w_reg_strs[((code >> 4) & 0x7)],
-              get_mem(MEM_ROM_ID, addr+immed_offset),
-              get_mem(MEM_ROM_ID, addr+immed_offset+1) );
+              rom->get(addr+immed_offset),
+              rom->get(addr+immed_offset+1) );
       immed_offset += 2;
     break;
     case IREGOFF8_DATA16 :
       sprintf(parm_str, "[%s+%02x], 0x%04x",
               w_reg_strs[((code >> 4) & 0x7)],
-              get_mem(MEM_ROM_ID, addr+immed_offset),
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+2)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8)) );
+              rom->get(addr+immed_offset),
+              (short)((rom->get(addr+immed_offset+2)) |
+                     (rom->get(addr+immed_offset+1)<<8)) );
       immed_offset += 3;
     break;
     case IREGOFF16_DATA8 :
       sprintf(parm_str, "[%s+%04x], 0x%02x",
               w_reg_strs[((code >> 4) & 0x7)],
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+1)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset+0)<<8)),
-              get_mem(MEM_ROM_ID, addr+immed_offset+2) );
+              (short)((rom->get(addr+immed_offset+1)) |
+                     (rom->get(addr+immed_offset+0)<<8)),
+              rom->get(addr+immed_offset+2) );
       immed_offset += 3;
     break;
     case IREGOFF16_DATA16 :
       sprintf(parm_str, "[%s+%04x], 0x%04x",
               w_reg_strs[((code >> 4) & 0x7)],
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+1)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset+0)<<8)),
-              (short)((get_mem(MEM_ROM_ID, addr+immed_offset+3)) |
-                     (get_mem(MEM_ROM_ID, addr+immed_offset+2)<<8)) );
+              (short)((rom->get(addr+immed_offset+1)) |
+                     (rom->get(addr+immed_offset+0)<<8)),
+              (short)((rom->get(addr+immed_offset+3)) |
+                     (rom->get(addr+immed_offset+2)<<8)) );
       immed_offset += 4;
     break;
     case DIRECT_DATA8 :
       sprintf(parm_str, "%s,#0x%02x",
-              get_dir_name(((code & 0x0070) << 4) | 
-                           get_mem(MEM_ROM_ID, addr+immed_offset)),
-              get_mem(MEM_ROM_ID, addr+immed_offset+1));
+              get_dir_name(((code & 0x0070) << 4) |
+                           rom->get(addr+immed_offset)),
+              rom->get(addr+immed_offset+1));
       immed_offset += 3;
     break;
     case DIRECT_DATA16 :
       sprintf(parm_str, "%s,#0x%04x",
-              get_dir_name(((code & 0x0070) << 4) | 
-                           get_mem(MEM_ROM_ID, addr+immed_offset)),
-              get_mem(MEM_ROM_ID, addr+immed_offset+2) +
-              (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8));
+              get_dir_name(((code & 0x0070) << 4) |
+                           rom->get(addr+immed_offset)),
+              rom->get(addr+immed_offset+2) +
+              (rom->get(addr+immed_offset+1)<<8));
       immed_offset += 3;
     break;
 
@@ -530,12 +715,12 @@ cl_xa::disass(t_addr addr, char *sep)
       strcpy(parm_str, "");
     break;
     case CY_BIT :
-      sprintf(parm_str, "C,%s", 
-             get_bit_name(((code&0x0003)<<8) + get_mem(MEM_ROM_ID, addr+2)));
+      sprintf(parm_str, "C,%s",
+             get_bit_name(((code&0x0003)<<8) + rom->get(addr+2)));
     break;
     case BIT_CY :
-      sprintf(parm_str, "%s,C", 
-              get_bit_name(((code&0x0003)<<8) + get_mem(MEM_ROM_ID, addr+2)));
+      sprintf(parm_str, "%s,C",
+              get_bit_name(((code&0x0003)<<8) + rom->get(addr+2)));
     break;
     case REG_DATA4 :
       strcpy(parm_str, "REG_DATA4");
@@ -557,14 +742,14 @@ cl_xa::disass(t_addr addr, char *sep)
     break;
     case DIRECT_DATA4 :
       sprintf(parm_str, "%s,#0x%x",
-              get_dir_name(((code & 0x70)<<4) |
-                           get_mem(MEM_ROM_ID, addr+2)),
-              code&0x0f);
+	      get_dir_name(((code & 0x70)<<4) |
+			   rom->get(addr+2)),
+	      code&0x0f);
     break;
     case DIRECT :
       sprintf(parm_str, "%s",
-              get_dir_name(((code & 0x007) << 4) + 
-                           get_mem(MEM_ROM_ID, addr+2)));
+              get_dir_name(((code & 0x007) << 4) +
+                           rom->get(addr+2)));
     break;
     case REG :
       sprintf(parm_str, "%s",
@@ -576,33 +761,33 @@ cl_xa::disass(t_addr addr, char *sep)
     break;
     case BIT_ALONE :
       sprintf(parm_str, "%s",
-              get_bit_name(((code&0x0003)<<8) + get_mem(MEM_ROM_ID, addr+2)));
+	      get_bit_name(((code&0x0003)<<8) + rom->get(addr+2)));
     break;
     case BIT_REL8 :
-      sprintf(parm_str, "%s,0x%04x",
-              get_bit_name((code&0x0003)<<8) + get_mem(MEM_ROM_ID, addr+2),
-              ((signed char)get_mem(MEM_ROM_ID, addr+3)*2+addr+len)&0xfffe);
+      sprintf(parm_str, "%s,0x%04lx",
+	      get_bit_name((code&0x0003)<<8) + rom->get(addr+2),
+	      (long)(((signed char)rom->get(addr+3)*2+addr+len)&0xfffe));
     break;
     case DATA4:
       sprintf(parm_str, "#0x%02x", code&0x0f);
       break;
     case ADDR24 :
-      sprintf(parm_str, "0x%06x", 
-             (get_mem(MEM_ROM_ID, addr+3)<<16) + 
-             (get_mem(MEM_ROM_ID, addr+1)<<8) + 
-             get_mem(MEM_ROM_ID, addr+2));
+      sprintf(parm_str, "0x%06x",
+             (rom->get(addr+3)<<16) +
+             (rom->get(addr+1)<<8) +
+             rom->get(addr+2));
       break;
     break;
     case REG_REL8 :
-      sprintf(parm_str, "%s,0x%04x",
-              reg_strs[(code>>4) & 0xf],
-              ((signed char)get_mem(MEM_ROM_ID, addr+2)*2+addr+len)&0xfffe);
+      sprintf(parm_str, "%s,0x%04lx",
+	      reg_strs[(code>>4) & 0xf],
+	      (long)(((signed char)rom->get(addr+2)*2+addr+len)&0xfffe));
     break;
     case DIRECT_REL8 :
-      sprintf(parm_str, "%s,0x%04x",
-              get_dir_name(((code&0x07)<<8) +
-                           get_mem(MEM_ROM_ID, addr+2)),
-              ((signed char)get_mem(MEM_ROM_ID, addr+2)*2+addr+len)&0xfffe);
+      sprintf(parm_str, "%s,0x%04lx",
+	      get_dir_name(((code&0x07)<<8) +
+			   rom->get(addr+2)),
+	      (long)(((signed char)rom->get(addr+2)*2+addr+len)&0xfffe));
     break;
     case REG_USP:
       sprintf(parm_str, "REG_USP");
@@ -611,59 +796,59 @@ cl_xa::disass(t_addr addr, char *sep)
       sprintf(parm_str, "USP_REG");
     break;
     case REL8 :
-      sprintf(parm_str, "0x%04x",
-              ((signed char)get_mem(MEM_ROM_ID, addr+1)*2+addr+len)&0xfffe);
+      sprintf(parm_str, "0x%04lx",
+	      (long)(((signed char)rom->get(addr+1)*2+addr+len)&0xfffe));
     break;
     case REL16 :
-      sprintf(parm_str, "0x%04x",
-              ((signed short)((get_mem(MEM_ROM_ID, addr+1)<<8) + get_mem(MEM_ROM_ID, addr+2))*2+addr+len)&0xfffe);
+      sprintf(parm_str, "0x%04lx",
+	      (long)(((signed short)((rom->get(addr+1)<<8) + rom->get(addr+2))*2+addr+len)&0xfffe));
     break;
 
     case RLIST : {
       /* TODO: the list should be comma reperated
-         and maybe for POP the list should be reversed */
+	 and maybe for POP the list should be reversed */
       unsigned char rlist=code&0xff;
       parm_str[0]='\0';
       if (code&0x0800) { // word list
-        if (code&0x4000) { // R8-R15
-          if (rlist&0x80) strcat (parm_str, "R15 ");
-          if (rlist&0x40) strcat (parm_str, "R14");
-          if (rlist&0x20) strcat (parm_str, "R13 ");
-          if (rlist&0x10) strcat (parm_str, "R12 ");
-          if (rlist&0x08) strcat (parm_str, "R11 ");
-          if (rlist&0x04) strcat (parm_str, "R10 ");
-          if (rlist&0x02) strcat (parm_str, "R9 ");
-          if (rlist&0x01) strcat (parm_str, "R8 ");
-        } else { // R7-R0
-          if (rlist&0x80) strcat (parm_str, "R7 ");
-          if (rlist&0x40) strcat (parm_str, "R6 ");
-          if (rlist&0x20) strcat (parm_str, "R5 ");
-          if (rlist&0x10) strcat (parm_str, "R4 ");
-          if (rlist&0x08) strcat (parm_str, "R3 ");
-          if (rlist&0x04) strcat (parm_str, "R2 ");
-          if (rlist&0x02) strcat (parm_str, "R1 ");
-          if (rlist&0x01) strcat (parm_str, "R0 ");
-        }
+	if (code&0x4000) { // R8-R15
+	  if (rlist&0x80) strcat (parm_str, "R15 ");
+	  if (rlist&0x40) strcat (parm_str, "R14");
+	  if (rlist&0x20) strcat (parm_str, "R13 ");
+	  if (rlist&0x10) strcat (parm_str, "R12 ");
+	  if (rlist&0x08) strcat (parm_str, "R11 ");
+	  if (rlist&0x04) strcat (parm_str, "R10 ");
+	  if (rlist&0x02) strcat (parm_str, "R9 ");
+	  if (rlist&0x01) strcat (parm_str, "R8 ");
+	} else { // R7-R0
+	  if (rlist&0x80) strcat (parm_str, "R7 ");
+	  if (rlist&0x40) strcat (parm_str, "R6 ");
+	  if (rlist&0x20) strcat (parm_str, "R5 ");
+	  if (rlist&0x10) strcat (parm_str, "R4 ");
+	  if (rlist&0x08) strcat (parm_str, "R3 ");
+	  if (rlist&0x04) strcat (parm_str, "R2 ");
+	  if (rlist&0x02) strcat (parm_str, "R1 ");
+	  if (rlist&0x01) strcat (parm_str, "R0 ");
+	}
       } else { // byte list
-        if (code&0x4000) { //R7h-R4l
-          if (rlist&0x80) strcat (parm_str, "R7h ");
-          if (rlist&0x40) strcat (parm_str, "R7l ");
-          if (rlist&0x20) strcat (parm_str, "R6h ");
-          if (rlist&0x10) strcat (parm_str, "R6l ");
-          if (rlist&0x08) strcat (parm_str, "R5h ");
-          if (rlist&0x04) strcat (parm_str, "R5l ");
-          if (rlist&0x02) strcat (parm_str, "R4h ");
-          if (rlist&0x01) strcat (parm_str, "R4l ");
-        } else { // R3h-R0l
-          if (rlist&0x80) strcat (parm_str, "R3h ");
-          if (rlist&0x40) strcat (parm_str, "R3l ");
-          if (rlist&0x20) strcat (parm_str, "R2h ");
-          if (rlist&0x10) strcat (parm_str, "R2l ");
-          if (rlist&0x08) strcat (parm_str, "R1h ");
-          if (rlist&0x04) strcat (parm_str, "R1l ");
-          if (rlist&0x02) strcat (parm_str, "R0h ");
-          if (rlist&0x01) strcat (parm_str, "R0l ");
-        }
+	if (code&0x4000) { //R7h-R4l
+	  if (rlist&0x80) strcat (parm_str, "R7h ");
+	  if (rlist&0x40) strcat (parm_str, "R7l ");
+	  if (rlist&0x20) strcat (parm_str, "R6h ");
+	  if (rlist&0x10) strcat (parm_str, "R6l ");
+	  if (rlist&0x08) strcat (parm_str, "R5h ");
+	  if (rlist&0x04) strcat (parm_str, "R5l ");
+	  if (rlist&0x02) strcat (parm_str, "R4h ");
+	  if (rlist&0x01) strcat (parm_str, "R4l ");
+	} else { // R3h-R0l
+	  if (rlist&0x80) strcat (parm_str, "R3h ");
+	  if (rlist&0x40) strcat (parm_str, "R3l ");
+	  if (rlist&0x20) strcat (parm_str, "R2h ");
+	  if (rlist&0x10) strcat (parm_str, "R2l ");
+	  if (rlist&0x08) strcat (parm_str, "R1h ");
+	  if (rlist&0x04) strcat (parm_str, "R1l ");
+	  if (rlist&0x02) strcat (parm_str, "R0h ");
+	  if (rlist&0x01) strcat (parm_str, "R0l ");
+	}
       }
     }
     break;
@@ -671,35 +856,35 @@ cl_xa::disass(t_addr addr, char *sep)
     case REG_DIRECT_REL8 :
       sprintf(parm_str, "%s,%s,0x%02x",
               reg_strs[((code >> 4) & 0xf)],
-              get_dir_name(((code & 0x7) << 8) + 
-                           get_mem(MEM_ROM_ID, addr+immed_offset)),
-              ((signed char) get_mem(MEM_ROM_ID, addr+immed_offset+1) * 2) & 0xfffe );
+              get_dir_name(((code & 0x7) << 8) +
+                           rom->get(addr+immed_offset)),
+              ((signed char) rom->get(addr+immed_offset+1) * 2) & 0xfffe );
     break;
     case REG_DATA8_REL8 :
       sprintf(parm_str, "%s,#0x%02x,0x%02x",
               reg_strs[((code >> 4) & 0xf)],
-              get_mem(MEM_ROM_ID, addr+immed_offset+1),
-              ((signed char)get_mem(MEM_ROM_ID, addr+immed_offset) * 2) & 0xfffe );
+              rom->get(addr+immed_offset+1),
+              ((signed char)rom->get(addr+immed_offset) * 2) & 0xfffe );
     break;
     case REG_DATA16_REL8 :
       sprintf(parm_str, "%s,#0x%04x,0x%02x",
               w_reg_strs[(code >> 4) & 0xf],
-              get_mem(MEM_ROM_ID, addr+immed_offset+2) +
-                 (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8),
-              ((signed char)get_mem(MEM_ROM_ID, addr+immed_offset) * 2) & 0xfffe );
+              rom->get(addr+immed_offset+2) +
+                 (rom->get(addr+immed_offset+1)<<8),
+              ((signed char)rom->get(addr+immed_offset) * 2) & 0xfffe );
     break;
     case IREG_DATA8_REL8 :
       sprintf(parm_str, "[%s],#0x%02x,0x%02x",
               reg_strs[((code >> 4) & 0x7)],
-              get_mem(MEM_ROM_ID, addr+immed_offset+1),
-              ((signed char)get_mem(MEM_ROM_ID, addr+immed_offset) * 2) & 0xfffe );
+              rom->get(addr+immed_offset+1),
+              ((signed char)rom->get(addr+immed_offset) * 2) & 0xfffe );
     break;
     case IREG_DATA16_REL8 :
       sprintf(parm_str, "[%s],#0x%04x,0x%02x",
               w_reg_strs[(code >> 4) & 0x7],
-              get_mem(MEM_ROM_ID, addr+immed_offset+2) +
-                (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8),
-              ((signed char)get_mem(MEM_ROM_ID, addr+immed_offset) * 2) & 0xfffe );
+              rom->get(addr+immed_offset+2) +
+	        (rom->get(addr+immed_offset+1)<<8),
+              ((signed char)rom->get(addr+immed_offset) * 2) & 0xfffe );
     break;
 
     case A_APLUSDPTR :
@@ -711,18 +896,18 @@ cl_xa::disass(t_addr addr, char *sep)
     break;
 
     case REG_REGOFF8 :
-      sprintf(parm_str, "%s,%s+0x%02x", 
+      sprintf(parm_str, "%s,%s+0x%02x",
               w_reg_strs[(code >> 4) & 0x7],
               w_reg_strs[code & 0x7],
-              get_mem(MEM_ROM_ID, addr+immed_offset));
+              rom->get(addr+immed_offset));
       break;
 
     case REG_REGOFF16 :
-      sprintf(parm_str, "%s,%s+0x%02x", 
+      sprintf(parm_str, "%s,%s+0x%02x",
               w_reg_strs[(code >> 4) & 0x7],
               w_reg_strs[code & 0x7],
-              get_mem(MEM_ROM_ID, addr+immed_offset+1) +
-                (get_mem(MEM_ROM_ID, addr+immed_offset+0)<<8));
+              rom->get(addr+immed_offset+1) +
+	        (rom->get(addr+immed_offset+0)<<8));
       break;
 
     case A_PLUSDPTR :
