@@ -60,14 +60,14 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  *  (base for Rabbit 3000/4000/5000)
  */
 
-cl_r2k::cl_r2k(int Itype, int Itech, class cl_sim *asim):
-  cl_z80(Itype, Itech, asim), mmu(this)
+cl_r2k::cl_r2k(struct cpu_entry *Itype, class cl_sim *asim):
+  cl_z80(Itype, asim), mmu(this)
 {
   type= Itype;
 }
 
-cl_r3ka::cl_r3ka(int Itype, int Itech, class cl_sim *asim):
-  cl_r2k(Itype, Itech, asim)
+cl_r3ka::cl_r3ka(struct cpu_entry *Itype, class cl_sim *asim):
+  cl_r2k(Itype, asim)
 {
   SU = 0;
 }
@@ -77,9 +77,9 @@ cl_r2k::init(void)
 {
   cl_uc::init(); /* Memories now exist */
 
-  rom= address_space(MEM_ROM_ID);
+  //rom= address_space(MEM_ROM_ID);
 //  ram= mem(MEM_XRAM);
-  ram= rom;
+  //ram= rom;
 
   // zero out ram(this is assumed in regression tests)
   for (int i=0x8000; i<0x10000; i++) {
@@ -89,16 +89,16 @@ cl_r2k::init(void)
   return(0);
 }
 
-const char *
+char *
 cl_r2k::id_string(void)
 {
-  return("rabbit 2000");
+  return((char*)"rabbit 2000");
 }
 
-const char *
+char *
 cl_r3ka::id_string(void)
 {
-  return("rabbit 3000A");
+  return((char*)"rabbit 3000A");
 }
 
 /*
@@ -122,7 +122,7 @@ void
 cl_r2k::mk_hw_elements(void)
 {
   //class cl_base *o;
-  /* t_uc::mk_hw() does nothing */
+  cl_uc::mk_hw_elements();
 }
 
 void
@@ -130,7 +130,7 @@ cl_r2k::make_memories(void)
 {
   class cl_address_space *as;
 
-  as= new cl_address_space("rom", 0, 0x10000, 8);
+  rom= ram= as= new cl_address_space("rom", 0, 0x10000, 8);
   as->init();
   address_spaces->add(as);
 
@@ -144,6 +144,108 @@ cl_r2k::make_memories(void)
   ad->init();
   as->decoders->add(ad);
   ad->activate(0);
+
+  regs8= new cl_address_space("regs8", 0, 18, 8);
+  regs8->init();
+  regs8->get_cell(0)->decode((t_mem*)&regs.raf.A);
+  regs8->get_cell(1)->decode((t_mem*)&regs.raf.F);
+  regs8->get_cell(2)->decode((t_mem*)&regs.bc.h);
+  regs8->get_cell(3)->decode((t_mem*)&regs.bc.l);
+  regs8->get_cell(4)->decode((t_mem*)&regs.de.h);
+  regs8->get_cell(5)->decode((t_mem*)&regs.de.l);
+  regs8->get_cell(6)->decode((t_mem*)&regs.hl.h);
+  regs8->get_cell(7)->decode((t_mem*)&regs.hl.l);
+  regs8->get_cell(8)->decode((t_mem*)&iir);
+  regs8->get_cell(9)->decode((t_mem*)&eir);
+
+  regs8->get_cell(9)->decode((t_mem*)&regs.ralt_af.aA);
+  regs8->get_cell(10)->decode((t_mem*)&regs.ralt_af.aF);
+  regs8->get_cell(11)->decode((t_mem*)&regs.a_bc.h);
+  regs8->get_cell(12)->decode((t_mem*)&regs.a_bc.l);
+  regs8->get_cell(13)->decode((t_mem*)&regs.a_de.h);
+  regs8->get_cell(14)->decode((t_mem*)&regs.a_de.l);
+  regs8->get_cell(15)->decode((t_mem*)&regs.a_hl.h);
+  regs8->get_cell(16)->decode((t_mem*)&regs.a_hl.l);
+
+  regs16= new cl_address_space("regs16", 0, 11, 16);
+  regs16->init();
+
+  regs16->get_cell(0)->decode((t_mem*)&regs.AF);
+  regs16->get_cell(1)->decode((t_mem*)&regs.BC);
+  regs16->get_cell(2)->decode((t_mem*)&regs.DE);
+  regs16->get_cell(3)->decode((t_mem*)&regs.HL);
+  regs16->get_cell(4)->decode((t_mem*)&regs.IX);
+  regs16->get_cell(5)->decode((t_mem*)&regs.IY);
+  regs16->get_cell(6)->decode((t_mem*)&regs.SP);
+  regs16->get_cell(7)->decode((t_mem*)&regs.aAF);
+  regs16->get_cell(8)->decode((t_mem*)&regs.aBC);
+  regs16->get_cell(9)->decode((t_mem*)&regs.aDE);
+  regs16->get_cell(10)->decode((t_mem*)&regs.aHL);
+
+  address_spaces->add(regs8);
+  address_spaces->add(regs16);
+
+  class cl_var *v;
+  vars->add(v= new cl_var(cchars("A"), regs8, 0, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("F"), regs8, 1, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("B"), regs8, 2, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("C"), regs8, 3, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("D"), regs8, 4, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("E"), regs8, 5, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("H"), regs8, 6, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("L"), regs8, 7, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("IIR"), regs8, 8, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("EIR"), regs8, 9, ""));
+  v->init();
+
+  vars->add(v= new cl_var(cchars("ALT_A"), regs8, 10, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_F"), regs8, 11, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_B"), regs8, 12, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_C"), regs8, 13, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_D"), regs8, 14, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_E"), regs8, 15, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_H"), regs8, 16, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_L"), regs8, 17, ""));
+  v->init();
+
+  vars->add(v= new cl_var(cchars("AF"), regs16, 0, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("BC"), regs16, 1, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("DE"), regs16, 2, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("HL"), regs16, 3, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("IX"), regs16, 4, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("IY"), regs16, 5, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("SP"), regs16, 6, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_AF"), regs16, 7, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_BC"), regs16, 8, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_DE"), regs16, 9, ""));
+  v->init();
+  vars->add(v= new cl_var(cchars("ALT_HL"), regs16, 10, ""));
+  v->init();
 }
 
 
@@ -211,12 +313,12 @@ cl_r2k::get_disasm_info(t_addr addr,
   int start_addr = addr;
   struct dis_entry *dis_e;
 
-  code= get_mem(MEM_ROM_ID, addr++);
+  code= rom->get/*_mem*/(/*MEM_ROM_ID,*/ addr++);
   dis_e = NULL;
 
   switch(code) {
     case 0xcb:  /* ESC code to lots of op-codes, all 2-byte */
-      code= get_mem(MEM_ROM_ID, addr++);
+      code= rom->get/*_mem*/(/*MEM_ROM_ID,*/ addr++);
       i= 0;
       while ((code & disass_r2k_cb[i].mask) != disass_r2k_cb[i].code &&
         disass_r2k_cb[i].mnemonic)
@@ -228,7 +330,7 @@ cl_r2k::get_disasm_info(t_addr addr,
     break;
 
     case 0xed: /* ESC code to about 80 opcodes of various lengths */
-      code= get_mem(MEM_ROM_ID, addr++);
+      code= rom->get/*_mem*/(/*MEM_ROM_ID,*/ addr++);
       i= 0;
       while ((code & disass_r2k_ed[i].mask) != disass_r2k_ed[i].code &&
         disass_r2k_ed[i].mnemonic)
@@ -240,11 +342,11 @@ cl_r2k::get_disasm_info(t_addr addr,
     break;
 
     case 0xdd: /* ESC codes,about 284, vary lengths, IX centric */
-      code= get_mem(MEM_ROM_ID, addr++);
+      code= rom->get(addr++);
       if (code == 0xcb) {
         immed_n = 2;
         addr++;  // pass up immed data
-        code= get_mem(MEM_ROM_ID, addr++);
+        code= rom->get(addr++);
         i= 0;
         while ((code & disass_r2k_ddcb[i].mask) != disass_r2k_ddcb[i].code &&
           disass_r2k_ddcb[i].mnemonic)
@@ -266,11 +368,11 @@ cl_r2k::get_disasm_info(t_addr addr,
     break;
 
     case 0xfd: /* ESC codes,sme as dd but IY centric */
-      code= get_mem(MEM_ROM_ID, addr++);
+      code= rom->get(addr++);
       if (code == 0xcb) {
         immed_n = 2;
         addr++;  // pass up immed data
-        code= get_mem(MEM_ROM_ID, addr++);
+        code= rom->get(addr++);
         i= 0;
         while ((code & disass_r2k_fdcb[i].mask) != disass_r2k_fdcb[i].code &&
           disass_r2k_fdcb[i].mnemonic)
@@ -323,7 +425,7 @@ cl_r2k::get_disasm_info(t_addr addr,
   return b;
 }
 
-const char *
+char *
 cl_r2k::disass(t_addr addr, const char *sep)
 {
   char work[256], temp[20];
@@ -350,18 +452,18 @@ cl_r2k::disass(t_addr addr, const char *sep)
           switch (*(b++))
             {
             case 'd': // d    jump relative target, signed? byte immediate operand
-              sprintf(temp, "#%d", (char)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "#%d", (char)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             case 'w': // w    word immediate operand
               sprintf(temp, "#0x%04x",
-                 (uint)((get_mem(MEM_ROM_ID, addr+immed_offset)) |
-                        (get_mem(MEM_ROM_ID, addr+immed_offset+1)<<8)) );
+                 (uint)((rom->get(addr+immed_offset)) |
+                        (rom->get(addr+immed_offset+1)<<8)) );
               ++immed_offset;
               ++immed_offset;
               break;
             case 'b': // b    byte immediate operand
-              sprintf(temp, "#0x%02x", (uint)get_mem(MEM_ROM_ID, addr+immed_offset));
+              sprintf(temp, "#0x%02x", (uint)rom->get(addr+immed_offset));
               ++immed_offset;
               break;
             default:
@@ -407,16 +509,16 @@ void
 cl_r2k::print_regs(class cl_console_base *con)
 {
   con->dd_printf("SZ-A-PNC  Flags= 0x%02x %3d %c  ",
-                 regs.F, regs.F, isprint(regs.F)?regs.F:'.');
+                 regs.raf.F, regs.raf.F, isprint(regs.raf.F)?regs.raf.F:'.');
   con->dd_printf("A= 0x%02x %3d %c\n",
-                 regs.A, regs.A, isprint(regs.A)?regs.A:'.');
+                 regs.raf.A, regs.raf.A, isprint(regs.raf.A)?regs.raf.A:'.');
   con->dd_printf("%c%c-%c-%c%c%c\n",
-                 (regs.F&BIT_S)?'1':'0',
-                 (regs.F&BIT_Z)?'1':'0',
-                 (regs.F&BIT_A)?'1':'0',
-                 (regs.F&BIT_P)?'1':'0',
-                 (regs.F&BIT_N)?'1':'0',
-                 (regs.F&BIT_C)?'1':'0');
+                 (regs.raf.F&BIT_S)?'1':'0',
+                 (regs.raf.F&BIT_Z)?'1':'0',
+                 (regs.raf.F&BIT_A)?'1':'0',
+                 (regs.raf.F&BIT_P)?'1':'0',
+                 (regs.raf.F&BIT_N)?'1':'0',
+                 (regs.raf.F&BIT_C)?'1':'0');
   con->dd_printf("BC= 0x%04x [BC]= %02x %3d %c  ",
                  regs.BC, ram->get(regs.BC), ram->get(regs.BC),
                  isprint(ram->get(regs.BC))?ram->get(regs.BC):'.');
