@@ -54,7 +54,7 @@ struct
   { E_SYNTAX_ERROR, ERROR_LEVEL_ERROR,
      "Syntax error, declaration ignored at '%s'", 0 },
   { E_CONST_EXPECTED, ERROR_LEVEL_ERROR,
-      "Initializer element is not constant", 0 },
+      "Initializer element is not a constant expression", 0 },
   { E_OUT_OF_MEM, ERROR_LEVEL_ERROR,
      "'malloc' failed file '%s' for size %ld", 0 },
   { E_FILE_OPEN_ERR, ERROR_LEVEL_ERROR,
@@ -112,9 +112,9 @@ struct
   { E_CONV_ERR, ERROR_LEVEL_ERROR,
      "conversion error: integral promotion failed", 0 },
   { E_INT_REQD, ERROR_LEVEL_ERROR,
-     "type must be INT for bit field definition", 0 },
+     "type must be INT for bit-field definition", 0 },
   { E_BITFLD_SIZE, ERROR_LEVEL_ERROR,
-     "bit field size cannot be greater than int (%d bits)", 0 },
+     "bit-field size cannot be greater than int (%d bits)", 0 },
   { W_TRUNCATION, ERROR_LEVEL_WARNING,
      "high order truncation might occur", 0 },
   { E_CODE_WRITE, ERROR_LEVEL_ERROR,
@@ -140,7 +140,7 @@ struct
   { E_SHIFT_OP_INVALID, ERROR_LEVEL_ERROR,
      "invalid operand for shift operator", 0 },
   { E_COMPARE_OP, ERROR_LEVEL_ERROR,
-     "compare operand cannot be struct/union", 0 },
+     "operands are not comparable", 0 },
   { E_BITWISE_OP, ERROR_LEVEL_ERROR,
      "operand invalid for bitwise operation", 0 },
   { E_ANDOR_OP, ERROR_LEVEL_ERROR,
@@ -247,8 +247,8 @@ struct
      "Pre-Processor %s", 0 },
   { E_STRUCT_AS_ARG, ERROR_LEVEL_ERROR,
      "SDCC cannot pass structure '%s' as function argument", 0 },
-  { E_PREV_DEF_CONFLICT, ERROR_LEVEL_ERROR,
-     "conflict with previous definition of '%s' for attribute '%s'", 0 },
+  { E_PREV_DECL_CONFLICT, ERROR_LEVEL_ERROR,
+     "conflict with previous declaration of '%s' for attribute '%s' at %s:%d", 0 },
   { E_CODE_NO_INIT, ERROR_LEVEL_WARNING,
      "variable '%s' declared in code space must have initialiser", 0 },
   { E_OPS_INTEGRAL, ERROR_LEVEL_ERROR,
@@ -289,13 +289,13 @@ struct
      "unknown compiler option '%s' ignored", 0 },
   { W_UNSUPP_OPTION, ERROR_LEVEL_WARNING,
      "option '%s' no longer supported  '%s' ", 0 },
-  { W_UNKNOWN_FEXT, ERROR_LEVEL_WARNING,
+  { E_UNKNOWN_FEXT, ERROR_LEVEL_ERROR,
      "don't know what to do with file '%s'. file extension unsupported", 0 },
   { W_TOO_MANY_SRC, ERROR_LEVEL_WARNING,
      "cannot compile more than one source file. file '%s' ignored", 0 },
   { I_CYCLOMATIC, ERROR_LEVEL_INFO,
      "function '%s', # edges %d , # nodes %d , cyclomatic complexity %d", 0 },
-  { E_DIVIDE_BY_ZERO, ERROR_LEVEL_ERROR,
+  { E_DIVIDE_BY_ZERO, ERROR_LEVEL_WARNING,
      "dividing by ZERO", 0 },
   { E_FUNC_BIT, ERROR_LEVEL_ERROR,
      "function cannot return 'bit'", 0 },
@@ -396,7 +396,7 @@ struct
   { W_SAVE_RESTORE, ERROR_LEVEL_PEDANTIC,
      "unmatched #pragma save and #pragma restore", 0 },
   { E_INVALID_CRITICAL, ERROR_LEVEL_ERROR,
-     "not allowed in a critical section", 0 },
+     "not allowed in a critical section or critical function", 0 },
   { E_NOT_ALLOWED, ERROR_LEVEL_ERROR,
      "%s not allowed here", 0 },
   { E_BAD_TAG, ERROR_LEVEL_ERROR,
@@ -439,7 +439,7 @@ struct
   { W_BAD_PRAGMA_ARGUMENTS, ERROR_LEVEL_WARNING,
      "#pragma %s: bad argument(s); pragma ignored", 0 },
   { E_BAD_RESTRICT, ERROR_LEVEL_ERROR,
-     "Only pointers may be qualified with 'restrict'", 0 },
+     "Only object pointers may be qualified with 'restrict'", 0 },
   { E_BAD_INLINE, ERROR_LEVEL_ERROR,
      "Only functions may be qualified with 'inline'", 0 },
   { E_BAD_INT_ARGUMENT, ERROR_LEVEL_ERROR,
@@ -451,7 +451,7 @@ struct
   { W_DEPRECATED_KEYWORD, ERROR_LEVEL_WARNING,
      "keyword '%s' is deprecated, use '%s' instead", 0 },
   { E_STORAGE_CLASS_FOR_PARAMETER, ERROR_LEVEL_ERROR,
-     "storage class specified for parameter '%s'", 0 },
+     "storage class other than register specified for parameter '%s'", 0 },
   { E_OFFSETOF_TYPE, ERROR_LEVEL_ERROR,
      "offsetof can only be applied to structs/unions", 0 },
   { E_INCOMPLETE_FIELD, ERROR_LEVEL_ERROR,
@@ -463,9 +463,9 @@ struct
   { W_DUPLICATE_INIT, ERROR_LEVEL_WARNING,
      "Duplicate initializer at position %d; ignoring previous.", 0 },
   { E_INVALID_UNIVERSAL, ERROR_LEVEL_ERROR,
-     "incomplete universal character name \\%s.", 0 },
-  { W_UNIVERSAL_C99, ERROR_LEVEL_WARNING,
-     "universal character names are only valid in C99", 0 },
+     "invalid universal character name \\%s.", 0 },
+  { W_UNIVERSAL_C95, ERROR_LEVEL_WARNING,
+     "universal character names are only valid in C95 or later", 0 },
   { E_SHORTLONG, ERROR_LEVEL_ERROR,
      "invalid combination of short / long", 0 },
   { E_INTEGERSUFFIX, ERROR_LEVEL_ERROR,
@@ -479,7 +479,7 @@ struct
   { W_STRING_CANNOT_BE_TERMINATED, ERROR_LEVEL_PEDANTIC,
     "string '%s'cannot be terminated within array", 0 },
   { W_LONGLONG_LITERAL, ERROR_LEVEL_WARNING,
-    "support for large long long literals is incomplete", 0 },
+    "support for large long long literals is incomplete", 1 },
   { S_SYNTAX_ERROR, ERROR_LEVEL_SYNTAX_ERROR,
     "token -> '%s' ; column %d", 0 },
   { E_MIXING_CONFIG, ERROR_LEVEL_ERROR,
@@ -495,8 +495,8 @@ struct
      "%s() failed to parse line node, assuming %d bytes\n'%s'\n", 0 },
   { W_FLEXARRAY_INSTRUCT, ERROR_LEVEL_WARNING,
      "type of variable '%s' is struct with flexible array field", 0},
-  { W_ANONYMOUS_MEMBER, ERROR_LEVEL_WARNING,
-     "anonymous member in struct/union", 0},
+  { E_TYPE_IS_FUNCTION, ERROR_LEVEL_ERROR,
+     "'%s' has function type", 0},
   { W_INLINE_NAKED, ERROR_LEVEL_WARNING,
      "inline function '%s' is __naked", 0},
   { E_Z88DK_FASTCALL_PARAMETERS, ERROR_LEVEL_ERROR,
@@ -505,6 +505,52 @@ struct
     "ivalid parameter type in __z88dk_fastcall", 0 },
   { W_REPEAT_QUALIFIER, ERROR_LEVEL_WARNING,
     "duplicate specifier '%s'", 0 },
+  { W_NO_TYPE_SPECIFIER, ERROR_LEVEL_WARNING,
+    "no type specifier for '%s'", 0 },
+  { E_NO_TYPE_SPECIFIER, ERROR_LEVEL_ERROR,
+    "no type specifier for '%s'", 0 },
+  { E_MULTIPLE_DEFAULT_IN_GENERIC, ERROR_LEVEL_ERROR,
+    "multiple default expressions in generic association", 0 },
+  { E_MULTIPLE_MATCHES_IN_GENERIC, ERROR_LEVEL_ERROR,
+    "multiple matching expressions in generic association", 0 },
+  { E_NO_MATCH_IN_GENERIC, ERROR_LEVEL_ERROR,
+    "no matching expression in generic association", 0 },
+  { W_LABEL_WITHOUT_STATEMENT, ERROR_LEVEL_WARNING,
+    "label without statement", 0},
+  { E_WCHAR_CONST_C95, ERROR_LEVEL_ERROR,
+    "character constant of type wchar_t requires C95 or later", 0},
+  { E_WCHAR_CONST_C11, ERROR_LEVEL_ERROR,
+    "character constant of type char16_t or char32_t requires C11 or later", 0},
+  { E_WCHAR_STRING_C95, ERROR_LEVEL_ERROR,
+    "wide character string of type L requires C95 or later", 0},
+  { E_WCHAR_STRING_C11, ERROR_LEVEL_ERROR,
+    "wide character string of type u8, u, U requires C11 or later", 0},
+  { W_UNKNOWN_REG, ERROR_LEVEL_WARNING,
+    "unknown register specification %s", 0},
+  { E_HEXFLOAT_C99, ERROR_LEVEL_ERROR,
+    "hexadecimal floating constant requires ISO C99 or later", 0},
+  { E_ANONYMOUS_STRUCT_TAG, ERROR_LEVEL_ERROR,
+    "tagged anonymous struct/union '%s'", 0},
+  { W_INLINE_FUNCATTR, ERROR_LEVEL_WARNING,
+    "inline function '%s' might lose function attributes", 0},
+  { E_FOR_INITAL_DECLARATION_C99, ERROR_LEVEL_ERROR,
+    "initial declaration in for loop requires ISO C99 or later", 0},
+  { E_QUALIFIED_ARRAY_PARAM_C99, ERROR_LEVEL_ERROR,
+    "qualifiers in array parameters require ISO C99 or later", 0},
+  { E_QUALIFIED_ARRAY_NOPARAM, ERROR_LEVEL_ERROR,
+    "qualifier or static in array declarator that is not a parameter", 0},
+  { E_STATIC_ARRAY_PARAM_C99, ERROR_LEVEL_ERROR,
+    "static in array parameters requires ISO C99 or later", 0},
+  { E_INT_MULTIPLE, ERROR_LEVEL_ERROR,
+    "mutiple interrupt numbers for '%s'", 0},
+  { W_INCOMPAT_PTYPES, ERROR_LEVEL_WARNING,
+     "pointer types incompatible ", 0 },
+  { E_STATIC_ASSERTION_C2X, ERROR_LEVEL_ERROR,
+    "static assertion with one argument requires C2X or later", 0 },
+  { W_STATIC_ASSERTION_2, ERROR_LEVEL_WARNING,
+    "static assertion failed", 0 },
+  { E_DECL_AFTER_STATEMENT_C99, ERROR_LEVEL_ERROR,
+    "declaration after statement requires ISO C99 or later", 0 },
 };
 
 /* -------------------------------------------------------------------------------
@@ -689,6 +735,23 @@ setWarningDisabled (int errNum)
     ErrTab[errNum].disabled = 1;
 }
 
+/* -------------------------------------------------------------------------------
+ * disabledState - Enable/Disable output of specified warning
+ * -------------------------------------------------------------------------------
+ */
+int
+setWarningDisabledState (int errNum, int disabled)
+{
+  if ((errNum >= 0) && (errNum < NELEM (ErrTab)) && (ErrTab[errNum].errType <= ERROR_LEVEL_WARNING))
+  {
+    int originalState = ErrTab[errNum].disabled;
+    ErrTab[errNum].disabled = disabled;
+    return originalState;
+  }
+  return 0;
+}
+
+
 /* -------------------------------------------------------------------------------
  * Set the flag to treat warnings as errors
  * -------------------------------------------------------------------------------
