@@ -56,8 +56,8 @@ enum {
   E_PTR_REQD                    =  27, /* pointer required     */
   E_UNARY_OP                    =  28, /* unary operator bad op*/
   E_CONV_ERR                    =  29, /* conversion error     */
-  E_INT_REQD                    =  30, /* bit field must be int*/
-  E_BITFLD_SIZE                 =  31, /* bit field size > 16  */
+  E_INT_REQD                    =  30, /* bit-field must be int*/
+  E_BITFLD_SIZE                 =  31, /* bit-field size > 16  */
   W_TRUNCATION                  =  32, /* high order trucation */
   E_CODE_WRITE                  =  33, /* trying 2 write to code */
   E_LVALUE_CONST                =  34, /* lvalue is a const   */
@@ -124,7 +124,7 @@ enum {
   W_FUNC_NO_RETURN              =  95, /* no return statement found */
   W_PRE_PROC_WARNING            =  96, /* preprocessor generated warning */
   E_STRUCT_AS_ARG               =  97, /* structure passed as argument */
-  E_PREV_DEF_CONFLICT           =  98, /* previous definition conflicts with current */
+  E_PREV_DECL_CONFLICT          =  98, /* previous declaration conflicts with current */
   E_CODE_NO_INIT                =  99, /* vars in code space must have initializer */
   E_OPS_INTEGRAL                = 100, /* operans must be integral for certain assignments */
   E_TOO_MANY_PARMS              = 101, /* too many parameters */
@@ -145,7 +145,7 @@ enum {
   W_SHIFT_CHANGED               = 116, /* shift changed to zero */
   W_UNKNOWN_OPTION              = 117, /* don't know the option */
   W_UNSUPP_OPTION               = 118, /* processor reset has been redifned */
-  W_UNKNOWN_FEXT                = 119, /* unknown file extension */
+  E_UNKNOWN_FEXT                = 119, /* unknown file extension */
   W_TOO_MANY_SRC                = 120, /* can only compile one .c file at a time */
   I_CYCLOMATIC                  = 121, /* information message */
   E_DIVIDE_BY_ZERO              = 122, /* / 0 */
@@ -218,7 +218,7 @@ enum {
   E_FLEXARRAY_INEMPTYSTRCT      = 189, /* flexible array in otherwise empty struct */
   W_EMPTY_SOURCE_FILE           = 190, /* ISO C forbids an empty source file */
   W_BAD_PRAGMA_ARGUMENTS        = 191, /* #pragma %s: bad argument(s); pragma ignored */
-  E_BAD_RESTRICT                = 192, /* Only pointers may be qualified with 'restrict' */
+  E_BAD_RESTRICT                = 192, /* Only object pointers may be qualified with 'restrict' */
   E_BAD_INLINE                  = 193, /* Only functions may be qualified with 'inline' */
   E_BAD_INT_ARGUMENT            = 194, /* Bad integer option argument */
   E_NEGATIVE_ARRAY_SIZE         = 195, /* Size of array '%s' is negative */
@@ -230,8 +230,8 @@ enum {
   W_DEPRECATED_OPTION           = 201, /* deprecated compiler option '%s' */
   E_BAD_DESIGNATOR              = 202, /* Bad designated initializer */
   W_DUPLICATE_INIT              = 203, /* duplicate initializer */
-  E_INVALID_UNIVERSAL           = 204, /* incomplete universal character name %s. */
-  W_UNIVERSAL_C99               = 205, /* universal character names are only valid in C99 */
+  E_INVALID_UNIVERSAL           = 204, /* invalid universal character name %s. */
+  W_UNIVERSAL_C95               = 205, /* universal character names are only valid in C95 or later */
   E_SHORTLONG                   = 206, /* Invalid combination of short / long */
   E_INTEGERSUFFIX               = 207, /* Invalid integer suffix */
   E_AUTO_ADDRSPACE              = 208, /* named address space for auto var */
@@ -246,11 +246,34 @@ enum {
   W_INTERNAL_ERROR              = 217, /* warning for non-fatal internal errors - things that should not have happened, but can be handled */
   W_UNRECOGNIZED_ASM            = 218, /* unrecognized asm instruction in peephole optimizer */
   W_FLEXARRAY_INSTRUCT          = 219, /* using flexible arrays in a struct */
-  W_ANONYMOUS_MEMBER            = 220, /* anonymous member in struct/union */
+  E_TYPE_IS_FUNCTION            = 220, /* typedef void foo_t(void); foo_t foo; */
   W_INLINE_NAKED                = 221, /* inline function is naked */
   E_Z88DK_FASTCALL_PARAMETERS   = 222, /* invalid number of parameters in __z88dk_fastcall */
   E_Z88DK_FASTCALL_PARAMETER    = 223, /* invalid parameter type in __z88dk_fastcall */
   W_REPEAT_QUALIFIER            = 224, /* the same qualifier appears more than once */
+  W_NO_TYPE_SPECIFIER           = 225, /* type specifier missing in declaration */
+  E_NO_TYPE_SPECIFIER           = 226, /* type specifier missing in declaration */
+  E_MULTIPLE_DEFAULT_IN_GENERIC = 227, /* multiple default expressions in generic association */
+  E_MULTIPLE_MATCHES_IN_GENERIC = 228, /* multiple matching expressions in generic association */
+  E_NO_MATCH_IN_GENERIC         = 229, /* no matching expression in generic association */
+  W_LABEL_WITHOUT_STATEMENT     = 230, /* label without statement, not allowed in standard C */
+  E_WCHAR_CONST_C95             = 231, /* character constant of type wchar_t requires ISO C 95 or later */
+  E_WCHAR_CONST_C11             = 232, /* character constant of type char16_t or char32_t equires ISO C 11 or later */
+  E_WCHAR_STRING_C95            = 233, /* wide character string literal requires ISO C 95 or later */
+  E_WCHAR_STRING_C11            = 234, /* wide character string literal requires ISO C 11 or later */
+  W_UNKNOWN_REG                 = 235, /* unknown register specified */
+  E_HEXFLOAT_C99                = 236, /* hexadecimal floating constant requires ISO C99 or later */
+  E_ANONYMOUS_STRUCT_TAG        = 237, /* anonymous struct/union should not have a tag */
+  W_INLINE_FUNCATTR             = 238, /* inline functions should not be z88dk_fastcall or z88dk_callee */
+  E_FOR_INITAL_DECLARATION_C99  = 239, /* initial declaration in for loop requires ISO C99 or later */
+  E_QUALIFIED_ARRAY_PARAM_C99   = 240, /* qualifiers in array parameters require ISO C99 or later */
+  E_QUALIFIED_ARRAY_NOPARAM     = 241, /* qualifier or static in array declarator that is not a parameter */
+  E_STATIC_ARRAY_PARAM_C99      = 242, /* static in array parameters requires ISO C99 or later */
+  E_INT_MULTIPLE                = 243, /* mutiple interrupt numbers */
+  W_INCOMPAT_PTYPES             = 244, /* incompatible pointer assignment (not allowed by the standard, but allowed in SDCC) */
+  E_STATIC_ASSERTION_C2X        = 245, /* static assertion with one argument requires C2X or later */
+  W_STATIC_ASSERTION_2          = 246, /* static assertion failed */
+  E_DECL_AFTER_STATEMENT_C99    = 247, /* declaration after statement requires ISO C99 or later */
 
   /* don't touch this! */
   NUMBER_OF_ERROR_MESSAGES             /* Number of error messages */
@@ -365,6 +388,14 @@ disabled - Disable output of specified warning
 
 void setWarningDisabled (int errNum);
 
+/*
+-------------------------------------------------------------------------------
+disabledState - Enable/Disable output of specified warning
+-------------------------------------------------------------------------------
+*/
+
+int setWarningDisabledState (int errNum, int disabled);
+
 /*
 -------------------------------------------------------------------------------
 Set the flag to treat warnings as errors
