@@ -36,6 +36,8 @@ symbol *interrupts[INTNO_MAX + 1];
 void printIval (symbol *, sym_link *, initList *, struct dbuf_s *, bool check);
 set *publics = NULL;            /* public variables */
 set *externs = NULL;            /* Variables that are declared as extern */
+set *strSym = NULL;             /* string initializers */
+set *ccpStr = NULL;             /* char * const pointers with a string literal initializer */
 
 unsigned maxInterrupts = 0;
 int allocInfo = 1;
@@ -129,14 +131,14 @@ emitDebugSym (struct dbuf_s *oBuf, symbol * sym)
     {
       dbuf_printf (oBuf, "G$");
     }
-  dbuf_printf (oBuf, "%s$%d$%d", sym->name, sym->level, sym->block);
+  dbuf_printf (oBuf, "%s$%ld_%ld$%d", sym->name, sym->level / LEVEL_UNIT, sym->level % LEVEL_UNIT, sym->block);
 }
 
 /*-----------------------------------------------------------------*/
 /* emitRegularMap - emit code for maps with no special cases       */
 /*-----------------------------------------------------------------*/
 static void
-emitRegularMap (memmap * map, bool addPublics, bool arFlag)
+emitRegularMap (memmap *map, bool addPublics, bool arFlag)
 {
   symbol *sym;
   ast *ival = NULL;
@@ -150,7 +152,11 @@ emitRegularMap (memmap * map, bool addPublics, bool arFlag)
       if (!strcmp (map->sname, CODE_NAME))
         dbuf_tprintf (&map->oBuf, "\t!areacode\n", map->sname);
       else if (!strcmp (map->sname, DATA_NAME))
-        dbuf_tprintf (&map->oBuf, "\t!areadata\n", map->sname);
+        {
+          dbuf_tprintf (&map->oBuf, "\t!areadata\n", map->sname);
+          if (options.data_seg && strcmp (DATA_NAME, options.data_seg))
+            dbuf_tprintf (&map->oBuf, "\t!area\n", options.data_seg);
+        }
       else if (!strcmp (map->sname, HOME_NAME))
         dbuf_tprintf (&map->oBuf, "\t!areahome\n", map->sname);
       else
@@ -208,6 +214,35 @@ emitRegularMap (memmap * map, bool addPublics, bool arFlag)
           if ((SPEC_OCLS (sym->etype) == xidata || SPEC_OCLS (sym->etype) == initialized) && !SPEC_ABSA (sym->etype) && !SPEC_ADDRSPACE (sym->etype))
             {
               sym_link *t;
+              ast *ival = NULL;
+              symbol *tsym = copySymbol (sym);
+
+              // check for constant
+              if (IS_AGGREGATE (tsym->type))
+                {
+                  ival = initAggregates (tsym, tsym->ival, NULL);
+                }
+              else
+                {
+                  if (getNelements (tsym->type, tsym->ival) > 1)
+                    {
+                      werrorfl (tsym->fileDef, tsym->lineDef, W_EXCESS_INITIALIZERS, "scalar", tsym->name);
+                    }
+                  ival = newNode ('=', newAst_VALUE (symbolVal (tsym)),
+                                  decorateType (resolveSymbols (list2expr (tsym->ival)), RESULT_TYPE_NONE));
+                }
+              if (ival)
+                {
+                  // set ival's lineno to where the symbol was defined
+                  setAstFileLine (ival, filename = tsym->fileDef, lineno = tsym->lineDef);
+                  // check if this is not a constant expression
+                  if (!constExprTree (ival))
+                    {
+                      werrorfl (ival->filename, ival->lineno, E_CONST_EXPECTED, "found expression");
+                      // but try to do it anyway
+                    }
+                }
+
               /* create a new "XINIT (CODE)" symbol, that will be emitted later
                  in the static seg */
               newSym = copySymbol (sym);
@@ -225,8 +260,10 @@ emitRegularMap (memmap * map, bool addPublics, bool arFlag)
                 DCL_PTR_CONST (t) = 1;
               SPEC_STAT (newSym->etype) = 1;
 
-              /* This results in unencessary calls to stringToSymbol() through decorateType(), which make strings take twice as much  space as they should */
+              strSym = NULL;
+              ++noAlloc;
               resolveIvalSym (newSym->ival, newSym->type);
+              --noAlloc;
 
               // add it to the "XINIT (CODE)" segment
               addSet ((SPEC_OCLS (sym->etype) == xidata) ? &xinit->syms : &initializer->syms, newSym);
@@ -234,8 +271,10 @@ emitRegularMap (memmap * map, bool addPublics, bool arFlag)
               if (!SPEC_ABSA (sym->etype))
                 {
                   struct dbuf_s tmpBuf;
+                  symbol *ps = NULL;
+                  set *tmpSym = NULL;
 
-                  dbuf_init (&tmpBuf, 4096);
+                  wassert(dbuf_init (&tmpBuf, 4096));
                   // before allocation we must parse the sym->ival tree
                   // but without actually generating initialization code
                   ++noAlloc;
@@ -244,8 +283,20 @@ emitRegularMap (memmap * map, bool addPublics, bool arFlag)
                   printIval (sym, sym->type, sym->ival, &tmpBuf, TRUE);
                   --noInit;
                   --noAlloc;
+
+                  // delete redundant __str_%d symbols (initializer for char arrays)
+                  for (ps = setFirstItem (statsg->syms); ps; ps = setNextItem (statsg->syms))
+                    if (!strstr (tmpBuf.buf, ps->name) && isinSet (strSym, ps))
+                      addSet (&tmpSym, ps);
+                  for (ps = setFirstItem (tmpSym); ps; ps = setNextItem (tmpSym))
+                    deleteSetItem (&statsg->syms, ps);
+
+                  deleteSet (&tmpSym);
                   dbuf_destroy (&tmpBuf);
                 }
+
+              if (strSym)
+                deleteSet (&strSym);
             }
           else
             {
@@ -294,17 +345,6 @@ emitRegularMap (memmap * map, bool addPublics, bool arFlag)
               emitDebugSym (&map->oBuf, sym);
               dbuf_printf (&map->oBuf, " == 0x%04x\n", SPEC_ADDR (sym->etype));
             }
-          if (TARGET_IS_XA51)
-            {
-              if (map == sfr)
-                {
-                  equ = "sfr";
-                }
-              else if (map == bit || map == sfrbit)
-                {
-                  equ = "bit";
-                }
-            }
           dbuf_printf (&map->oBuf, "%s\t%s\t0x%04x\n", sym->rname, equ, SPEC_ADDR (sym->etype));
         }
       else
@@ -340,7 +380,7 @@ emitRegularMap (memmap * map, bool addPublics, bool arFlag)
 /* initValPointer - pointer initialization code massaging          */
 /*-----------------------------------------------------------------*/
 value *
-initValPointer (ast * expr)
+initValPointer (ast *expr)
 {
   value *val;
 
@@ -488,7 +528,7 @@ initValPointer (ast * expr)
 /* initPointer - pointer initialization code massaging             */
 /*-----------------------------------------------------------------*/
 value *
-initPointer (initList * ilist, sym_link * toType, int showError)
+initPointer (initList *ilist, sym_link *toType, int showError)
 {
   value *val;
   ast *expr;
@@ -501,7 +541,11 @@ initPointer (initList * ilist, sym_link * toType, int showError)
   expr = list2expr (ilist);
 
   if (!expr)
-    goto wrong;
+    {
+      if (showError)
+        werror (E_CONST_EXPECTED);
+      return 0;
+    }
 
   /* try it the old way first */
   if ((val = constExprValue (expr, FALSE)))
@@ -520,7 +564,7 @@ initPointer (initList * ilist, sym_link * toType, int showError)
   /* (char *)(expr1) */
   if (IS_CAST_OP (expr))
     {
-      if (compareType (toType, expr->left->ftype) == 0)
+      if (compareType (toType, expr->left->ftype) == 0 && showError)
         {
           werror (W_INIT_WRONG);
           printFromToType (expr->left->ftype, toType);
@@ -543,33 +587,28 @@ initPointer (initList * ilist, sym_link * toType, int showError)
   if (val)
     return val;
 
-wrong:
   if (showError)
-    if (expr)
-      werrorfl (expr->filename, expr->lineno, E_INCOMPAT_PTYPES);
-    else
-      werror (E_INCOMPAT_PTYPES);
-  return NULL;
+    {
+      if (expr)
+        werrorfl (expr->filename, expr->lineno, E_CONST_EXPECTED);
+      else
+        werror (E_CONST_EXPECTED);
+    }
+  return 0;
 }
 
 /*-----------------------------------------------------------------*/
-/* printChar - formats and prints a characater string with DB      */
+/* printChar - formats and prints a UTF-8 character string with DB */
 /*-----------------------------------------------------------------*/
 void
 printChar (struct dbuf_s *oBuf, const char *s, int plen)
 {
   int i;
-  int len = plen;
   int pplen = 0;
   char buf[100];
   char *p = buf;
-  int strEnd = plen - 1;
 
-  if (s)
-    while (s[strEnd] != 0)
-      strEnd--;
-
-  while (len && pplen < plen)
+  while (pplen < plen)
     {
       i = 60;
       while (i && pplen < plen)
@@ -579,29 +618,71 @@ printChar (struct dbuf_s *oBuf, const char *s, int plen)
               *p = '\0';
               if (p != buf)
                 dbuf_tprintf (oBuf, "\t!ascii\n", buf);
-              dbuf_tprintf (oBuf, "\t!db !constbyte\n", pplen < strEnd ? ((unsigned char) *s) : 0x00);
+              dbuf_tprintf (oBuf, "\t!db !constbyte\n", (unsigned char) *s);
               p = buf;
+              i = 60;
             }
           else
             {
               *p = *s;
               p++;
+              i--;
             }
           s++;
           pplen++;
-          i--;
         }
       if (p != buf)
         {
           *p = '\0';
           dbuf_tprintf (oBuf, "\t!ascii\n", buf);
           p = buf;
+          i = 60;
         }
+    }
+}
 
-      if (len > 60)
-        len -= 60;
+/*-----------------------------------------------------------------*/
+/* printChar16 - formats and prints a UTF-16 character string with DB*/
+/*-----------------------------------------------------------------*/
+void
+printChar16 (struct dbuf_s *oBuf, const TYPE_TARGET_UINT *s, int plen)
+{
+  int pplen = 0;
+
+  while (pplen < plen)
+    {
+      if (port->little_endian)
+        dbuf_printf (oBuf, "\t.byte %d,%d\n", (*s >> 0) & 0xff, (*s >> 8) & 0xff);
       else
-        len = 0;
+        dbuf_printf (oBuf, "\t.byte %d,%d\n", (*s >> 8) & 0xff, (*s >> 0) & 0xff);
+
+      s++;
+      pplen++;
+    }
+  while (pplen < plen)
+    {
+      dbuf_tprintf (oBuf, "\t!db !constbyte\n", 0);
+      pplen++;
+    }
+}
+
+/*-----------------------------------------------------------------*/
+/* printChar32 - formats and prints a UTF-32 character string with DB*/
+/*-----------------------------------------------------------------*/
+void
+printChar32 (struct dbuf_s *oBuf, const TYPE_TARGET_ULONG *s, int plen)
+{
+  int pplen = 0;
+
+  while (pplen < plen)
+    {
+      if (port->little_endian)
+        dbuf_printf (oBuf, "\t.byte %d,%d,%d,%d\n", (*s >> 0) & 0xff, (*s >> 8) & 0xff, (*s >> 16) & 0xff, (*s >> 24) & 0xff);
+      else
+        dbuf_printf (oBuf, "\t.byte %d,%d,%d,%d\n", (*s >> 24) & 0xff, (*s >> 16) & 0xff, (*s >> 8) & 0xff,(*s >> 0) & 0xff);
+
+      s++;
+      pplen++;
     }
   while (pplen < plen)
     {
@@ -759,7 +840,7 @@ printIvalType (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *o
 
   if (val->type != type)
     {
-      val = valCastLiteral (type, floatFromVal (val), ullFromVal (val));
+      val = valCastLiteral (type, floatFromVal (val), (TYPE_TARGET_ULONGLONG) ullFromVal (val));
     }
 
   if (IS_INTEGRAL (val->type))
@@ -861,8 +942,9 @@ printIvalType (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *o
 
 /*-----------------------------------------------------------------*/
 /* printIvalBitFields - generate initializer for bitfields         */
+/* return the number of bytes written                              */
 /*-----------------------------------------------------------------*/
-static void
+static unsigned int
 printIvalBitFields (symbol ** sym, initList ** ilist, struct dbuf_s *oBuf)
 {
   symbol *lsym = *sym;
@@ -870,6 +952,7 @@ printIvalBitFields (symbol ** sym, initList ** ilist, struct dbuf_s *oBuf)
   unsigned long ival = 0;
   unsigned size = 0;
   unsigned bit_start = 0;
+  unsigned long int bytes_written = 0;
 
   while (lsym && IS_BITFIELD (lsym->type))
     {
@@ -918,31 +1001,43 @@ printIvalBitFields (symbol ** sym, initList ** ilist, struct dbuf_s *oBuf)
 
   switch (size)
     {
+    case 0:
+      dbuf_tprintf (oBuf, "\n");
+      break;
+
     case 1:
       dbuf_tprintf (oBuf, "\t!db !constbyte\n", ival);
+      bytes_written++;
       break;
 
     case 2:
       dbuf_tprintf (oBuf, "\t!db !constbyte, !constbyte\n", (ival & 0xff), (ival >> 8) & 0xff);
+      bytes_written += 2;
       break;
 
     case 4:
       dbuf_tprintf (oBuf, "\t!db !constbyte, !constbyte, !constbyte, !constbyte\n",
                     (ival & 0xff), (ival >> 8) & 0xff, (ival >> 16) & 0xff, (ival >> 24) & 0xff);
+      bytes_written += 4;
       break;
+    default:
+      wassert (0);
     }
   *sym = lsym;
   *ilist = lilist;
+
+  return (bytes_written);
 }
 
 /*-----------------------------------------------------------------*/
 /* printIvalStruct - generates initial value for structures        */
 /*-----------------------------------------------------------------*/
 static void
-printIvalStruct (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *oBuf)
+printIvalStruct (symbol *sym, sym_link *type, initList *ilist, struct dbuf_s *oBuf)
 {
   symbol *sflds;
   initList *iloop = NULL;
+  unsigned int skip_holes = 0;
 
   sflds = SPEC_STRUCT (type)->fields;
 
@@ -985,6 +1080,8 @@ printIvalStruct (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s
     {
       while (sflds)
         {
+          unsigned int oldoffset = sflds->offset;
+
           if (IS_BITFIELD (sflds->type))
             printIvalBitFields (&sflds, &iloop, oBuf);
           else
@@ -993,6 +1090,19 @@ printIvalStruct (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s
               sflds = sflds->next;
               iloop = iloop ? iloop->next : NULL;
             }
+
+          // Handle members from anonymous unions. Just a hack to fix bug #2643.
+          while (sflds && sflds->offset == oldoffset)
+            {
+              sflds = sflds->next;
+              skip_holes++;
+            }
+        }
+
+      while (skip_holes && iloop && iloop->type == INIT_HOLE)
+        {
+          skip_holes--;
+          iloop = iloop ? iloop->next : NULL;
         }
     }
 
@@ -1007,7 +1117,9 @@ int
 printIvalChar (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *oBuf, const char *s, bool check)
 {
   value *val;
-  unsigned int size = DCL_ELEM (type);
+  size_t size = DCL_ELEM(type);
+  char *p;
+  size_t asz;
 
   if (!s)
     {
@@ -1029,7 +1141,15 @@ printIvalChar (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *o
           if (check && DCL_ELEM (val->type) > size)
             werror (W_EXCESS_INITIALIZERS, "array of chars", sym->name, sym->lineDef);
 
-          printChar (oBuf, SPEC_CVAL (val->etype).v_char, size);
+          if (size > (asz = DCL_ELEM (val->type)) && !!(p = malloc (size)))
+            {
+              memcpy (p, SPEC_CVAL (val->etype).v_char, asz);
+              memset (p + asz, 0x00, size - asz);
+              printChar (oBuf, p, size);
+              free (p);
+            }
+          else
+            printChar (oBuf, SPEC_CVAL (val->etype).v_char, size);
 
           return 1;
         }
@@ -1041,6 +1161,125 @@ printIvalChar (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *o
   return 1;
 }
 
+static size_t strLen16(const TYPE_TARGET_UINT *s)
+{
+  size_t l = 0;
+  while(*s++)
+    l++;
+
+  return l;
+}
+
+/*-----------------------------------------------------------------*/
+/* printIvalChar16 - generates initital value for character array  */
+/*-----------------------------------------------------------------*/
+int
+printIvalChar16 (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *oBuf, const TYPE_TARGET_UINT *s, bool check)
+{
+  value *val;
+  size_t size = DCL_ELEM(type);
+  TYPE_TARGET_UINT *p;
+  size_t asz;
+
+  if (!s)
+    {
+      val = list2val (ilist, TRUE);
+      /* if the value is a character string  */
+      if (IS_ARRAY (val->type) && IS_INT (val->etype) && IS_UNSIGNED (val->etype) && !IS_LONG (val->etype))
+        {
+          if (!size)
+            {
+              /* we have not been given a size, but now we know it */
+              size = strLen16 (SPEC_CVAL (val->etype).v_char16) + 1;
+              /* but first check, if it's a flexible array */
+              if (sym && IS_STRUCT (sym->type))
+                sym->flexArrayLength = size;
+              else
+                DCL_ELEM (type) = size;
+            }
+
+          if (check && DCL_ELEM (val->type) > size)
+            werror (W_EXCESS_INITIALIZERS, "array of chars", sym->name, sym->lineDef);
+
+          if (size > (asz = DCL_ELEM (val->type)) && !!(p = malloc (size * 2)))
+            {
+              memcpy (p, SPEC_CVAL (val->etype).v_char16, asz * 2);
+              memset (p + asz, 0x00, size * 2 - asz * 2);
+              printChar16 (oBuf, p, size);
+              free (p);
+            }
+          else
+            printChar16 (oBuf, SPEC_CVAL (val->etype).v_char16, size);
+
+          return 1;
+        }
+      else
+        return 0;
+    }
+  else
+    printChar16 (oBuf, s, strLen16 (s) + 1);
+  return 1;
+}
+
+static size_t strLen32(const TYPE_TARGET_ULONG *s)
+{
+  size_t l = 0;
+  while(*s++)
+    l++;
+  return l;
+}
+
+/*-----------------------------------------------------------------*/
+/* printIvalChar32 - generates initital value for character array  */
+/*-----------------------------------------------------------------*/
+int
+printIvalChar32 (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *oBuf, const TYPE_TARGET_ULONG *s, bool check)
+{
+  value *val;
+  size_t size = DCL_ELEM(type);
+  TYPE_TARGET_ULONG *p;
+  size_t asz;
+
+  if (!s)
+    {
+      val = list2val (ilist, TRUE);
+      /* if the value is a character string  */
+      if (IS_ARRAY (val->type) && IS_INT (val->etype) && IS_UNSIGNED (val->etype) && IS_LONG (val->etype))
+        {
+          if (!size)
+            {
+              /* we have not been given a size, but now we know it */
+              size = strLen32 (SPEC_CVAL (val->etype).v_char32) + 1;
+              /* but first check, if it's a flexible array */
+              if (sym && IS_STRUCT (sym->type))
+                sym->flexArrayLength = size;
+              else
+                DCL_ELEM (type) = size;
+            }
+
+          if (check && DCL_ELEM (val->type) > size)
+            werror (W_EXCESS_INITIALIZERS, "array of chars", sym->name, sym->lineDef);
+
+          if (size > (asz = DCL_ELEM (val->type)) && !!(p = malloc (size * 4)))
+            {
+              memcpy (p, SPEC_CVAL (val->etype).v_char32, asz * 4);
+              memset (p + asz, 0x00, size * 4 - asz * 4);
+              printChar32 (oBuf, p, size);
+              free (p);
+            }
+          else
+            printChar32 (oBuf, SPEC_CVAL (val->etype).v_char32, size);
+
+          return 1;
+        }
+      else
+        return 0;
+    }
+  else
+    printChar32 (oBuf, s, strLen32 (s) + 1);
+  return 1;
+}
+
 /*-----------------------------------------------------------------*/
 /* printIvalArray - generates code for array initialization        */
 /*-----------------------------------------------------------------*/
@@ -1057,7 +1296,7 @@ printIvalArray (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *
       /* array of characters can be init  */
       /* by a string                      */
       /* char *p = "abc";                 */
-      if (IS_CHAR (type->next) && ilist->type == INIT_NODE)
+      if ((IS_CHAR (type->next) || IS_INT (type->next)) && ilist->type == INIT_NODE)
         {
           val = list2val (ilist, TRUE);
           if (!val)
@@ -1070,12 +1309,18 @@ printIvalArray (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *
               werrorfl (ilist->filename, ilist->lineno, E_CONST_EXPECTED);
               return;
             }
-          if (printIvalChar (sym, type,
-                             ilist, oBuf, SPEC_CVAL (sym->etype).v_char, check))
+          if (IS_CHAR (type->next) && printIvalChar (sym, type, ilist, oBuf, SPEC_CVAL (sym->etype).v_char, check))
             return;
+          if (IS_INT (type->next) && IS_UNSIGNED (type->next))
+            {
+              if (!IS_LONG (type->next) && printIvalChar16 (sym, type, ilist, oBuf, SPEC_CVAL (sym->etype).v_char16, check))
+                return;
+              if (IS_LONG (type->next) && printIvalChar32 (sym, type, ilist, oBuf, SPEC_CVAL (sym->etype).v_char32, check))
+                return;
+            }
         }
       /* char *p = {"abc"}; */
-      if (IS_CHAR (type->next) && ilist->type == INIT_DEEP && ilist->init.deep && ilist->init.deep->type == INIT_NODE)
+      if ((IS_CHAR (type->next) || IS_INT (type->next)) && ilist->type == INIT_DEEP && ilist->init.deep && ilist->init.deep->type == INIT_NODE)
         {
           val = list2val (ilist->init.deep, TRUE);
           if (!val)
@@ -1088,9 +1333,15 @@ printIvalArray (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *
               werrorfl (ilist->init.deep->filename, ilist->init.deep->lineno, E_CONST_EXPECTED);
               return;
             }
-          if (printIvalChar (sym, type,
-                             ilist->init.deep, oBuf, SPEC_CVAL (sym->etype).v_char, check))
+          if (IS_CHAR (type->next) && printIvalChar (sym, type, ilist->init.deep, oBuf, SPEC_CVAL (sym->etype).v_char, check))
             return;
+          if (IS_INT (type->next) && IS_UNSIGNED (type->next))
+            {
+              if (!IS_LONG (type->next) && printIvalChar16 (sym, type, ilist->init.deep, oBuf, SPEC_CVAL (sym->etype).v_char16, check))
+                return;
+              if (IS_LONG (type->next) && printIvalChar32 (sym, type, ilist->init.deep, oBuf, SPEC_CVAL (sym->etype).v_char32, check))
+                return;
+            }
         }
 
       /* not the special case             */
@@ -1179,9 +1430,14 @@ printIvalFuncPtr (sym_link * type, initList * ilist, struct dbuf_s *oBuf)
 
   size = getSize (type);
 
-  if (size == FPTRSIZE)
+  if (size == FUNCPTRSIZE)
     {
-      if (port->use_dw_for_init)
+      if (TARGET_IS_STM8 && FUNCPTRSIZE == 3)
+        {
+          _printPointerType (oBuf, name, size);
+          dbuf_printf (oBuf, "\n");
+        }
+      else if (port->use_dw_for_init)
         {
           dbuf_tprintf (oBuf, "\t!dws\n", name);
         }
@@ -1190,14 +1446,14 @@ printIvalFuncPtr (sym_link * type, initList * ilist, struct dbuf_s *oBuf)
           printPointerType (oBuf, name);
         }
     }
-  else if (size == GPTRSIZE)
+  else if (size == BFUNCPTRSIZE)
     {
       _printPointerType (oBuf, name, size);
       dbuf_printf (oBuf, "\n");
     }
   else
     {
-      assert (0);
+      wassertl (0, "Invalid function pointer size.");
     }
 
   return;
@@ -1210,6 +1466,13 @@ int
 printIvalCharPtr (symbol * sym, sym_link * type, value * val, struct dbuf_s *oBuf)
 {
   int size = 0;
+  char *p;
+
+  if (val && !!(p = (char *) malloc (strlen (val->name) + 1)))
+    {
+      strcpy (p, val->name);
+      addSet (&ccpStr, p);
+    }
 
   /* PENDING: this is _very_ mcs51 specific, including a magic
      number...
@@ -1223,7 +1486,7 @@ printIvalCharPtr (symbol * sym, sym_link * type, value * val, struct dbuf_s *oBu
         {
           dbuf_tprintf (oBuf, "\t!dbs\n", val->name);
         }
-      else if (size == FPTRSIZE)
+      else if (size == FARPTRSIZE)
         {
           if (port->use_dw_for_init)
             {
@@ -1274,7 +1537,7 @@ printIvalCharPtr (symbol * sym, sym_link * type, value * val, struct dbuf_s *oBu
             dbuf_tprintf (oBuf, "\t.byte %s,%s\n", aopLiteral (val, 1), aopLiteral (val, 0));
           break;
         case 3:
-          if (IS_GENPTR (type) && GPTRSIZE > FPTRSIZE && floatFromVal (val) != 0)
+          if (IS_GENPTR (type) && GPTRSIZE > FARPTRSIZE && floatFromVal (val) != 0)
             {
               if (!IS_PTR (val->type) && !IS_FUNC (val->type))
                 {
@@ -1295,7 +1558,7 @@ printIvalCharPtr (symbol * sym, sym_link * type, value * val, struct dbuf_s *oBu
             }
           break;
         case 4:
-          if (IS_GENPTR (type) && GPTRSIZE > FPTRSIZE && floatFromVal (val) != 0)
+          if (IS_GENPTR (type) && GPTRSIZE > FARPTRSIZE && floatFromVal (val) != 0)
             {
               if (!IS_PTR (val->type) && !IS_FUNC (val->type))
                 {
@@ -1370,7 +1633,7 @@ printIvalPtr (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *oB
     return;
 
   /* if character pointer */
-  if (IS_CHAR (type->next))
+  if (IS_CHAR (type->next) || IS_INT (type->next) && IS_UNSIGNED (type->next))
     if (printIvalCharPtr (sym, type, val, oBuf))
       return;
 
@@ -1398,7 +1661,7 @@ printIvalPtr (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *oB
           else
             dbuf_tprintf (oBuf, "\t.byte %s,%s\n", aopLiteral (val, 1), aopLiteral (val, 0));
           break;
-        case 3:                // how about '390??
+        case 3:
           dbuf_printf (oBuf, "; generic printIvalPtr\n");
           if (port->little_endian)
             dbuf_printf (oBuf, "\t.byte %s,%s", aopLiteral (val, 0), aopLiteral (val, 1));
@@ -1410,6 +1673,16 @@ printIvalPtr (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *oB
             dbuf_tprintf (oBuf, ",!immedbyte\n", pointerTypeToGPByte (DCL_TYPE (val->type), val->name, sym->name));
           else
             dbuf_printf (oBuf, ",%s\n", aopLiteral (val, 2));
+          break;
+        case 4:
+          if (TARGET_IS_DS390 || TARGET_IS_DS400)
+            dbuf_printf (oBuf, "\t.byte %s,%s,%s,%s\n", aopLiteral (val, 0), aopLiteral (val, 1), aopLiteral (val, 2), aopLiteral (val, 3));
+          else
+            wassertl(0, "Printing pointer of invalid size");
+          break;
+       default:
+          wassertl(0, "Printing pointer of invalid size");
+          break;
         }
       return;
     }
@@ -1420,7 +1693,7 @@ printIvalPtr (symbol * sym, sym_link * type, initList * ilist, struct dbuf_s *oB
     {
       dbuf_tprintf (oBuf, "\t!dbs\n", val->name);
     }
-  else if (size == FPTRSIZE)
+  else if (size == FARPTRSIZE)
     {
       if (port->use_dw_for_init)
         dbuf_tprintf (oBuf, "\t!dws\n", val->name);
@@ -1543,9 +1816,14 @@ void
 emitStaticSeg (memmap *map, struct dbuf_s *oBuf)
 {
   symbol *sym;
+  set *tmpSet = NULL;
 
   /* fprintf(out, "\t.area\t%s\n", map->sname); */
 
+  /* eliminate redundant __str_%d (generated in stringToSymbol(), SDCCast.c) */
+  for (sym = setFirstItem (map->syms); sym; sym = setNextItem (map->syms))
+    addSet (&tmpSet, sym);
+
   /* for all variables in this segment do */
   for (sym = setFirstItem (map->syms); sym; sym = setNextItem (map->syms))
     {
@@ -1553,6 +1831,19 @@ emitStaticSeg (memmap *map, struct dbuf_s *oBuf)
       if (IS_EXTERN (sym->etype) && !sym->ival)
         continue;
 
+      /* eliminate redundant __str_%d (generated in stringToSymbol(), SDCCast.c) */
+      if (!isinSet (tmpSet, sym))
+        {
+          const char *p;
+          if (!ccpStr)
+            continue;
+          for (p = setFirstItem (ccpStr); p; p = setNextItem (ccpStr))
+            if (strcmp (p, sym->name) == 0)
+              break;
+          if (!p)
+            continue;
+        }
+
       /* if it is not static add it to the public table */
       if (!IS_STATIC (sym->etype))
         {
@@ -1612,9 +1903,19 @@ emitStaticSeg (memmap *map, struct dbuf_s *oBuf)
                 }
               dbuf_printf (oBuf, "%s:\n", sym->rname);
               /* special case for character strings */
-              if (IS_ARRAY (sym->type) && IS_CHAR (sym->type->next) && SPEC_CVAL (sym->etype).v_char)
+              if (IS_ARRAY (sym->type) &&
+                (IS_CHAR (sym->type->next) && SPEC_CVAL (sym->etype).v_char ||
+                 IS_INT (sym->type->next) && !IS_LONG (sym->type->next) && SPEC_CVAL (sym->etype).v_char16 ||
+                 IS_INT (sym->type->next) && IS_LONG (sym->type->next) && SPEC_CVAL (sym->etype).v_char32))
                 {
-                  printChar (oBuf, SPEC_CVAL (sym->etype).v_char, size);
+                  if (IS_CHAR (sym->type->next))
+                    printChar (oBuf, SPEC_CVAL (sym->etype).v_char, size);
+                  else if (IS_INT (sym->type->next) && !IS_LONG (sym->type->next))
+                    printChar16 (oBuf, SPEC_CVAL (sym->etype).v_char16, size / 2);
+                  else if (IS_INT (sym->type->next) && IS_LONG (sym->type->next))
+                    printChar32 (oBuf, SPEC_CVAL (sym->etype).v_char32, size / 4);
+                  else
+                    wassert (0);
                 }
               else
                 {
@@ -1623,6 +1924,17 @@ emitStaticSeg (memmap *map, struct dbuf_s *oBuf)
             }
         }
     }
+
+  if (tmpSet)
+    deleteSet (&tmpSet);
+  if (ccpStr)
+    {
+      char *p;
+      for (p = setFirstItem (ccpStr); p; p = setNextItem (ccpStr))
+        if (p)
+          free (p);
+      deleteSet (&ccpStr);
+    }
 }
 
 /*-----------------------------------------------------------------*/
@@ -1665,10 +1977,9 @@ emitMaps (void)
   emitRegularMap (code, TRUE, FALSE);
 
   if (options.const_seg)
-    {
-      dbuf_tprintf (&code->oBuf, "\t!area\n", options.const_seg);
-    }
+    dbuf_tprintf (&code->oBuf, "\t!area\n", options.const_seg);
   emitStaticSeg (statsg, &code->oBuf);
+
   if (port->genXINIT)
     {
       dbuf_tprintf (&code->oBuf, "\t!area\n", xinit->sname);
@@ -1748,11 +2059,8 @@ char *iComments2 = {
 void
 initialComments (FILE * afile)
 {
-  time_t t;
-  time (&t);
   fprintf (afile, "%s", iComments1);
-  fprintf (afile, "; Version " SDCC_VERSION_STR " #%s (%s) (%s)\n", getBuildNumber (), getBuildDate (), getBuildEnvironment ());
-  fprintf (afile, "; This file was generated %s", asctime (localtime (&t)));
+  fprintf (afile, "; Version " SDCC_VERSION_STR " #%s (%s)\n", getBuildNumber (), getBuildEnvironment ());
   fprintf (afile, "%s", iComments2);
 }
 
@@ -1883,7 +2191,7 @@ glue (void)
   dbuf_init (&ovrBuf, 4096);
 
   mcs51_like = (port->general.glue_up_main &&
-                (TARGET_IS_MCS51 || TARGET_IS_DS390 || TARGET_IS_XA51 || TARGET_IS_DS400));
+                (TARGET_IS_MCS51 || TARGET_IS_DS390 || TARGET_IS_DS400));
 
   /* print the global struct definitions */
   if (options.debug)
@@ -1997,7 +2305,7 @@ glue (void)
   if (port->assembler.externGlobal)
     printExterns (asmFile);
 
-  if ((mcs51_like) || (TARGET_IS_Z80 || TARGET_IS_Z180 || TARGET_IS_RABBIT))  /*.p.t.20030924 need to output SFR table for Z80 as well */
+  if ((mcs51_like) || (TARGET_IS_Z80 || TARGET_IS_GBZ80 || TARGET_IS_Z180 || TARGET_IS_RABBIT))  /*.p.t.20030924 need to output SFR table for Z80 as well */
     {
       /* copy the sfr segment */
       fprintf (asmFile, "%s", iComments2);
@@ -2239,7 +2547,7 @@ glue (void)
 
       /* put in jump or call to main */
       if(TARGET_IS_STM8)
-        fprintf (asmFile, "\tjp\t_main\n");
+        fprintf (asmFile, options.model == MODEL_LARGE ? "\tjpf\t_main\n" : "\tjp\t_main\n");
       else
         fprintf (asmFile, "\t%cjmp\t_main\n", options.acall_ajmp ? 'a' : 'l');        /* needed? */
       fprintf (asmFile, ";\treturn from main will return to caller\n");
@@ -2257,3 +2565,39 @@ glue (void)
     }
   fclose (asmFile);
 }
+
+/* will return 1 if the string is a part
+   of a target specific keyword */
+int
+isTargetKeyword (const char *s)
+{
+  int i;
+
+  if (port->keywords == NULL)
+    return 0;
+
+  if (s[0] == '_' && s[1] == '_')
+    {
+      /* Keywords in the port's array have either 0 or 1 underscore, */
+      /* so skip over the appropriate number of chars when comparing */
+      for (i = 0 ; port->keywords[i] ; i++ )
+        {
+          if (port->keywords[i][0] == '_' &&
+              strcmp(port->keywords[i],s+1) == 0)
+            return 1;
+          else if (strcmp(port->keywords[i],s+2) == 0)
+            return 1;
+        }
+    }
+  else
+    {
+      for (i = 0 ; port->keywords[i] ; i++ )
+        {
+          if (strcmp(port->keywords[i],s) == 0)
+            return 1;
+        }
+    }
+
+  return 0;
+}
+
