@@ -82,7 +82,7 @@ typedef struct bucket
 {
   void *sym;                    /* pointer to the object      */
   char name[SDCC_NAME_MAX + 1]; /* name of this symbol        */
-  int level;                    /* nest level for this symbol */
+  long level;                   /* nest level for this symbol */
   int block;                    /* belongs to which block     */
   struct bucket *prev;          /* ptr 2 previous bucket      */
   struct bucket *next;          /* ptr 2 next bucket          */
@@ -92,7 +92,7 @@ bucket;
 typedef struct structdef
 {
   char tag[SDCC_NAME_MAX + 1];  /* tag part of structure      */
-  unsigned char level;          /* Nesting level              */
+  long level;                   /* Nesting level              */
   int block;                    /* belongs to which block     */
   struct symbol *fields;        /* pointer to fields          */
   unsigned size;                /* sizeof the table in bytes  */
@@ -177,6 +177,7 @@ typedef struct specifier
   unsigned b_isregparm:1;           /* is the first parameter     */
   unsigned b_isenum:1;              /* is an enumerated type      */
   unsigned b_bitUnnamed:1;          /* is an unnamed bit-field    */
+  unsigned b_needspar:1;            /* has to be a parameter      */
   unsigned _bitStart;               /* bit start position         */
   unsigned _bitLength;              /* bit length                 */
   unsigned _addr;                   /* address of symbol          */
@@ -185,7 +186,9 @@ typedef struct specifier
   union
   {                                   /* Values if constant or enum */
     TYPE_TARGET_INT v_int;            /* 2 bytes: int and char values            */
-    const char *v_char;               /*          character string               */
+    const char *v_char;               /*          char character string          */
+    const TYPE_TARGET_UINT *v_char16; /*          char16_t character string      */
+    const TYPE_TARGET_ULONG *v_char32;/*          char32_t character string      */
     TYPE_TARGET_UINT v_uint;          /* 2 bytes: unsigned int const value       */
     TYPE_TARGET_LONG v_long;          /* 4 bytes: long constant value            */
     TYPE_TARGET_ULONG v_ulong;        /* 4 bytes: unsigned long constant value   */
@@ -271,12 +274,13 @@ typedef struct sym_link
     unsigned smallc:1;              /* Parameters on stack are passed in reverse order */
     unsigned z88dk_fastcall:1;      /* For the z80-related ports: Function has a single paramter of at most 32 bits that is passed in dehl */
     unsigned z88dk_callee:1;        /* Stack pointer adjustment for parameters passed on the stack is done by the callee */
-    unsigned intno;                 /* 1=Interrupt service routine          */
+    unsigned intno;                 /* Number of interrupt for interrupt service routine */
     short regbank;                  /* register bank 2b used                */
     unsigned builtin;               /* is a builtin function                */
     unsigned javaNative;            /* is a JavaNative Function (TININative ONLY) */
     unsigned overlay;               /* force parameters & locals into overlay segment */
     unsigned hasStackParms;         /* function has parameters on stack     */
+    bool preserved_regs[9];         /* Registers preserved by the function - may be an underestimate */
   } funcAttrs;
 
   struct sym_link *next;            /* next element on the chain  */
@@ -288,7 +292,7 @@ typedef struct symbol
   char name[SDCC_SYMNAME_MAX + 1];  /* Input Variable Name     */
   char rname[SDCC_NAME_MAX + 1];    /* internal name           */
 
-  short level;                      /* declaration lev,fld offset */
+  long level;                       /* declaration lev,fld offset */
   short block;                      /* sequential block # of definition */
   int seqPoint;                     /* sequence point defined or, if unbound, used */
   int key;
@@ -334,6 +338,7 @@ typedef struct symbol
   unsigned spildir:1;               /* spilt in direct space */
   unsigned ptrreg:1;                /* this symbol assigned to a ptr reg */
   unsigned noSpilLoc:1;             /* cannot be assigned a spil location */
+  unsigned div_flag_safe:1;         /* we know this function is safe to call with undocumented stm8 flag bit 6 set*/
   unsigned isstrlit;                /* is a string literal and it's usage count  */
   unsigned accuse;                  /* can be left in the accumulator
                                        On the Z80 accuse is divided into
@@ -363,7 +368,7 @@ typedef struct symbol
   }
   usl;
   signed char bitVar;               /* if bitVar != 0: this is a bit variable, bitVar is the size in bits */
-  char bitUnnamed:1;                /* unnamed bit variable */
+  unsigned bitUnnamed:1;            /* unnamed bit variable */
   unsigned offset;                  /* offset from top if struct */
 
   int lineDef;                      /* defined line number        */
@@ -476,7 +481,8 @@ extern sym_link *validateLink (sym_link * l,
 #define SPEC_CVAL(x) validateLink(x, "SPEC_CVAL", #x, SPECIFIER, __FILE__, __LINE__)->select.s.const_val
 #define SPEC_BSTR(x) validateLink(x, "SPEC_BSTR", #x, SPECIFIER, __FILE__, __LINE__)->select.s._bitStart
 #define SPEC_BLEN(x) validateLink(x, "SPEC_BLEN", #x, SPECIFIER, __FILE__, __LINE__)->select.s._bitLength
-#define SPEC_BUNNAMED(x) validateLink(x, "SPEC_BLEN", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_bitUnnamed
+#define SPEC_BUNNAMED(x) validateLink(x, "SPEC_BUNNAMED", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_bitUnnamed
+#define SPEC_NEEDSPAR(x) validateLink(x, "SPEC_NEEDSPAR", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_needspar
 
 /* Sleaze: SPEC_ISR_SAVED_BANKS is only used on
  * function type symbols, which obviously cannot
@@ -586,9 +592,6 @@ extern symbol *fsdiv;
 extern symbol *fseq;
 extern symbol *fsneq;
 extern symbol *fslt;
-extern symbol *fslteq;
-extern symbol *fsgt;
-extern symbol *fsgteq;
 
 extern symbol *fps16x16_add;
 extern symbol *fps16x16_sub;
@@ -603,6 +606,8 @@ extern symbol *fps16x16_gteq;
 
 /* Dims: mul/div/mod, BYTE/WORD/DWORD/QWORD, SIGNED/UNSIGNED/BOTH */
 extern symbol *muldiv[3][4][4];
+/* 16 x 16 -> 32 multiplication SIGNED/UNSIGNED */
+extern symbol *muls16tos32[2];
 /* Dims: BYTE/WORD/DWORD/QWORD SIGNED/UNSIGNED */
 extern sym_link *multypes[4][2];
 /* Dims: to/from float, BYTE/WORD/DWORD/QWORD, SIGNED/USIGNED */
@@ -639,7 +644,7 @@ typedef enum
 
 /* forward definitions for the symbol table related functions */
 void initSymt ();
-symbol *newSymbol (const char *, int);
+symbol *newSymbol (const char *, long);
 sym_link *newLink (SYM_LINK_CLASS);
 sym_link *newFloatLink ();
 structdef *newStruct (const char *);
@@ -649,14 +654,14 @@ sym_link *mergeSpec (sym_link *, sym_link *, const char *name);
 sym_link *mergeDeclSpec (sym_link *, sym_link *, const char *name);
 symbol *reverseSyms (symbol *);
 sym_link *reverseLink (sym_link *);
-symbol *copySymbol (symbol *);
-symbol *copySymbolChain (symbol *);
+symbol *copySymbol (const symbol *);
+symbol *copySymbolChain (const symbol *);
 void printSymChain (symbol *, int);
 void printStruct (structdef *, int);
-char *genSymName (int);
+char *genSymName (long);
 sym_link *getSpec (sym_link *);
 int compStructSize (int, structdef *);
-sym_link *copyLinkChain (sym_link *);
+sym_link *copyLinkChain (const sym_link *);
 int checkDecl (symbol *, int);
 void checkBasic (sym_link *, sym_link *);
 value *checkPointerIval (sym_link *, value *);
@@ -671,10 +676,10 @@ sym_link *newLongLink ();
 sym_link *newBoolLink ();
 sym_link *newVoidLink ();
 int compareType (sym_link *, sym_link *);
-int compareTypeExact (sym_link *, sym_link *, int);
+int compareTypeExact (sym_link *, sym_link *, long);
 int compareTypeInexact (sym_link *, sym_link *);
 int checkFunction (symbol *, symbol *);
-void cleanUpLevel (bucket **, int);
+void cleanUpLevel (bucket **, long);
 void cleanUpBlock (bucket **, int);
 symbol *getAddrspace (sym_link *type);
 int funcInChain (sym_link *);
@@ -695,11 +700,11 @@ void pointerTypes (sym_link *, sym_link *);
 void cdbStructBlock (int);
 void initHashT ();
 bucket *newBucket ();
-void addSym (bucket **, void *, char *, int, int, int checkType);
+void addSym (bucket **, void *, char *, long, int, int checkType);
 void deleteSym (bucket **, void *, const char *);
 void *findSym (bucket **, void *, const char *);
 void *findSymWithLevel (bucket **, struct symbol *);
-void *findSymWithBlock (bucket **, struct symbol *, int);
+void *findSymWithBlock (bucket **, struct symbol *, int, long);
 void changePointer (sym_link * p);
 void checkTypeSanity (sym_link * etype, const char *name);
 sym_link *typeFromStr (const char *);
