@@ -678,15 +678,6 @@ getBuildNumber (void)
   return (SDCC_BUILD_NUMBER);
 }
 
-/*-----------------------------------------------------------------*/
-/* getBuildDate - return build date                                */
-/*-----------------------------------------------------------------*/
-const char *
-getBuildDate (void)
-{
-  return __DATE__;
-}
-
 /*-----------------------------------------------------------------*/
 /* getBuildEnvironment - return environment used to build SDCC     */
 /*-----------------------------------------------------------------*/
@@ -734,7 +725,6 @@ getBuildEnvironment (void)
 #endif
 }
 
-#if defined(HAVE_VSNPRINTF) || defined(HAVE_VSPRINTF)
 size_t
 SDCCsnprintf (char *dst, size_t n, const char *fmt, ...)
 {
@@ -743,12 +733,7 @@ SDCCsnprintf (char *dst, size_t n, const char *fmt, ...)
 
   va_start (args, fmt);
 
-#if defined(HAVE_VSNPRINTF)
   len = vsnprintf (dst, n, fmt, args);
-#else
-  vsprintf (dst, fmt, args);
-  len = strlen (dst) + 1;
-#endif
 
   va_end (args);
 
@@ -763,7 +748,6 @@ SDCCsnprintf (char *dst, size_t n, const char *fmt, ...)
 
   return len;
 }
-#endif
 
 /** Pragma tokenizer
  */
@@ -832,7 +816,7 @@ free_pragma_token (struct pragma_token_s *token)
     /param src Pointer to 'x' from start of hex character value
 */
 
-unsigned char
+unsigned long int
 hexEscape (const char **src)
 {
   char *s;
@@ -847,16 +831,10 @@ hexEscape (const char **src)
       /* no valid hex found */
       werror (E_INVALID_HEX);
     }
-  else
-    {
-      if (value > 255)
-        {
-          werror (W_ESC_SEQ_OOR_FOR_CHAR);
-        }
-    }
+
   *src = s;
 
-  return (unsigned char) value;
+  return value;
 }
 
 /*------------------------------------------------------------------*/
@@ -865,16 +843,16 @@ hexEscape (const char **src)
 /* adjust src to point at the last proccesed char                   */
 /*------------------------------------------------------------------*/
 
-unsigned char
+unsigned long int
 universalEscape (const char **str, unsigned int n)
 {
   unsigned int digits;
-  unsigned value = 0;
+  unsigned long value = 0;
   const char *s = *str;
 
-  if (!options.std_c99)
+  if (!options.std_c95)
     {
-      werror (W_UNIVERSAL_C99);
+      werror (W_UNIVERSAL_C95);
     }
 
   ++*str;                       /* Skip over the 'u'  or 'U' */
@@ -888,20 +866,16 @@ universalEscape (const char **str, unsigned int n)
         }
       else if ((**str | 0x20) >= 'a' && (**str | 0x20) <= 'f')
         {
-          value = (value << 4) + (**str - ('a' + 10));
+          value = (value << 4) + (**str - 'a' + 10);
           ++*str;
         }
       else
           break;
     }
-  if (digits != n)
+  if (digits != n || value < 0x00a0 && value != 0x0024 && value != 0x0040 && value != 0x0060 || value >= 0xd800 && 0xdfff >= value)
     {
       werror (E_INVALID_UNIVERSAL, s);
     }
-  else if (value > 255)
-    {
-      werror (W_ESC_SEQ_OOR_FOR_CHAR);
-    }
 
   return value;
 }
@@ -912,7 +886,7 @@ universalEscape (const char **str, unsigned int n)
 /* adjust src to point at the last proccesed char                   */
 /*------------------------------------------------------------------*/
 
-unsigned char
+unsigned long int
 octalEscape (const char **str)
 {
   int digits;
@@ -930,13 +904,7 @@ octalEscape (const char **str)
           break;
         }
     }
-  if (digits)
-    {
-      if (value > 255 /* || (**str>='0' && **str<='7') */ )
-        {
-          werror (W_ESC_SEQ_OOR_FOR_CHAR);
-        }
-    }
+
   return value;
 }
 
@@ -954,8 +922,8 @@ octalEscape (const char **str)
 const char *
 copyStr (const char *src, size_t *size)
 {
- const char *begin = NULL;
- struct dbuf_s dbuf;
+  const char *begin = NULL;
+  struct dbuf_s dbuf;
 
   dbuf_init(&dbuf, 128);
 
@@ -973,7 +941,8 @@ copyStr (const char *src, size_t *size)
         }
       else if (*src == '\\')
         {
-          int c;
+          unsigned long int c;
+          bool universal = FALSE;
 
           if (begin)
             {
@@ -1031,11 +1000,13 @@ copyStr (const char *src, size_t *size)
 
             case 'u':
               c = universalEscape (&src, 4);
+              universal = TRUE;
               --src;
               break;
 
             case 'U':
               c = universalEscape (&src, 8);
+              universal = TRUE;
               --src;
               break;
 
@@ -1047,7 +1018,36 @@ copyStr (const char *src, size_t *size)
               c = *src;
               break;
             }
-          dbuf_append_char (&dbuf, c);
+          if (universal) // Encode one utf-32 character to utf-8
+            {
+              char s[5] = "\0\0\0\0";
+              if (c < 0x80)
+                s[0] = (char)c;
+              else if (c < 0x800)
+                {
+                  s[0] = (c >> 6) & 0x1f | 0xc0;
+                  s[1] = (c >> 0) & 0x3f | 0x80;
+                }
+              else if (c < 0x10000)
+                {
+                  s[0] = (c >> 12) & 0x0f | 0xe0;
+                  s[1] = (c >> 6) & 0x3f  | 0x80;
+                  s[2] = (c >> 0) & 0x3f  | 0x80;
+                }
+              else if (c < 0x110000)
+                {
+                  s[0] = (c >> 18) & 0x07 | 0xf0;
+                  s[1] = (c >> 12) & 0x3f | 0x80;
+                  s[2] = (c >> 6) & 0x3f  | 0x80;
+                  s[3] = (c >> 0) & 0x3f  | 0x80;
+                }
+              else
+                wassert (0);
+              dbuf_append_str (&dbuf, s);
+            }
+          else
+            dbuf_append_char (&dbuf, (char)c);
+
           ++src;
         }
       else
@@ -1128,3 +1128,46 @@ char *setPrefixSuffix(const char *arg)
 
   return cmd;
 }
+
+char *formatInlineAsm (char *asmStr)
+{
+  char *p, *q;
+
+  if (!asmStr)
+    return NULL;
+  else
+    q = asmStr;
+
+  for (;;)
+    {
+      // omit leading space or tab
+      while (*q == '\t' || *q == ' ')
+        q++;
+      // then record the head of current line
+      p = q;
+      // search for CL or reach the end
+      while (*q != '\n' && *q != '\r' && *q != 0)
+        q++;
+      // omit more CL characters
+      while (*q == '\n' || *q == '\r')
+        q++;
+      // replace the first with tab
+      while (p != q)
+        if (*p == '\t') // '\t' appears first then no need to do
+          {
+            break;
+          }
+        else if (*p == ' ') // find the first space then replace it with tab
+          {
+            *p = '\t';
+            break;
+          }
+        else // go on to search
+          {
+            p++;
+          }
+      // check if end
+      if (*q == 0)
+        return asmStr;
+    }
+}
