@@ -765,66 +765,6 @@ createRegMask (eBBlock ** ebbs, int count)
     }
 }
 
-/*-----------------------------------------------------------------*/
-/* rematStr - returns the rematerialized string for a remat var    */
-/*-----------------------------------------------------------------*/
-static char *
-rematStr (symbol * sym, int offset)
-{
-  iCode *ic = sym->rematiCode;
-
-  while (1)
-    {
-      /* if plus adjust offset to right hand side */
-      if (ic->op == '+')
-        {
-          offset += (int) operandLitValue (IC_RIGHT (ic));
-          ic = OP_SYMBOL (IC_LEFT (ic))->rematiCode;
-          continue;
-        }
-
-      /* if minus adjust offset to right hand side */
-      if (ic->op == '-')
-        {
-          offset -= (int) operandLitValue (IC_RIGHT (ic));
-          ic = OP_SYMBOL (IC_LEFT (ic))->rematiCode;
-          continue;
-        }
-
-      /* cast then continue */
-      if (IS_CAST_ICODE(ic))
-        {
-          ic = OP_SYMBOL (IC_RIGHT (ic))->rematiCode;
-          continue;
-        }
-      /* we reached the end */
-      break;
-    }
-
-  if (ic->op == ADDRESS_OF)
-    {
-      if (offset)
-        {
-          SNPRINTF (buffer, sizeof(buffer),
-                    "(%s %c 0x%04x)",
-                    OP_SYMBOL (IC_LEFT (ic))->rname,
-                    offset >= 0 ? '+' : '-',
-                    abs (offset) & 0xffff);
-        }
-      else
-        {
-          strncpyz (buffer, OP_SYMBOL (IC_LEFT (ic))->rname, sizeof(buffer));
-        }
-    }
-  else if (ic->op == '=')
-    {
-      offset += (int) operandLitValue (IC_RIGHT (ic));
-      SNPRINTF (buffer, sizeof(buffer),
-                "0x%04x",
-                offset & 0xffff);
-    }
-  return buffer;
-}
 
 /*-----------------------------------------------------------------*/
 /* regTypeNum - computes the type & number of registers required   */
@@ -834,7 +774,6 @@ regTypeNum (eBBlock *ebbs)
 {
   symbol *sym;
   int k;
-  iCode *ic;
 
   /* for each live range do */
   for (sym = hTabFirstItem (liveRanges, &k); sym;
@@ -860,27 +799,6 @@ regTypeNum (eBBlock *ebbs)
               continue;
             }
 
-          /* if the symbol has only one definition &
-             that definition is a get_pointer */
-          if (bitVectnBitsOn (sym->defs) == 1 &&
-              (ic = hTabItemWithKey (iCodehTab, bitVectFirstBit (sym->defs))) &&
-              POINTER_GET (ic) &&
-              !IS_BITVAR (sym->etype) &&
-              (aggrToPtrDclType (operandType (IC_LEFT (ic)), FALSE) == POINTER))
-            {
-              if (ptrPseudoSymSafe (sym, ic))
-                {
-                  int rightval = 0;
-                  if (IC_RIGHT (ic))
-                    rightval = (int)operandLitValue (IC_RIGHT (ic));
-                  ptrPseudoSymConvert (sym, ic, rematStr (OP_SYMBOL (IC_LEFT (ic)), rightval));
-                  continue;
-                }
-
-              /* if in data space or idata space then try to
-                 allocate pointer register */
-            }
-
           /* if not then we require registers */
           sym->nRegs = ((IS_AGGREGATE (sym->type) || sym->isptr) ?
                         getSize (sym->type = aggrToPtr (sym->type, FALSE)) :
@@ -1918,21 +1836,12 @@ packPointerOp (iCode * ic, eBBlock ** ebpp)
     }
 
   /* Put the remaining operand on the right and convert to assignment     */
-  /* or cast (Sometimes the operands to the addition are different sizes, */
-  /* so there is an implicit cast. If so, need to make it explicit so     */
-  /* that all the bytes of the pointer are defined. */
   if (IS_SYMOP (offsetOp))
     bitVectUnSetBit (OP_USES (offsetOp), dic->key);
   IC_RIGHT (dic) = nonOffsetOp;
   IC_LEFT (dic) = NULL;
   SET_ISADDR (IC_RESULT (dic), 0);
-  if (getSize (operandType (pointer)) == getSize (operandType (nonOffsetOp)))
-    dic->op = '=';
-  else
-    {
-      dic->op = CAST;
-      IC_LEFT (dic) = operandFromLink (operandType (pointer));
-    }
+  dic->op = '=';
 }
 
 /*-----------------------------------------------------------------*/
@@ -1993,7 +1902,8 @@ packRegisters (eBBlock ** ebpp, int blockno)
           !POINTER_SET (ic) &&
           IS_SYMOP (IC_RIGHT (ic)) &&
           OP_SYMBOL (IC_RIGHT (ic))->remat &&
-          bitVectnBitsOn (OP_SYMBOL (IC_RESULT (ic))->defs) <= 1)
+          bitVectnBitsOn (OP_SYMBOL (IC_RESULT (ic))->defs) <= 1 &&
+          !OP_SYMBOL (IC_RESULT (ic))->_isparm)
         {
           OP_SYMBOL (IC_RESULT (ic))->remat = OP_SYMBOL (IC_RIGHT (ic))->remat;
           OP_SYMBOL (IC_RESULT (ic))->rematiCode = OP_SYMBOL (IC_RIGHT (ic))->rematiCode;
@@ -2004,7 +1914,8 @@ packRegisters (eBBlock ** ebpp, int blockno)
       if (ic->op == CAST &&
           IS_SYMOP(IC_RIGHT(ic)) &&
           OP_SYMBOL(IC_RIGHT(ic))->remat &&
-          bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1)
+          bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 &&
+          !OP_SYMBOL (IC_RESULT (ic))->_isparm)
         {
           sym_link *to_type = operandType(IC_LEFT(ic));
           sym_link *from_type = operandType(IC_RIGHT(ic));
