@@ -163,18 +163,18 @@ _mcs51_finaliseOptions (void)
     case MODEL_SMALL:
       port->mem.default_local_map = data;
       port->mem.default_globl_map = data;
-      port->s.gptr_size = 3;
+      port->s.ptr_size = 3;
       break;
     case MODEL_MEDIUM:
       port->mem.default_local_map = pdata;
       port->mem.default_globl_map = pdata;
-      port->s.gptr_size = 3;
+      port->s.ptr_size = 3;
       break;
     case MODEL_LARGE:
     case MODEL_HUGE:
       port->mem.default_local_map = xdata;
       port->mem.default_globl_map = xdata;
-      port->s.gptr_size = 3;
+      port->s.ptr_size = 3;
       break;
     default:
       port->mem.default_local_map = data;
@@ -347,6 +347,12 @@ oclsExpense (struct memmap *oclass)
   return 0;
 }
 
+static bool
+_hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
+{
+  return getSize (left) == 1 && getSize (right) == 1;
+}
+
 static int
 instructionSize(char *inst, char *op1, char *op2)
 {
@@ -826,8 +832,8 @@ PORT mcs51_port =
     NULL,
     NULL,
   },
-  /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
-  { 1, 2, 2, 4, 8, 1, 2, 3, 1, 4, 4 },
+  /* Sizes: char, short, int, long, long long, near ptr, far ptr, gptr, func ptr, banked func ptr, bit, float */
+  { 1, 2, 2, 4, 8, 1, 2, 3, 2, 3, 1, 4 },
   /* tags for generic pointers */
   { 0x00, 0x40, 0x60, 0x80 },   /* far, near, xstack, code */
   {
@@ -864,10 +870,10 @@ PORT mcs51_port =
     4,          /* isr_overhead */
     1,          /* call_overhead (2 for return address - 1 for pre-incrementing push */
     1,          /* reent_overhead */
-    1           /* banked_overhead (switch between code banks) */
+    1,          /* banked_overhead (switch between code banks) */
+    0           /* sp points directly at last item pushed */
   },
-  /* mcs51 has an 8 bit mul */
-  { 1, -1 },
+  { -1, FALSE },
   { mcs51_emitDebuggerSymbol },
   {
     256,        /* maxCount */
@@ -886,6 +892,7 @@ PORT mcs51_port =
   _mcs51_setDefaultOptions,
   mcs51_assignRegisters,
   _mcs51_getRegName,
+  0,
   _mcs51_rtrackUpdate,
   _mcs51_keywords,
   _mcs51_genAssemblerPreamble,
@@ -897,7 +904,7 @@ PORT mcs51_port =
   _mcs51_regparm,
   NULL,                         /* process_pragma */
   NULL,                         /* getMangledFunctionName */
-  NULL,                         /* hasNativeMulFor */
+  _hasNativeMulFor,             /* hasNativeMulFor */
   hasExtBitOp,                  /* hasExtBitOp */
   oclsExpense,                  /* oclsExpense */
   FALSE,                        /* use_dw_for_init */
