@@ -25,6 +25,7 @@
 #include <ctype.h>
 #include "common.h"
 #include "ralloc.h"
+#include "gen.h"
 
 #define D(x) x
 #define DEADMOVEERROR() do {werror(E_INTERNAL_ERROR, __FILE__, __LINE__, "error in deadmove");} while(0)
@@ -192,8 +193,8 @@ findLabel (const lineNode *pl)
   /* 3. search lineNode with label definition and return it */
   for (cpl = _G.head; cpl; cpl = cpl->next)
     {
-      if (   cpl->isLabel
-          && strcmp (p, cpl->line) == 0)
+      if (cpl->isLabel
+          && strncmp (p, cpl->line, strlen(p)) == 0)
         {
           return cpl;
         }
@@ -234,12 +235,13 @@ termScanAtFunc (const lineNode *pl, int rIdx)
   // most notably :  (*(void (*)()) 0) ();  see bug 1749275
   if (IS_VALOP (IC_LEFT (pl->ic)))
     return (options.model == MODEL_HUGE) && banked_reg ? S4O_ABORT : options.all_callee_saves ? S4O_CONTINUE : S4O_TERM;
-
   ftype = OP_SYM_TYPE(IC_LEFT(pl->ic));
   if (IS_FUNCPTR (ftype))
     ftype = ftype->next;
   if (IFFUNC_ISBANKEDCALL(ftype) && banked_reg)
     return S4O_ABORT;
+  if (FUNC_ARGS (ftype) && getSize (FUNC_ARGS (ftype)->type) > 4)
+    return S4O_ABORT;
   if (FUNC_CALLEESAVES(ftype))
     return S4O_CONTINUE;
   if (FUNC_ISNAKED(ftype))
@@ -260,7 +262,7 @@ termScanAtFunc (const lineNode *pl, int rIdx)
 /*       points to a register (e.g. "ar0"). scan4op() tests for    */
 /*       read or write operations with this register               */
 /*    const char *untilOp                                          */
-/*       points to NULL or a opcode (e.g. "push").                 */
+/*       points to NULL or an opcode (e.g. "push").                */
 /*       scan4op() returns if it hits this opcode.                 */
 /*    lineNode **plCond                                            */
 /*       If a conditional branch is met plCond points to the       */
@@ -505,12 +507,19 @@ scan4op (lineNode **pl, const char *pReg, const char *untilOp,
                   }
 
                 /* it's a normal function return */
-                if (IS_SYMOP (IC_LEFT ((*pl)->ic)) &&
-                    IS_FUNC (OP_SYM_TYPE(IC_LEFT ((*pl)->ic))) &&
-                    FUNC_CALLEESAVES (OP_SYM_TYPE(IC_LEFT ((*pl)->ic))))
-                  return S4O_ABORT;
-                else
-                  return S4O_TERM;
+                if (!((*pl)->ic))
+                  return S4O_ABORT; /* but no ic? */
+                if (!currFunc->type)
+                  return S4O_ABORT;  /* not a function? */
+                if (FUNC_CALLEESAVES (currFunc->type))
+                  return S4O_ABORT; /* returning from callee saves function */
+                if (getSize(currFunc->etype) > 4)
+                  {
+                    for (unsigned i = 0; i < getSize(currFunc->etype); i++)
+                      if (strstr (pReg, fReturn8051[i]))
+                        return S4O_ABORT; /* return value is partially in r4-r7 */
+                  }
+                return S4O_TERM;
               }
             break;
           case 's':
@@ -563,6 +572,7 @@ doPushScan (lineNode **pl, const char *pReg)
             /* already checked */
             return TRUE;
           case S4O_CONDJMP:
+#if 0
             /* two possible destinations: recurse */
               {
                 lineNode *pushPl2 = plConditional;
@@ -572,6 +582,10 @@ doPushScan (lineNode **pl, const char *pReg)
                 pushPl = pushPl2;
               }
             continue;
+#else
+            /* two possible destinations: give up */
+            return FALSE;
+#endif   
           default:
             return FALSE;
         }
