@@ -29,10 +29,6 @@
       Made everything static
 -------------------------------------------------------------------------*/
 
-#if defined(_MSC_VER) && (_MSC_VER < 1300)
-#define __FUNCTION__    __FILE__
-#endif
-
 #include "common.h"
 #include "newalloc.h"
 //#include "SDCCglobl.h"
@@ -565,7 +561,7 @@ static void genAddLit (iCode *ic, int lit)
 
                       emitpcode(POC_MOVLW, popGetLit(lit & 0xff));
                       emitpcode(POC_ADDFW,  popGet(AOP(left),offset));
-                      emitpcode(POC_MOVWF, popGet(AOP(left),offset));
+                      emitpcode(POC_MOVWF, popGet(AOP(result),offset));
 
                       clear_carry = 0;
                     }
