@@ -80,9 +80,11 @@ emitPseudoStack(struct dbuf_s *oBuf, struct dbuf_s *oBufExt)
     if (!pic14_options.isLibrarySource)
     {
         dbuf_printf (oBuf, "\n");
-        dbuf_printf (oBuf, "\tglobal PSAVE\n");
-        dbuf_printf (oBuf, "\tglobal SSAVE\n");
-        dbuf_printf (oBuf, "\tglobal WSAVE\n");
+        if (!pic14_getPIC()->isEnhancedCore) {
+            dbuf_printf (oBuf, "\tglobal PSAVE\n");
+            dbuf_printf (oBuf, "\tglobal SSAVE\n");
+            dbuf_printf (oBuf, "\tglobal WSAVE\n");
+        }
         for (i = size - 4; i >= 0; i--) {
             dbuf_printf (oBuf, "\tglobal STK%02d\n", i);
         } // for i
@@ -97,9 +99,11 @@ emitPseudoStack(struct dbuf_s *oBuf, struct dbuf_s *oBufExt)
             // for devices with at least two banks, require a sharebank section
             dbuf_printf (oBuf, "sharebank udata_shr\n");
         }
-        dbuf_printf (oBuf, "PSAVE\tres 1\n");
-        dbuf_printf (oBuf, "SSAVE\tres 1\n");
-        dbuf_printf (oBuf, "WSAVE\tres 1\n"); // WSAVE *must* be in sharebank (IRQ handlers)
+        if (!pic14_getPIC()->isEnhancedCore) {
+            dbuf_printf (oBuf, "PSAVE\tres 1\n");
+            dbuf_printf (oBuf, "SSAVE\tres 1\n");
+            dbuf_printf (oBuf, "WSAVE\tres 1\n"); // WSAVE *must* be in sharebank (IRQ handlers)
+        }
         /* fill rest of sharebank with stack STKxx .. STK00 */
         for (i = size - 4; i >= 0; i--) {
             dbuf_printf (oBuf, "STK%02d\tres 1\n", i);
@@ -108,9 +112,11 @@ emitPseudoStack(struct dbuf_s *oBuf, struct dbuf_s *oBufExt)
         /* declare STKxx as extern for all files
          * except the one containing main() */
         dbuf_printf (oBufExt, "\n");
-        dbuf_printf (oBufExt, "\textern PSAVE\n");
-        dbuf_printf (oBufExt, "\textern SSAVE\n");
-        dbuf_printf (oBufExt, "\textern WSAVE\n");
+        if (!pic14_getPIC()->isEnhancedCore) {
+            dbuf_printf (oBufExt, "\textern PSAVE\n");
+            dbuf_printf (oBufExt, "\textern SSAVE\n");
+            dbuf_printf (oBufExt, "\textern WSAVE\n");
+        }
         for (i = size - 4; i >= 0; i--) {
             char buffer[128];
             SNPRINTF(&buffer[0], 127, "STK%02d", i);
@@ -146,8 +152,7 @@ pic14_constructAbsMap (struct dbuf_s *oBuf, struct dbuf_s *gloBuf)
 
   for (i=0; maps[i] != NULL; i++)
   {
-    for (sym = (symbol *)setFirstItem (maps[i]->syms);
-        sym; sym = setNextItem (maps[i]->syms))
+    for (sym = (symbol *)setFirstItem (maps[i]->syms); sym; sym = setNextItem (maps[i]->syms))
     {
       if (IS_DEFINED_HERE(sym) && SPEC_ABSA(sym->etype))
       {
@@ -203,9 +208,7 @@ pic14_constructAbsMap (struct dbuf_s *oBuf, struct dbuf_s *gloBuf)
 
       dbuf_printf (oBuf, "UD_abs_%s_%x\tudata_ovr\t0x%04x\n",
           moduleName, addr, addr);
-      for (sym = setFirstItem (aliases); sym;
-          sym = setNextItem (aliases))
-      {
+      for (sym = setFirstItem (aliases); sym; sym = setNextItem (aliases)) {
         if (getSize(sym->type) > size) {
           size = getSize(sym->type);
         }
@@ -277,7 +280,6 @@ pic14initialComments (FILE * afile)
         initialComments (afile);
         fprintf (afile, "; PIC port for the 14-bit core\n");
         fprintf (afile, "%s", iComments2);
-
 }
 
 int
@@ -285,18 +287,18 @@ pic14_stringInSet(const char *str, set **world, int autoAdd)
 {
   char *s;
 
-  if (!str) return 1;
+  if (!str) return TRUE;
   assert(world);
 
   for (s = setFirstItem(*world); s; s = setNextItem(*world))
   {
     /* found in set */
-    if (0 == strcmp(s, str)) return 1;
+    if (0 == strcmp(s, str)) return TRUE;
   }
 
   /* not found */
   if (autoAdd) addSet(world, Safe_strdup(str));
-  return 0;
+  return FALSE;
 }
 
 static void
@@ -305,8 +307,11 @@ pic14printLocals (struct dbuf_s *oBuf)
     set *allregs[6] = { dynAllocRegs/*, dynStackRegs, dynProcessorRegs*/,
         dynDirectRegs, dynDirectBitRegs/*, dynInternalRegs */ };
     reg_info *reg;
-    int i, is_first = 1;
+    int i, is_first = TRUE;
     static unsigned sectionNr = 0;
+    const char *split_locals;
+
+    split_locals = getenv("SDCC_PIC14_SPLIT_LOCALS");
 
     /* emit all registers from all possible sets */
     for (i = 0; i < 6; i++) {
@@ -316,14 +321,14 @@ pic14printLocals (struct dbuf_s *oBuf)
             if (reg->isEmitted) continue;
 
             if (reg->wasUsed && !reg->isExtern) {
-                if (!pic14_stringInSet(reg->name, &emitted, 1)) {
+                if (!pic14_stringInSet(reg->name, &emitted, TRUE)) {
                     if (reg->isFixed) {
                         // Should not happen, really...
                         assert ( !"Compiler-assigned variables should not be pinned... This is a bug." );
                         dbuf_printf(oBuf, "UDL_%s_%u\tudata\t0x%04X\n%s\tres\t%d\n",
                                 moduleName, sectionNr++, reg->address, reg->name, reg->size);
                     } else {
-                        if (getenv("SDCC_PIC14_SPLIT_LOCALS")) {
+                        if (split_locals != NULL) {
                             // assign each local register into its own section
                             dbuf_printf(oBuf, "UDL_%s_%u\tudata\n%s\tres\t%d\n",
                                     moduleName, sectionNr++, reg->name, reg->size);
@@ -332,14 +337,14 @@ pic14printLocals (struct dbuf_s *oBuf)
                             // This should greatly improve BANKSEL generation...
                             if (is_first) {
                                 dbuf_printf(oBuf, "UDL_%s_%u\tudata\n", moduleName, sectionNr++);
-                                is_first = 0;
+                                is_first = FALSE;
                             }
                             dbuf_printf(oBuf, "%s\tres\t%d\n", reg->name, reg->size);
                         }
                     }
                 }
             }
-            reg->isEmitted = 1;
+            reg->isEmitted = TRUE;
         } // for
     } // for
 }
@@ -422,7 +427,7 @@ pic14emitOverlay (struct dbuf_s * aBuf)
                                         /* symbol is local */
                                         dbuf_printf (aBuf, "L%s_",
                                         (sym->localof ? sym->localof->name : "-null-"));
-                                dbuf_printf (aBuf, "%s_%d_%d", sym->name, sym->level, sym->block);
+                                dbuf_printf (aBuf, "%s_%ld_%d", sym->name, sym->level, sym->block);
                         }
 
                         /* if is has an absolute address then generate
@@ -457,17 +462,16 @@ pic14_emitInterruptHandler (FILE * asmFile)
 {
         if (pic14_hasInterrupt)
         {
-
                 fprintf (asmFile, "%s", iComments2);
                 fprintf (asmFile, "; interrupt and initialization code\n");
                 fprintf (asmFile, "%s", iComments2);
                 // Note - for mplink may have to enlarge section vectors in .lnk file
                 // Note: Do NOT name this code_interrupt to avoid nameclashes with
                 //       source files's code segment (interrupt.c -> code_interrupt)
-                fprintf (asmFile, "c_interrupt\t%s\t0x4\n", CODE_NAME);
+                fprintf (asmFile, "c_interrupt\t%s\t0x0004\n", CODE_NAME);
 
                 /* interrupt service routine */
-                fprintf (asmFile, "__sdcc_interrupt\n");
+                fprintf (asmFile, "__sdcc_interrupt:\n");
                 copypCode(asmFile, 'I');
         }
 }
@@ -476,7 +480,7 @@ pic14_emitInterruptHandler (FILE * asmFile)
 /* glue - the final glue that hold the whole thing together        */
 /*-----------------------------------------------------------------*/
 void
-picglue ()
+picglue (void)
 {
         FILE *asmFile;
         struct dbuf_s ovrBuf;
@@ -496,7 +500,7 @@ picglue ()
         {
                 /* main missing -- import stack from main module */
                 //fprintf (stderr, "main() missing -- assuming we are NOT the main module\n");
-                pic14_options.isLibrarySource = 1;
+                pic14_options.isLibrarySource = TRUE;
         }
 
         /* At this point we've got all the code in the form of pCode structures */
@@ -538,12 +542,11 @@ picglue ()
 
         if ((noAssemble || options.c1mode) && fullDstFileName)
         {
-                sprintf (buffer, "%s", fullDstFileName);
+                SNPRINTF(buffer, sizeof(buffer), "%s", fullDstFileName);
         }
         else
         {
-                sprintf (buffer, "%s", dstFileName);
-                strcat (buffer, ".asm");
+                SNPRINTF(buffer, sizeof(buffer), "%s.asm", dstFileName);
         }
 
         if (!(asmFile = fopen (buffer, "w"))) {
@@ -588,7 +591,7 @@ picglue ()
         else
           dbuf_destroy(&vBuf);
 
-        /* create interupt ventor handler */
+        /* create interupt vector handler */
         pic14_emitInterruptHandler (asmFile);
 
         /* copy over code */
@@ -641,13 +644,13 @@ parseIvalAst (ast *node, int *inCodeSpace) {
         if (inCodeSpace && val->type
                 && (IS_FUNC(val->type) || IS_CODE(getSpec(val->type))))
         {
-            *inCodeSpace = 1;
+            *inCodeSpace = TRUE;
         }
         if (inCodeSpace && sym
                 && (IS_FUNC(sym->type)
                     || IS_CODE(getSpec(sym->type))))
         {
-            *inCodeSpace = 1;
+            *inCodeSpace = TRUE;
         }
 
         DEBUGprintf ("%s: AST_VALUE\n", __FUNCTION__);
@@ -718,17 +721,17 @@ static int
 emitIvalLabel(struct dbuf_s *oBuf, symbol *sym)
 {
     char *segname;
-    static int in_code = 0;
+    static int in_code = FALSE;
     static int sectionNr = 0;
 
     if (sym) {
         // code or data space?
         if (IS_CODE(getSpec(sym->type))) {
             segname = "code";
-            in_code = 1;
+            in_code = TRUE;
         } else {
             segname = "idata";
-            in_code  = 0;
+            in_code = FALSE;
         }
         dbuf_printf(oBuf, "\nID_%s_%d\t%s", moduleName, sectionNr++, segname);
         if (SPEC_ABSA(getSpec(sym->type))) {
@@ -752,7 +755,7 @@ emitIvals(struct dbuf_s *oBuf, symbol *sym, initList *list, long lit, int size)
     ast *node;
     operand *op;
     value *val = NULL;
-    int inCodeSpace = 0;
+    int inCodeSpace = FALSE;
     char *str = NULL;
     int in_code;
 
@@ -983,7 +986,8 @@ emitInitVal(struct dbuf_s *oBuf, symbol *topsym, sym_link *my_type, initList *li
         return;
     }
 
-    if (IS_CHAR(my_type) || IS_INT(my_type) || IS_LONG(my_type)) {
+    if (IS_CHAR(my_type) || IS_INT(my_type) || IS_LONG(my_type) ||
+	IS_BOOL(my_type)) {
         // integral type, 8, 16, or 32 bit
         DEBUGprintf ("(integral, %d byte) 0x%lx/%ld\n", size, list ? (long)list2int(list) : 0, list ? (long)list2int(list) : 0);
         emitIvals(oBuf, topsym, list, 0, size);
@@ -1072,7 +1076,7 @@ emitSymbolSet(set *s, int type)
 {
     symbol *sym;
     initList *list;
-    unsigned sectionNr = 0;
+    unsigned int sectionNr = 0;
 
     for (sym = setFirstItem(s); sym; sym = setNextItem(s)) {
 #if 0
@@ -1097,9 +1101,7 @@ emitSymbolSet(set *s, int type)
             continue;
         }
 
-        if (type != 0 || sym->cdef
-                || (!IS_STATIC(sym->etype)
-                    && IS_GLOBAL(sym)))
+        if (type != 0 || sym->cdef || (!IS_STATIC(sym->etype) && IS_GLOBAL(sym)))
         {
             // bail out for ___fsadd and friends
             if (sym->cdef && !sym->used) continue;
