@@ -358,8 +358,8 @@ PORT pic_port =
     _defaultRules
   },
   {
-    /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
-    1, 2, 2, 4, 8, 2, 2, 3, 1, 4, 4
+    /* Sizes: char, short, int, long, long long, near ptr, far ptr, gptr, func ptr, banked func ptr, bit, float */
+    1, 2, 2, 4, 8, 2, 2, 3, 2, 3, 1, 4
     /* TSD - I changed the size of gptr from 3 to 1. However, it should be
        2 so that we can accomodate the PIC's with 4 register banks (like the
        16f877)
@@ -396,11 +396,11 @@ PORT pic_port =
   },
   { NULL, NULL },
   {
-    +1, 1, 4, 1, 1, 0
+    +1, 1, 4, 1, 1, 0, 0
   },
     /* pic14 has an 8 bit mul */
   {
-    1, -1
+    -1, FALSE
   },
   {
     pic14_emitDebuggerSymbol
@@ -424,6 +424,7 @@ PORT pic_port =
   _pic14_setDefaultOptions,
   pic14_assignRegisters,
   _pic14_getRegName,
+  0,
   NULL,
   _pic14_keywords,
   _pic14_genAssemblerPreamble,
