@@ -30,6 +30,98 @@
 #include "dbuf_string.h"
 #include "peep.h"
 
+#define OPTION_CODE_SEG        "--codeseg"
+#define OPTION_CONST_SEG       "--constseg"
+#define OPTION_ELF             "--out-fmt-elf"
+
+extern DEBUGFILE dwarf2DebugFile;
+extern int dwarf2FinalizeFile(FILE *);
+
+static OPTION stm8_options[] = {
+  {0, OPTION_CODE_SEG,        &options.code_seg, "<name> use this name for the code segment", CLAT_STRING},
+  {0, OPTION_CONST_SEG,       &options.const_seg, "<name> use this name for the const segment", CLAT_STRING},
+  {0, OPTION_ELF,             NULL, "Output executable in ELF format"},
+  {0, NULL}
+};
+
+enum
+{
+  P_CODESEG = 1,
+  P_CONSTSEG,
+};
+
+static int
+stm8_do_pragma (int id, const char *name, const char *cp)
+{
+  struct pragma_token_s token;
+  int processed = 1, error = 0;
+
+  init_pragma_token (&token);
+
+  switch (id)
+    {
+      case P_CODESEG:
+      case P_CONSTSEG:
+        {
+          char *segname;
+
+          cp = get_pragma_token (cp, &token);
+          if (token.type == TOKEN_EOL)
+            {
+              error = 1;
+              break;
+            }
+          else
+            segname = Safe_strdup (get_pragma_string (&token));
+
+          cp = get_pragma_token (cp, &token);
+          if (token.type != TOKEN_EOL)
+            {
+              Safe_free (segname);
+              error = 1;
+              break;
+            }
+          else
+            {
+              if (id == P_CODESEG)
+                {
+                  if (options.code_seg)
+                    Safe_free (options.code_seg);
+                  options.code_seg = segname;
+                } 
+              else
+                {
+                  if (options.const_seg)
+                    Safe_free (options.const_seg);
+                  options.const_seg = segname;
+                }
+            }
+        }
+        break;
+      default:
+        processed = 0;
+        break;
+    }
+
+  if (error)
+    werror (W_BAD_PRAGMA_ARGUMENTS, name);
+
+  free_pragma_token (&token);
+  return processed;
+}
+
+static struct pragma_s stm8_pragma_tbl[] = {
+  {"codeseg", P_CODESEG, 0, stm8_do_pragma},
+  {"constseg", P_CONSTSEG, 0, stm8_do_pragma},
+  {NULL, 0, 0, NULL},
+};
+
+static int
+stm8_process_pragma (const char *s)
+{
+  return process_pragma_tbl (stm8_pragma_tbl, s);
+}
+
 static char stm8_defaultRules[] = {
 #include "peeph.rul"
   ""
@@ -45,10 +137,18 @@ static char *stm8_keywords[] = {
   NULL
 };
 
+static void
+stm8_genAssemblerEnd (FILE *of)
+{
+  if (options.out_fmt == 'E' && options.debug)
+    {
+      dwarf2FinalizeFile (of);
+    }
+}
+
 static void
 stm8_init (void)
 {
-  // fprintf(stderr, "stm8_init\n");
   asm_addTree (&asm_asxxxx_mapping);
 }
 
@@ -67,6 +167,12 @@ stm8_reg_parm (sym_link * l, bool reentrant)
 static bool
 stm8_parseOptions (int *pargc, char **argv, int *i)
 {
+  if (!strcmp (argv[*i], "--out-fmt-elf"))
+  {
+    options.out_fmt = 'E';
+    debugFile = &dwarf2DebugFile;
+    return TRUE;
+  }
   return FALSE;
 }
 
@@ -75,6 +181,13 @@ stm8_finaliseOptions (void)
 {
   port->mem.default_local_map = data;
   port->mem.default_globl_map = data;
+
+  if (options.model == MODEL_LARGE)
+    {
+      port->s.funcptr_size = 3;
+      port->stack.call_overhead = 3;
+      port->jumptableCost.maxCount = 0;
+    }
 }
 
 static void
@@ -88,6 +201,8 @@ stm8_setDefaultOptions (void)
   options.data_loc = 0x0001; /* We can't use the byte at address zero in C, since NULL pointers have special meaning */
   options.code_loc = 0x8000;
 
+  options.stack_loc = -1; /* Do not set the stack pointer in software- just use the device-specific reset value. */
+
   options.out_fmt = 'i';        /* Default output format is ihx */
 }
 
@@ -99,11 +214,30 @@ stm8_getRegName (const struct reg_info *reg)
   return "err";
 }
 
-void
-stm8_genInitStartup(FILE * of)
+static void
+stm8_genExtraArea (FILE *of, bool hasMain)
+{
+  fprintf (of, "\n; default segment ordering for linker\n");
+  tfprintf (of, "\t!area\n", HOME_NAME);
+  tfprintf (of, "\t!area\n", STATIC_NAME);
+  tfprintf (of, "\t!area\n", port->mem.post_static_name);
+  tfprintf (of, "\t!area\n", CONST_NAME);
+  tfprintf (of, "\t!area\n", "INITIALIZER");
+  tfprintf (of, "\t!area\n", CODE_NAME);
+  fprintf (of, "\n");
+}
+
+static void
+stm8_genInitStartup (FILE *of)
 {
   fprintf (of, "__sdcc_gs_init_startup:\n");
 
+  if (options.stack_loc >= 0)
+    {
+      fprintf (of, "\tldw\tx, #0x%04x\n", options.stack_loc);
+      fprintf (of, "\tldw\tsp, x\n");
+    }
+
   /* Init static & global variables */
   fprintf (of, "__sdcc_init_data:\n");
   fprintf (of, "; stm8_genXINIT() start\n");
@@ -129,32 +263,48 @@ stm8_genInitStartup(FILE * of)
   fprintf (of, "; stm8_genXINIT() end\n");
 }
 
+#define STM8_INTERRUPTS_COUNT 30
+
 int
 stm8_genIVT(struct dbuf_s * oBuf, symbol ** intTable, int intCount)
 {
-  #define STM8_INTERRUPTS_COUNT 30
   int i;
-  dbuf_tprintf (oBuf, "\tint s_GSINIT ;reset\n");
-  
-  if(interrupts[INTNO_TRAP])
-	dbuf_printf (oBuf, "\tint %s ;trap\n", interrupts[INTNO_TRAP]->rname);
-  else
-	dbuf_tprintf (oBuf, "\tint 0x0000 ;trap\n");
-	
-  for(i = 0; i < STM8_INTERRUPTS_COUNT; i++)
-  {
-      if (i < intCount && interrupts[i])
-        dbuf_printf (oBuf, "\tint %s ;int%d\n", interrupts[i]->rname, i);
-      else
-        dbuf_tprintf (oBuf, "\tint 0x0000 ;int%d\n", i); // int<n>
-  }
+  dbuf_tprintf (oBuf, "\tint s_GSINIT ; reset\n");
+
+  if(intCount > STM8_INTERRUPTS_COUNT)
+    {
+      werror(E_INT_BAD_INTNO, intCount - 1);
+      intCount = STM8_INTERRUPTS_COUNT;
+    }
+
+  if (interrupts[INTNO_TRAP] || intCount)
+    dbuf_printf (oBuf, "\tint %s ; trap\n", interrupts[INTNO_TRAP] ? interrupts[INTNO_TRAP]->rname : "0x000000");
+    
+  for (i = 0; i < intCount; i++)
+    dbuf_printf (oBuf, "\tint %s ; int%d\n", interrupts[i] ? interrupts[i]->rname : "0x000000", i);
+
   return TRUE;
 }
 
+/*----------------------------------------------------------------------*/
+/* stm8_dwarfRegNum - return the DWARF register number for a register.  */
+/*----------------------------------------------------------------------*/
+static int
+stm8_dwarfRegNum (const struct reg_info *reg)
+{
+  return reg->rIdx;
+}
+
 static bool
 _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
 {
   int result_size = IS_SYMOP (IC_RESULT (ic)) ? getSize (OP_SYM_TYPE (IC_RESULT(ic))) : 4;
+  sym_link *test = NULL;
+
+  if (IS_LITERAL (left))
+    test = left;
+  else if (IS_LITERAL (right))
+    test = right;
 
   switch (ic->op)
     {
@@ -162,7 +312,34 @@ _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
     case '%':
       return (getSize (left) <= 2 && IS_UNSIGNED (left) && getSize (right) <= 2 && IS_UNSIGNED (right));
     case '*':
-      return (result_size == 1 || getSize (left) <= 1 && getSize (right) <= 1 && result_size == 2 && IS_UNSIGNED (left) && IS_UNSIGNED (right));
+      {
+        if (result_size == 1 || getSize (left) <= 1 && getSize (right) <= 1 && result_size == 2 && IS_UNSIGNED (left) && IS_UNSIGNED (right))
+          return TRUE;
+
+        if ((getSize (left) != 2 || getSize (right) != 2) || result_size != 2 || !test)
+          return FALSE;
+
+        unsigned long long add, sub;
+        int topbit, nonzero;
+        
+
+        if (floatFromVal (valFromType (test)) < 0 || csdOfVal (&topbit, &nonzero, &add, &sub, valFromType (test)))
+          return FALSE;
+
+        int shifts = topbit;
+
+        // If the leading digits of the cse are 1 0 -1 we can use 0 1 1 instead to reduce the number of shifts.
+        if (topbit >= 2 && (add & (1ull << topbit)) && (sub & (1ull << (topbit - 2))))
+          shifts--;
+
+        wassert (nonzero);
+
+        // Shifts are 1 byte, additions and subtractions are 3 bytes.
+        if (shifts + 3 * (nonzero - 1) <= 9 - optimize.codeSize + 3 * optimize.codeSpeed)
+          return TRUE;
+
+        return FALSE;
+      }
     default:
       return FALSE;
     }
@@ -175,6 +352,23 @@ hasExtBitOp (int op, int size)
   return (op == GETABIT);
 }
 
+static const char *
+get_model (void)
+{
+  switch (options.model)
+    {
+    case MODEL_MEDIUM:
+      return ("stm8");
+      break;
+    case MODEL_LARGE:
+      return ("stm8-large");
+      break;
+    default:
+      werror (W_UNKNOWN_MODEL, __FILE__, __LINE__);
+      return "unknown";
+    }
+}
+
 /** $1 is always the basename.
     $2 is always the output file.
     $3 varies
@@ -203,9 +397,9 @@ PORT stm8_port =
   {
     glue,
     TRUE,                       /* We want stm8_genIVT to be triggered */
-    NO_MODEL,
-    NO_MODEL,
-    NULL,                       /* model == target */
+    MODEL_MEDIUM | MODEL_LARGE,
+    MODEL_MEDIUM,
+    &get_model,                 /* model string used as library destination */
   },
   {                             /* Assembler */
     stm8AsmCmd,
@@ -236,26 +430,39 @@ PORT stm8_port =
     NULL,
   },
   /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
-  { 1, 2, 2, 4, 8, 2, 2, 2, 1, 4, 4 },
+  {
+    1,                          /* char */
+    2,                          /* short */
+    2,                          /* int */
+    4,                          /* long */
+    8,                          /* long long */
+    2,                          /* near ptr */
+    2,                          /* far ptr */
+    2,                          /* generic ptr */
+    2,                          /* func ptr */
+    0,                          /* banked func ptr */
+    1,                          /* bit */
+    4,                          /* float */
+  },
   /* tags for generic pointers */
   { 0x00, 0x40, 0x60, 0x80 },   /* far, near, xstack, code */
   {
     "XSEG",
     "STACK",
-    "CODE",
-    "DATA",
+    "CODE",                     /* code */
+    "DATA",                     /* data */
     NULL,                       /* idata */
     NULL,                       /* pdata */
     NULL,                       /* xdata */
     NULL,                       /* bit */
-    "RSEG (ABS)",
+    "RSEG (ABS)",               /* reg */
     "GSINIT",                   /* static initialization */
     NULL,                       /* overlay */
-    "GSFINAL",
-    "HOME",
+    "GSFINAL",                  /* gsfinal */
+    "HOME",                     /* home */
     NULL,                       /* xidata */
     NULL,                       /* xinit */
-    NULL,                       /* const_name */
+    "CONST",                    /* const_name */
     "CABS (ABS)",               /* cabs_name */
     "DABS (ABS)",               /* xabs_name */
     NULL,                       /* iabs_name */
@@ -265,14 +472,32 @@ PORT stm8_port =
     NULL,
     1                           /* CODE  is read-only */
   },
-  { NULL, NULL },
-  { -1, 0, 7, 2, 0, 2 },        /* stack information */
-  /* Use more fine-grained control for multiplication / division. */
-  { 0, -1 },
-  { stm8_emitDebuggerSymbol },
+  { stm8_genExtraArea, NULL },
+  {                             /* stack information */
+    -1,                         /* direction */
+     0,
+     7,                         /* isr overhead */
+     2,                         /* call overhead */
+     0,
+     2,
+     1,                         /* sp points to next free stack location */
+  },     
+  { -1, TRUE },
+  { stm8_emitDebuggerSymbol,
+    {
+      stm8_dwarfRegNum,
+      0,                        /* cfiSame */
+      0,                        /* cfiUndef */
+      4,                        /* addressSize */
+      9,                        /* regNumRet */
+      SP_IDX,                   /* regNumSP */
+      0,                        /* regNumBP */
+      2,                        /* offsetSP */
+    },
+  },
   {
     32767,                      /* maxCount */
-     2,                         /* sizeofElement */
+    2,                          /* sizeofElement */
     {4, 5, 5},                  /* sizeofMatchJump[] - assuming operand in reg, inverse can be optimized away - would be much higher otherwise */
     {4, 5, 5},                  /* sizeofRangeCompare[] - same as above */
     3,                          /* sizeofSubtract - assuming 2 byte index, would be 2 otherwise */
@@ -281,22 +506,23 @@ PORT stm8_port =
   "_",
   stm8_init,
   stm8_parseOptions,
-  NULL,
+  stm8_options,
   NULL,
   stm8_finaliseOptions,
   stm8_setDefaultOptions,
   stm8_assignRegisters,
   stm8_getRegName,
+  0,
   NULL,
   stm8_keywords,
   NULL,
-  NULL,                         /* no genAssemblerEnd */
+  stm8_genAssemblerEnd,
   stm8_genIVT,
   0,                            /* no genXINIT code */
-  stm8_genInitStartup,                         /* genInitStartup */
+  stm8_genInitStartup,          /* genInitStartup */
   stm8_reset_regparm,
   stm8_reg_parm,
-  NULL,                         /* process_pragma */
+  stm8_process_pragma,          /* process_pragma */
   NULL,                         /* getMangledFunctionName */
   _hasNativeMulFor,             /* hasNativeMulFor */
   hasExtBitOp,                  /* hasExtBitOp */
