@@ -5,18 +5,12 @@
 
 #define NOTUSEDERROR() do {werror(E_INTERNAL_ERROR, __FILE__, __LINE__, "error in notUsed()");} while(0)
 
-//#define D(_s) { printf _s; fflush(stdout); }
+// #define D(_s) { printf _s; fflush(stdout); }
 #define D(_s)
 
 #define EQUALS(l, i) (!strcmp((l), (i)))
 #define ISINST(l, i) (!strncmp((l), (i), sizeof(i) - 1))
 
-// This function should behave just like C99 isblank(). Remove it, once MSVC supports isblank().
-static int _isblank(int c)
-{
-  return (c == ' ' || c == '\t');
-}
-
 typedef enum
 {
   S4O_CONDJMP,
@@ -33,6 +27,19 @@ static struct
   lineNode *head;
 } _G;
 
+static bool
+isInt(const char *str)
+{
+  int ret;
+  while(str[0] == '#' || str[0] == '(')
+    str++;
+  if(sscanf(str, "0x%x", &ret))
+    return(ret);
+  if(!sscanf(str, "%d", &ret))
+    return(false);
+  return(true);
+}
+
 static int
 readint(const char *str)
 {
@@ -108,7 +115,7 @@ isRelativeAddr(const char *what, const char *mode)
 static int
 isLabel(const char *what)
 {
-  const char *end;	
+  const char *end;    
   end = strchr(what, '+');
   if(!end)
     end = what + strlen(what);
@@ -126,13 +133,19 @@ isImmediate(const char *what)
 static int
 isShortoff(const char *what, const char *mode)
 {
-  return(isRelativeAddr(what, mode) && readint(what) <= 0xFF);
+  return(isRelativeAddr(what, mode) && isInt(what) && readint(what) <= 0xff);
 }
 
 static int
 isLongoff(const char *what, const char *mode)
 {
-  return(isRelativeAddr(what, mode) && readint(what) > 0xFF);
+  return(isRelativeAddr(what, mode) && (!isInt(what) || readint(what) > 0xff));
+}
+
+static int
+isPtr(const char *what)
+{
+  return(what[0] == '[' || what[0] == '(' && (what[1] == '[' || what[1] == '('));
 }
 
 static int
@@ -148,7 +161,7 @@ stm8instructionSize(const lineNode *pl)
   char *operand;
   char *op1start;
   char *op2start;
-  int i = 0;
+  
   operand = nextToken(pl->line);
   op1start = nextToken(NULL);
   op2start = nextToken(NULL);
@@ -237,17 +250,19 @@ stm8instructionSize(const lineNode *pl)
                 || EQUALS(operand, "rrc")
                 || EQUALS(operand, "tnz"))
   {
+    int i = 0;
+
     assert (op1start != NULL);
     if(!strcmp(op1start, "a") || !strcmp(op1start, "(x)"))
       return(1);
     if(!strcmp(op1start, "(y)"))
       return(2);
-    if(op1start[0] == '(')
+    if(op1start[0] == '('|| op1start[0] == '[')
       op1start++;
     if(strstr(op1start, ",y)"))
       i++; // costs extra byte for operating with y
     if(isLabel(op1start))
-      return(3);
+      return(4);
     if(readint(op1start) <= 0xFF)
       return(2+i);
     /* op1 > 0xFF */
@@ -309,6 +324,7 @@ stm8instructionSize(const lineNode *pl)
                 || EQUALS(operand, "sub")
                 || EQUALS(operand, "xor"))
   {
+    int i = 0;
     char suffix;
     assert (op1start != NULL && op2start != NULL);
     suffix = operand[strlen(operand)-1];
@@ -334,12 +350,16 @@ stm8instructionSize(const lineNode *pl)
       return(3);
     if(isLongoff(op1start, "y") || isLongoff(op2start, "y"))
       return(4);
+    if(isPtr(op1start) || isPtr(op2start))
+      return(4);
     if(strchr(op1start, 'y') || strchr(op2start, 'y'))
       i++; // costs extra byte for operating with y
     if(isLabel(op1start) || isLabel(op2start))
       return(3+i);
     if(isReg(op1start) && isReg(op2start))
       return(1+i);
+    if(!strcmp(op2start, "a"))
+      return(3);
     if(readint(op2start) <= 0xFF)
       return(2+i);
     else
@@ -347,7 +367,7 @@ stm8instructionSize(const lineNode *pl)
     return 4;
   }
 
-  /* mov costs 3, 4 or 5 bytes depending on it's addressing mode */
+  /* mov costs 3, 4 or 5 bytes depending on its addressing mode */
   if(EQUALS(operand, "mov")) {
     assert (op1start != NULL && op2start != NULL);
     if(isImmediate(op2start))
@@ -421,12 +441,16 @@ findLabel (const lineNode *pl)
 
   /* 1. extract label in opcode */
 
-  /* In each mcs51 jumping opcode the label is at the end of the opcode */
+  /* In each jump the label is at the end */
   p = strlen (pl->line) - 1 + pl->line;
 
-  /* scan backward until ',' or '\t' */
+  /* Skip trailing whitespace */
+  while(isspace(*p))
+    p--;
+
+  /* scan backward until space or ',' */
   for (; p > pl->line; p--)
-    if (*p == ',' || *p == '\t')
+    if (isspace(*p) || *p == ',')
       break;
 
   /* sanity check */
@@ -461,13 +485,13 @@ static bool argCont(const char *arg, char what)
   if (arg == NULL || strlen (arg) == 0 || !(what == 'a' || what == 'x' || what == 'y'))
     return FALSE;
 
-  while (_isblank ((unsigned char)(arg[0])))
+  while (isblank ((unsigned char)(arg[0])))
     arg++;
 
   if (arg[0] == ',')
     arg++;
 
-  while (_isblank ((unsigned char)(arg[0])))
+  while (isblank ((unsigned char)(arg[0])))
     arg++;
 
   if (arg[0] == '#')
@@ -479,6 +503,9 @@ static bool argCont(const char *arg, char what)
   if (strlen(arg) == 0)
     return FALSE;
 
+  if (arg[0] == '_' && what == 'a') // The STM8 has no a-relative addressing modes.
+    return FALSE;
+
   return (strchr(arg, what) != NULL);
 }
 
@@ -540,6 +567,26 @@ isReturned(const char *what)
     }
 }
 
+static bool
+stm8MightReadFlag(const lineNode *pl, const char *what)
+{
+  if (ISINST (pl->line, "push"))
+     return (pl->line[5] == 'c');
+
+  if (!strcmp (what, "n"))
+    return (ISINST (pl->line, "jrmi") || ISINST (pl->line, "jrpl") || ISINST (pl->line, "jrsge") || ISINST (pl->line, "jrsgte") || ISINST (pl->line, "jrsle") || ISINST (pl->line, "jrslt"));
+
+  if (!strcmp (what, "z"))
+    return (ISINST (pl->line, "jreq") || ISINST (pl->line, "jrne") || ISINST (pl->line, "jrsgte") || ISINST (pl->line, "jrsle"));
+
+  if (!strcmp (what, "c"))
+    return (ISINST (pl->line, "jrc") || ISINST (pl->line, "jrnc") || ISINST (pl->line, "jruge") || ISINST (pl->line, "jrugt") || ISINST (pl->line, "jrule") || ISINST (pl->line, "jrult") ||
+      ISINST (pl->line, "adc") || ISINST (pl->line, "sbc") ||
+      ISINST (pl->line, "ccf") || ISINST (pl->line, "rlc") || ISINST (pl->line, "rlcw") || ISINST (pl->line, "rrc") || ISINST (pl->line, "rrcw"));
+
+  return TRUE;
+}
+
 static bool
 stm8MightRead(const lineNode *pl, const char *what)
 {
@@ -550,17 +597,15 @@ stm8MightRead(const lineNode *pl, const char *what)
   else if (!strcmp (what, "yl") || !strcmp (what, "yh"))
     extra = 'y';
   else if (strcmp (what, "a") != 0)
-    return FALSE;
+    return stm8MightReadFlag(pl, what);
 
   if (!extra)
     {
       if (ISINST (pl->line, "adc")
-        || ISINST (pl->line, "add")
         || ISINST (pl->line, "and")
-        || ISINST (pl->line, "bcp")
+        || ISINST (pl->line, "bcp") && !ISINST (pl->line, "bcpl")
         || ISINST (pl->line, "cp")
         || ISINST (pl->line, "div")
-        || ISINST (pl->line, "exg")
         || ISINST (pl->line, "mul")
         || ISINST (pl->line, "or")
         || ISINST (pl->line, "rlwa")
@@ -570,8 +615,10 @@ stm8MightRead(const lineNode *pl, const char *what)
         || ISINST (pl->line, "xor"))
           return TRUE;
 
-      if (pl->line[4] == 'a' && (ISINST (pl->line, "cpl")
+      if ((ISINST (pl->line, "add")
+        || ISINST (pl->line, "cpl")
         || ISINST (pl->line, "dec")
+        || ISINST (pl->line, "exg")
         || ISINST (pl->line, "inc")
         || ISINST (pl->line, "neg")
         || ISINST (pl->line, "rlc")
@@ -581,11 +628,13 @@ stm8MightRead(const lineNode *pl, const char *what)
         || ISINST (pl->line, "sra")
         || ISINST (pl->line, "srl")
         || ISINST (pl->line, "sub")
-        || ISINST (pl->line, "tnz")))
+        || ISINST (pl->line, "tnz")) &&
+        pl->line[4] == 'a')
           return TRUE;
 
-      if ((pl->line[5] == 'a') && (ISINST (pl->line, "push")
-        || ISINST (pl->line, "swap")))
+      if ((ISINST (pl->line, "push")
+        || ISINST (pl->line, "swap")) &&
+        pl->line[5] == 'a')
           return TRUE;
 
       if ((ISINST (pl->line, "ld") || ISINST (pl->line, "ldf")) && argCont (strchr (pl->line, ','), 'a'))
@@ -599,10 +648,10 @@ stm8MightRead(const lineNode *pl, const char *what)
       if ((ISINST (pl->line, "exg") && !ISINST (pl->line, "exgw")) && strstr (strchr(pl->line, ','), what))
         return TRUE;
 
-      if (pl->line[4] == extra && (ISINST (pl->line, "div") && !ISINST (pl->line, "divw")))
+      if (((ISINST (pl->line, "div") || ISINST (pl->line, "mul")) && !ISINST (pl->line, "divw")) && pl->line[4] == extra)
         return TRUE;
 
-      if (pl->line[5] == extra && ISINST (pl->line, "addw")
+      if ((ISINST (pl->line, "addw")
         || ISINST (pl->line, "cplw")
         || ISINST (pl->line, "decw")
         || ISINST (pl->line, "incw")
@@ -616,18 +665,18 @@ stm8MightRead(const lineNode *pl, const char *what)
         || ISINST (pl->line, "sraw")
         || ISINST (pl->line, "srlw")
         || ISINST (pl->line, "subw")
-        || ISINST (pl->line, "addw")
-        || ISINST (pl->line, "tnzw"))
+        || ISINST (pl->line, "tnzw")) &&
+        pl->line[5] == extra)
           return TRUE;
 
-      if (pl->line[6] == extra && ISINST (pl->line, "pushw")
-        || ISINST (pl->line, "swapw"))
+      if (ISINST (pl->line, "pushw")
+        || ISINST (pl->line, "swapw") && pl->line[6] == extra)
           return TRUE;
 
       if (ISINST (pl->line, "cpw") && pl->line[4] == extra)
         return TRUE;
 
-      if (argCont (strchr (pl->line, ','), extra) && (ISINST (pl->line, "adc")
+      if ((strchr (pl->line, ',') ? argCont (strchr (pl->line, ','), extra) : argCont (strchr (pl->line, '('), extra)) && (ISINST (pl->line, "adc")
         || (ISINST (pl->line, "add") && !ISINST (pl->line, "addw"))
         || ISINST (pl->line, "and")
         || ISINST (pl->line, "bcp")
@@ -656,18 +705,28 @@ stm8MightRead(const lineNode *pl, const char *what)
         || (ISINST (pl->line, "ld") && !ISINST (pl->line, "ldw"))
         || ISINST (pl->line, "xor")))
           return TRUE;
+
+      if (ISINST (pl->line, "ld"))
+        {
+          char buf[64], *p;
+          strcpy (buf, pl->line);
+          if (!!(p = strstr (buf, "0x")) || !!(p = strstr (buf, "0X")))
+            p[0] = p[1] = ' ';
+          if (!!(p = strchr (buf, '(')) && !!strchr (p, extra))
+            return TRUE;
+        }
     }
 
-  if(ISINST(pl->line, "ret"))
+  if(ISINST(pl->line, "ret") || ISINST(pl->line, "retf"))
     return(isReturned(what));
-  else
-    return FALSE;
+
+  return FALSE;
 }
 
 static bool
 stm8UncondJump(const lineNode *pl)
 {
-  return (ISINST(pl->line, "jp\t") || ISINST(pl->line, "jra\t") || ISINST(pl->line, "jpf\t"));
+  return (ISINST(pl->line, "jp\t") || ISINST(pl->line, "jra\t") || ISINST(pl->line, "jrt\t") || ISINST(pl->line, "jpf\t"));
 }
 
 static bool
@@ -677,22 +736,59 @@ stm8CondJump(const lineNode *pl)
     ISINST(pl->line, "btjt\t") || ISINST(pl->line, "btjf\t"));
 }
 
+static bool
+stm8SurelyWritesFlag(const lineNode *pl, const char *what)
+{
+  if (!strcmp (what, "n") || !strcmp (what, "z"))
+    {
+      if (ISINST (pl->line, "addw") && !strcmp (pl->line + 5, "sp"))
+        return FALSE;
+      if (ISINST (pl->line, "sub") && !strcmp (pl->line + 4, "sp"))
+        return FALSE;
+      if (ISINST (pl->line, "ld") || ISINST (pl->line, "ldw")) // Todo: Improve accuracy of these two.
+        return FALSE;
+      if (ISINST (pl->line, "pop"))
+        return (pl->line[5] == 'c');
+      if (ISINST (pl->line, "bccm") || ISINST (pl->line, "bcpl") ||
+        ISINST (pl->line, "break") ||
+        ISINST (pl->line, "bres") || ISINST (pl->line, "bset") ||
+        ISINST (pl->line, "btjf") || ISINST (pl->line, "btjt") ||
+        ISINST (pl->line, "call") || ISINST (pl->line, "callf") || ISINST (pl->line, "callr") ||
+        ISINST (pl->line, "ccf") ||
+        ISINST (pl->line, "exg") || ISINST (pl->line, "exgw") ||
+        ISINST (pl->line, "halt") || ISINST (pl->line, "int") ||
+        ISINST (pl->line, "jp") || ISINST (pl->line, "jpf") ||
+        pl->line[0] == 'j' && pl->line[1] == 'r' ||
+        ISINST (pl->line, "mov") || ISINST (pl->line, "mul") ||
+        ISINST (pl->line, "nop") ||
+        ISINST (pl->line, "popw") || ISINST (pl->line, "push") || ISINST (pl->line, "pushw") ||
+        ISINST (pl->line, "rcf") ||
+        ISINST (pl->line, "ret") || ISINST (pl->line, "retf") ||
+        ISINST (pl->line, "rvf") || ISINST (pl->line, "scf") ||
+        ISINST (pl->line, "sim") || ISINST (pl->line, "trap") || ISINST (pl->line, "wfe") || ISINST (pl->line, "wfi"))
+        return FALSE;
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static bool
 stm8SurelyWrites(const lineNode *pl, const char *what)
 {
   char extra = 0;
   if (!strcmp (what, "xl") || !strcmp (what, "xh"))
     extra = 'x';
-  if (!strcmp (what, "yl") || !strcmp (what, "yh"))
+  else if (!strcmp (what, "yl") || !strcmp (what, "yh"))
     extra = 'y';
+  else if (strcmp (what, "a"))
+    return (stm8SurelyWritesFlag (pl, what));
 
   if (!extra)
     {
       if (ISINST (pl->line, "adc")
-        || ISINST (pl->line, "add")
         || ISINST (pl->line, "and")
         || ISINST (pl->line, "div")
-        || ISINST (pl->line, "exg")
         || ISINST (pl->line, "iret")
         || ISINST (pl->line, "or")
         || ISINST (pl->line, "rlwa")
@@ -701,9 +797,11 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         || ISINST (pl->line, "xor"))
           return TRUE;
 
-      if (pl->line[4] == 'a' && (ISINST (pl->line, "clr")
+      if ((ISINST (pl->line, "add")
+        || ISINST (pl->line, "clr")
         || ISINST (pl->line, "cpl")
         || ISINST (pl->line, "dec")
+        || ISINST (pl->line, "exg")
         || ISINST (pl->line, "inc")
         || ISINST (pl->line, "neg")
         || ISINST (pl->line, "pop")
@@ -714,13 +812,14 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         || ISINST (pl->line, "sra")
         || ISINST (pl->line, "srl")
         || ISINST (pl->line, "ldf")
-        || ISINST (pl->line, "sub")))
+        || ISINST (pl->line, "sub")) &&
+        pl->line[4] == 'a')
           return TRUE;
 
-      if (pl->line[5] == 'a' && ISINST (pl->line, "swap"))
+      if (ISINST (pl->line, "swap") && pl->line[5] == 'a')
         return TRUE;
 
-      if (pl->line[3] == 'a' && ISINST (pl->line, "ld"))
+      if (ISINST (pl->line, "ld") && pl->line[3] == 'a')
         return TRUE;
     }
   else
@@ -730,12 +829,13 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         || ISINST (pl->line, "iret"))
           return TRUE;
 
-      if (pl->line[4] == extra && ((ISINST (pl->line, "div") && !ISINST (pl->line, "divw"))
+      if (((ISINST (pl->line, "div") && !ISINST (pl->line, "divw"))
         || ISINST (pl->line, "ldw")
-        || ISINST (pl->line, "mul")))
+        || ISINST (pl->line, "mul"))
+        && pl->line[4] == extra)
           return TRUE;
 
-      if (pl->line[5] == extra && (ISINST (pl->line, "addw")
+      if ((ISINST (pl->line, "addw")
         || ISINST (pl->line, "clrw")
         || ISINST (pl->line, "cplw")
         || ISINST (pl->line, "decw")
@@ -750,10 +850,11 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         || ISINST (pl->line, "slaw")
         || ISINST (pl->line, "sraw")
         || ISINST (pl->line, "srlw")
-        || ISINST (pl->line, "subw")))
+        || ISINST (pl->line, "subw")) &&
+        pl->line[5] == extra)
           return TRUE;
 
-      if (pl->line[6] == extra && ISINST (pl->line, "swapw"))
+      if (ISINST (pl->line, "swapw") && pl->line[6] == extra)
         return TRUE;
 
       if ((ISINST (pl->line, "ld") && !ISINST (pl->line, "ldw") && !ISINST (pl->line, "ldf"))
@@ -770,7 +871,7 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
 static bool
 stm8SurelyReturns(const lineNode *pl)
 {
-  return(ISINST(pl->line, "ret"));
+  return(ISINST(pl->line, "ret") || ISINST(pl->line, "retf"));
 }
 
 /*-----------------------------------------------------------------*/
@@ -933,9 +1034,6 @@ stm8notUsed (const char *what, lineNode *endPl, lineNode *head)
   else if(strcmp(what, "y") == 0)
     return(stm8notUsed("yl", endPl, head) && stm8notUsed("yh", endPl, head));
 
-  if(strcmp(what, "a") && strcmp(what, "xl") && strcmp(what, "xh") && strcmp(what, "yl") && strcmp(what, "yh"))
-    return FALSE;
-
   _G.head = head;
 
   unvisitLines (_G.head);
