@@ -7,13 +7,15 @@ extern void genSTM08Code (iCode *);
 
 reg_info stm8_regs[] =
 {
-
   {REG_GPR, A_IDX,   "a"},
   {REG_GPR, XL_IDX,  "xl"},
   {REG_GPR, XH_IDX,  "xh"},
   {REG_GPR, YL_IDX,  "yl"},
   {REG_GPR, YH_IDX,  "yh"},
   {REG_CND, C_IDX,   "c"},
+  {REG_GPR, X_IDX,   "x"},
+  {REG_GPR, Y_IDX,   "y"},
+  {0, SP_IDX,        "sp"},
 };
 
 /* Flags to turn on debugging code.
@@ -178,7 +180,7 @@ spillThis (symbol *sym, bool force_spill)
   if (!(sym->remat || sym->usl.spillLoc) || (sym->usl.spillLoc && !sym->usl.spillLoc->onStack)) // stm8 port currently only supports on-stack spill locations in code generation.
     createStackSpil (sym);
 
-  /* mark it has spilt & put it in the spilt set */
+  /* mark it as spilt */
   sym->isspilt = sym->spillA = 1;
 
   if (force_spill)
@@ -261,7 +263,27 @@ regTypeNum (void)
     }
 }
 
-#if 0
+/** Transform weird SDCC handling of writes via pointers
+    into something more sensible. */
+static void
+transformPointerSet (eBBlock **ebbs, int count)
+{
+  /* for all blocks */
+  for (int i = 0; i < count; i++)
+    {
+      iCode *ic;
+
+      /* for all instructions do */
+      for (ic = ebbs[i]->sch; ic; ic = ic->next)
+        if (POINTER_SET (ic))
+          {
+            IC_LEFT (ic) = IC_RESULT (ic);
+            IC_RESULT (ic) = 0;
+            ic->op = SET_VALUE_AT_ADDRESS;
+          }
+    }
+}
+
 /** Register reduction for assignment.
  */
 static int
@@ -310,9 +332,6 @@ packRegsForAssign (iCode *ic, eBBlock *ebp)
 
           if (IS_SYMOP (IC_RESULT (dic)) && IC_RESULT (dic)->key == IC_RIGHT (ic)->key)
             {
-              if (POINTER_SET (dic))
-                dic = NULL;
-
               break;
             }
 
@@ -342,7 +361,7 @@ packRegsForAssign (iCode *ic, eBBlock *ebp)
     return 0;                   /* did not find */
 
   /* if assignment then check that right is not a bit */
-  if (ASSIGNMENT (ic) && !POINTER_SET (ic))
+  if (ic->op == '=')
     {
       sym_link *etype = operandType (IC_RESULT (dic));
       if (IS_BITFIELD (etype))
@@ -356,8 +375,18 @@ packRegsForAssign (iCode *ic, eBBlock *ebp)
         }
     }
 
+  /* For now eliminate 8-bit temporary variables only.
+     The STM8 instructions operating directly on memory
+     operands are 8-bit, so the most benefit is in 8-bit
+     operations. On the other hand, supporting wider
+     operations well in codegen is also more effort. */
+  if (bitsForType (operandType (IC_RESULT (dic))) > 8 &&
+    !((dic->op == LEFT_OP || dic->op == RIGHT_OP) && IS_OP_LITERAL (IC_RIGHT (dic)) && operandLitValue (IC_RIGHT (dic))  == 1 && // Can do wide shift by 1 in place.
+      IS_SYMOP (IC_LEFT (dic)) && IS_SYMOP (IC_RESULT (ic)) && OP_SYMBOL (IC_LEFT (dic)) == OP_SYMBOL (IC_RESULT (ic))))
+    return 0;
+
   /* if the result is on stack or iaccess then it must be
-     the same atleast one of the operands */
+     the same as at least one of the operands */
   if (OP_SYMBOL (IC_RESULT (ic))->onStack || OP_SYMBOL (IC_RESULT (ic))->iaccess)
     {
       /* the operation has only one symbol
@@ -381,7 +410,7 @@ pack:
       OP_SYMBOL (IC_RESULT (dic))->liveFrom = dic->seq;
     }
   /* delete from liverange table also
-     delete from all the points inbetween and the new
+     delete from all the points in between and the new
      one */
   for (sic = dic; sic != ic; sic = sic->next)
     {
@@ -398,6 +427,84 @@ pack:
   return 1;
 }
 
+/** Will reduce some registers for single use.
+ */
+static int
+packRegsForOneuse (iCode *ic, operand **opp, eBBlock *ebp)
+{
+  iCode *dic;
+
+  operand *op = *opp;
+//printf("packRegsForOneuse() at ic %d\n", ic->key);
+  /* if returning a literal then do nothing */
+  if (!IS_ITEMP (op))
+    return 0;
+
+  /* if rematerializable do nothing */
+  if (OP_SYMBOL (op)->remat)
+    return 0;
+
+  /* this routine will mark the symbol as used in one
+     instruction use only && if the definition is local
+     (ie. within the basic block) && has only one definition  */
+  if (bitVectnBitsOn (OP_USES (op)) != 1 || bitVectnBitsOn (OP_DEFS (op)) != 1)
+    return 0;
+
+  /* get the definition */
+  if (!(dic = hTabItemWithKey (iCodehTab, bitVectFirstBit (OP_DEFS (op)))))
+    return 0;
+//printf("Found dic %d\n", dic->key);
+  /* found the definition now check if it is local */
+  if (dic->seq < ebp->fSeq || dic->seq > ebp->lSeq)
+    return 0;                /* non-local */
+
+  /* for now handle results from assignments from globals only */
+  if (dic->op != '=' || !isOperandGlobal (IC_RIGHT (dic)))
+    return 0;
+  /* also make sure the intervenening instructions
+     don't have any thing in far space */
+  for (iCode *nic = dic->next; nic && nic != ic; nic = nic->next)
+    {
+      /* if there is an intervening function call then no */
+      if (nic->op == CALL || nic->op == PCALL)
+        return 0;
+
+      if (nic->op == GET_VALUE_AT_ADDRESS || nic->op == SET_VALUE_AT_ADDRESS)
+        return 0;
+
+      /* if address of & the result is remat the okay */
+      if (nic->op == ADDRESS_OF && OP_SYMBOL (IC_RESULT (nic))->remat)
+        continue;
+
+      if (IS_OP_VOLATILE (IC_LEFT (nic)) ||
+          IS_OP_VOLATILE (IC_RIGHT (nic)) ||
+          isOperandGlobal (IC_RESULT (nic)))
+        return 0;
+    }
+
+  /* Optimize out the assignment */
+  *opp = operandFromOperand (IC_RIGHT(dic));
+  (*opp)->isaddr = true;
+  
+  bitVectUnSetBit (OP_SYMBOL (op)->defs, dic->key);
+  bitVectUnSetBit (OP_SYMBOL (op)->uses, ic->key);
+
+  if (IS_ITEMP (IC_RESULT (dic)) && OP_SYMBOL (IC_RESULT (dic))->liveFrom > dic->seq)
+    OP_SYMBOL (IC_RESULT (dic))->liveFrom = dic->seq;
+
+  /* delete from liverange table also
+     delete from all the points in between and the new
+     one */
+  for (iCode *nic = dic; nic != ic; nic = nic->next) 
+    bitVectUnSetBit (nic->rlive, op->key);
+
+  remiCodeFromeBBlock (ebp, dic);
+
+  hTabDeleteItem (&iCodehTab, dic->key, ic, DELETE_ITEM, NULL);
+  
+  return 1;
+}
+
 /** Does some transformations to reduce register pressure.
  */
 static void
@@ -418,14 +525,62 @@ packRegisters (eBBlock * ebp)
       for (ic = ebp->sch; ic; ic = ic->next)
         {
           /* find assignment of the form TrueSym := iTempNN:1 */
-          if (ic->op == '=' && !POINTER_SET (ic))
+          if (ic->op == '=')
             change += packRegsForAssign (ic, ebp);
         }
       if (!change)
         break;
     }
+
+  for (ic = ebp->sch; ic; ic = ic->next)
+    {
+      D (D_ALLOC, ("packRegisters: looping on ic %p\n", ic));
+
+      /* Safe: address of a true sym is always constant. */
+      /* if this is an itemp & result of a address of a true sym
+         then mark this as rematerialisable   */
+      if (ic->op == ADDRESS_OF && !operandLitValue (IC_RIGHT (ic)) /* STM8 codegen cannot handle non-zero right operand in rematerialization yet */ && 
+          IS_ITEMP (IC_RESULT (ic)) && bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 && !IS_PARM (IC_RESULT (ic)) /* The receiving of the paramter isnot accounted for in DEFS */ &&
+          IS_TRUE_SYMOP (IC_LEFT (ic)) && !OP_SYMBOL (IC_LEFT (ic))->onStack)
+        {
+          OP_SYMBOL (IC_RESULT (ic))->remat = 1;
+          OP_SYMBOL (IC_RESULT (ic))->rematiCode = ic;
+          OP_SYMBOL (IC_RESULT (ic))->usl.spillLoc = NULL;
+        }
+
+      /* Safe: just propagates the remat flag */
+      /* if straight assignment then carry remat flag if this is the
+         only definition */
+      if (ic->op == '=' && IS_SYMOP (IC_RIGHT (ic)) && OP_SYMBOL (IC_RIGHT (ic))->remat &&
+        !isOperandGlobal (IC_RESULT (ic)) && bitVectnBitsOn (OP_SYMBOL (IC_RESULT (ic))->defs) == 1 && !IS_PARM (IC_RESULT (ic)) /* The receiving of the paramter isnot accounted for in DEFS */)
+        {
+          OP_SYMBOL (IC_RESULT (ic))->remat = OP_SYMBOL (IC_RIGHT (ic))->remat;
+          OP_SYMBOL (IC_RESULT (ic))->rematiCode = OP_SYMBOL (IC_RIGHT (ic))->rematiCode;
+        }
+
+      /* if cast to a generic pointer & the pointer being
+         cast is remat, then we can remat this cast as well */
+      if (ic->op == CAST &&
+        IS_SYMOP (IC_RIGHT (ic)) && OP_SYMBOL (IC_RIGHT (ic))->remat &&
+        !isOperandGlobal (IC_RESULT (ic)) && bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 && !IS_PARM (IC_RESULT (ic)) /* The receiving of the paramter isnot accounted for in DEFS */)
+        {
+          sym_link *to_type = operandType (IC_LEFT (ic));
+          sym_link *from_type = operandType (IC_RIGHT (ic));
+          if (IS_GENPTR (to_type) && IS_PTR (from_type))
+            {
+              OP_SYMBOL (IC_RESULT (ic))->remat = 1;
+              OP_SYMBOL (IC_RESULT (ic))->rematiCode = ic;
+              OP_SYMBOL (IC_RESULT (ic))->usl.spillLoc = NULL;
+            }
+        }
+
+      /* In some cases redundant moves can be eliminated */
+      if (ic->op == GET_VALUE_AT_ADDRESS || ic->op == SET_VALUE_AT_ADDRESS ||
+        ic->op == IFX && operandSize (IC_COND (ic)) == 1 ||
+        ic->op == IPUSH && operandSize (IC_LEFT (ic)) == 1)
+        packRegsForOneuse (ic, &(IC_LEFT (ic)), ebp);
+    }
 }
-#endif
 
 /**
   Mark variables for assignment by the register allocator.
@@ -467,7 +622,7 @@ serialRegMark (eBBlock ** ebbs, int count)
 
           /* some don't need registers, since there is no result. */
           if (SKIP_IC2 (ic) ||
-              ic->op == JUMPTABLE || ic->op == IFX || ic->op == IPUSH || ic->op == IPOP || (IC_RESULT (ic) && POINTER_SET (ic)))
+              ic->op == JUMPTABLE || ic->op == IFX || ic->op == IPUSH || ic->op == IPOP || ic->op == SET_VALUE_AT_ADDRESS)
             continue;
 
           /* now we need to allocate registers only for the result */
@@ -475,7 +630,7 @@ serialRegMark (eBBlock ** ebbs, int count)
             {
               symbol *sym = OP_SYMBOL (IC_RESULT (ic));
 
-              D (D_ALLOC, ("serialRegAssign: in loop on result %p\n", sym));
+              D (D_ALLOC, ("serialRegMark: in loop on result %p\n", sym));
 
               if (sym->isspilt && sym->usl.spillLoc) // todo: Remove once remat is supported!
                 {
@@ -493,7 +648,7 @@ serialRegMark (eBBlock ** ebbs, int count)
               if (!sym->nRegs ||
                   sym->isspilt || sym->for_newralloc || sym->liveTo <= ic->seq)
                 {
-                  D (D_ALLOC, ("serialRegAssign: won't live long enough.\n"));
+                  D (D_ALLOC, ("serialRegMark: won't live long enough.\n"));
                   continue;
                 }
 
@@ -527,7 +682,7 @@ verifyRegsAssigned (operand * op, iCode * ic)
 {
   symbol *sym;
   int i;
-  bool completly_in_regs;
+  bool completely_in_regs;
 
   if (!op)
     return;
@@ -545,10 +700,10 @@ verifyRegsAssigned (operand * op, iCode * ic)
   if (sym->isspilt)
     return;
 
-  for(i = 0, completly_in_regs = TRUE; i < sym->nRegs; i++)
+  for(i = 0, completely_in_regs = TRUE; i < sym->nRegs; i++)
     if (!sym->regs[i])
-      completly_in_regs = FALSE;
-  if (completly_in_regs)
+      completely_in_regs = FALSE;
+  if (completely_in_regs)
     return;
 
   spillThis (sym, FALSE);
@@ -597,23 +752,20 @@ void stm8_init_asmops (void);
 /* assignRegisters - assigns registers to each live range as need  */
 /*-----------------------------------------------------------------*/
 void
-stm8_assignRegisters (ebbIndex * ebbi)
+stm8_assignRegisters (ebbIndex *ebbi)
 {
   eBBlock **ebbs = ebbi->bbOrder;
   int count = ebbi->count;
   iCode *ic;
-#if 0 // ENABLE WHEN LINKER BUG #2198 IS FIXED
-  int i;
-#endif
 
   stm8_init_asmops();
 
+  transformPointerSet (ebbs, count);
+
   /* change assignments this will remove some
      live ranges reducing some register pressure */
-#if 0 // ENABLE WHEN LINKER BUG #2198 IS FIXED
-  for (i = 0; i < count; i++)
+  for (int i = 0; i < count; i++)
     packRegisters (ebbs[i]);
-#endif
 
   /* liveranges probably changed by register packing
      so we compute them again */
@@ -634,7 +786,7 @@ stm8_assignRegisters (ebbIndex * ebbi)
   /* Invoke optimal register allocator */
   ic = stm8_ralloc2_cc (ebbi);
 
-  /* Get spilllocs for all variables that have not been placed completly in regs */
+  /* Get spilllocs for all variables that have not been placed completely in regs */
   RegFix (ebbs, count);
 
   /* redo the offsets for stacked automatic variables */
@@ -657,7 +809,7 @@ stm8_assignRegisters (ebbIndex * ebbi)
           /* Invoke optimal register allocator */
           ic = stm8_ralloc2_cc (ebbi);
 
-          /* Get spilllocs for all variables that have not been placed completly in regs */
+          /* Get spilllocs for all variables that have not been placed completely in regs */
           RegFix (ebbs, count);
 
           redoStackOffsets ();
