@@ -44,12 +44,8 @@ typedef enum
   AOP_IY,
   /* Is pointed to by HL */
   AOP_HL,
-  /* Is in A */
-  AOP_ACC,
   /* Is in H and L */
   AOP_HLREG,
-  /* Simple literal. */
-  AOP_SIMPLELIT,
   /* Is in the extended stack pointer (IY on the Z80) */
   AOP_EXSTK,
   /* Is referenced by a pointer in a register pair. */
@@ -79,7 +75,6 @@ typedef struct asmop
     char *aop_immd;             /* if immediate others are implied */
     int aop_stk;                /* stack offset when AOP_STK */
     const char *aop_str[4];     /* just a string array containing the location */
-    unsigned long aop_simplelit;        /* Just the value. */
     int aop_pairId;             /* The pair ID */
   }
   aopu;
