@@ -24,12 +24,11 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include <unistd.h>
 #endif
 
-char ProgName[PATH_MAX];
-char LibName[PATH_MAX];
-char LibNameTmp[PATH_MAX];
-char IndexName[PATH_MAX];
-char AdbName[PATH_MAX];
-char ListName[PATH_MAX];
+char ProgName[FILENAME_MAX];
+char LibName[FILENAME_MAX];
+char LibNameTmp[FILENAME_MAX];
+char IndexName[FILENAME_MAX];
+char ListName[FILENAME_MAX];
 
 char **RelName;
 int NumRelFiles=0;
@@ -58,7 +57,10 @@ void GetNameFromPath(char * path, char * name)
     int i, j;
 
     for(i=0; path[i]!=0; i++);
-    for(; (path[i]!='\\')&&(path[i]!='/')&&(i>=0); i--);
+    for(; i>=0; i--)
+    {
+    	if((path[i]=='\\') || (path[i]=='/')) break;
+    }
     for(j=0, i++; (path[i]!='.')&&(path[i]!=0); i++, j++) name[j]=path[i];
     name[j]=0;
 }
@@ -68,15 +70,26 @@ void ChangeExtension(char * path, char * ext)
     int i;
 
     for(i=0; path[i]!=0; i++);
-    for(; (path[i]!='.')&&(path[i]!='\\')&&(path[i]!='/')&&(i>=0); i--);
-    if(path[i]=='.')
+    for(; i>=0; i--)
     {
-        path[i+1]=0;
-        strcat(path, ext);
+    	if( (path[i]=='.') || (path[i]=='\\') || (path[i]=='/') ) break;
+    }
+    if(i>=0)
+    {
+	    if(path[i]=='.')
+	    {
+	        path[i+1]=0;
+	        strcat(path, ext);
+	    }
+	    else
+	    {
+	        printf("ERROR: Filename '%s' must have an extension\n", path);
+	        exit(1);
+	    }
     }
     else
     {
-        printf("ERROR: Filename '%s' must have an extension\n", path);
+        printf("ERROR: Empty filename\n");
         exit(1);
     }
 }
@@ -169,6 +182,11 @@ void ProcLineOptions (int argc, char **argv)
                     else
                     {
                         cont_par++;
+                        if(strlen(LibName) > FILENAME_MAX - 1)
+                        {
+                            printf("ERROR: Library name too long.\n");
+                            exit(2);
+                        }
                         strcpy(LibName, argv[j]);
                     }
                 break;
@@ -183,16 +201,15 @@ void ProcLineOptions (int argc, char **argv)
                         RelName = (char **) calloc (1, sizeof (char *));
                         if(RelName==NULL)
                         {
-                            printf("ERROR: Insuficient memory.\n");
+                            printf("ERROR: Insufficient memory.\n");
                             exit(2);
                         }
-                        RelName[0]=(char *)malloc(PATH_MAX);
+                        RelName[0]=strdup(argv[j]);
                         if(RelName[0]==NULL)
                         {
-                            printf("ERROR: Insuficient memory.\n");
+                            printf("ERROR: Insufficient memory.\n");
                             exit(2);
                         }
-                        strcpy(RelName[0], argv[j]);
                     }
                 break;
 
@@ -202,16 +219,15 @@ void ProcLineOptions (int argc, char **argv)
                     RelName = (char **) realloc (RelName, NumRelFiles * sizeof (char *));
                     if(RelName==NULL)
                     {
-                        printf("ERROR: Insuficient memory.\n");
+                        printf("ERROR: Insufficient memory.\n");
                         exit(2);
                     }
-                    RelName[NumRelFiles-1]=(char *)malloc(PATH_MAX);
+                    RelName[NumRelFiles-1]=strdup(argv[j]);
                     if(RelName[NumRelFiles-1]==NULL)
                     {
-                        printf("ERROR: Insuficient memory.\n");
+                        printf("ERROR: Insufficient memory.\n");
                         exit(2);
                     }
-                    strcpy(RelName[NumRelFiles-1], argv[j]);
                 break;
             }
         }
@@ -237,6 +253,7 @@ void AddRel(char * RelName)
     char symname[MAXLINE+1];
     char c;
     int IsDOSStyle=0;
+    char *AdbName = 0;
 
     strcpy(LibNameTmp, LibName);
     ChangeExtension(LibNameTmp, "__L");
@@ -244,7 +261,7 @@ void AddRel(char * RelName)
     strcpy(IndexName, LibName);
     ChangeExtension(IndexName, "__I");
 
-    strcpy(AdbName, RelName);
+    AdbName = strdup (RelName);
     ChangeExtension(AdbName, "adb");
 
     lib=fopen(LibName, "r");
@@ -256,6 +273,7 @@ void AddRel(char * RelName)
         {
             printf("ERROR: Couldn't open file '%s'\n", RelName);
             if(lib!=NULL) fclose(lib);
+            free(AdbName);
             return;
         }
     }
@@ -267,6 +285,7 @@ void AddRel(char * RelName)
         printf("ERROR: Couldn't create temporary file '%s'\n", LibNameTmp);
         if(lib!=NULL) fclose(lib);
         fclose(rel);
+        free(AdbName);
         return;
     }
     fprintf(newlib, "<FILES>\n\n");
@@ -278,13 +297,15 @@ void AddRel(char * RelName)
         if(lib!=NULL) fclose(lib);
         fclose(newlib);
         fclose(rel);
+        free(AdbName);
         return;
     }
 
     if(lib!=NULL) while(!feof(lib))
     {
         FLine[0]=0;
-        fgets(FLine, MAXLINE, lib);
+        if (fgets(FLine, MAXLINE, lib) == NULL)
+          break;
         CleanLine(FLine);
 
         switch(state)
@@ -293,7 +314,8 @@ void AddRel(char * RelName)
                 if(EQ(FLine, "<FILE>"))
                 {
                     FLine[0]=0;
-                    fgets(FLine, MAXLINE, lib);
+                    if (fgets(FLine, MAXLINE, lib) == NULL)
+                      break;
                     CleanLine(FLine);
                     if(NEQ(FLine, ModName))
                     {
@@ -302,7 +324,7 @@ void AddRel(char * RelName)
                         fprintf(libindex, "<MODULE>\n%s %ld\n", FLine, newlibpos);
                         state++;
                     }
-                }                
+                }
             break;
             case 1:
                 fprintf(newlib, "%s\n", FLine);
@@ -335,7 +357,8 @@ void AddRel(char * RelName)
         while(!feof(rel))
         {
             FLine[0]=0;
-            fgets(FLine, MAXLINE, rel);
+            if (fgets(FLine, MAXLINE, rel) == NULL)
+              break;
             CleanLine(FLine);
             if(strlen(FLine)>0)
             {
@@ -350,14 +373,15 @@ void AddRel(char * RelName)
         fclose(rel);
         fprintf(libindex, "</MODULE>\n");
         fprintf(newlib, "</REL>\n<ADB>\n");
-    
+
         adb=fopen(AdbName, "r");
         if(adb!=NULL)
         {
             while(!feof(rel))
             {
                 FLine[0]=0;
-                fgets(FLine, MAXLINE, adb);
+                if (fgets(FLine, MAXLINE, adb) == NULL)
+                  break;
                 CleanLine(FLine);
                 if(strlen(FLine)>0)
                 {
@@ -397,7 +421,8 @@ void AddRel(char * RelName)
     while(!feof(libindex))
     {
         FLine[0]=0;
-        fgets(FLine, MAXLINE, libindex);
+        if (fgets(FLine, MAXLINE, libindex) == NULL)
+          break;
         fprintf(lib, "%s", FLine);
     }
     fprintf(lib, "\n</INDEX>\n\n");
@@ -405,7 +430,8 @@ void AddRel(char * RelName)
     while(!feof(newlib))
     {
         FLine[0]=0;
-        fgets(FLine, MAXLINE, newlib);
+        if (fgets(FLine, MAXLINE, newlib) == NULL)
+          break;
         fprintf(lib, "%s", FLine);
     }
     fprintf(lib, "\n</FILES>\n\n");
@@ -417,19 +443,23 @@ void AddRel(char * RelName)
 
     remove(LibNameTmp);
     remove(IndexName);
+
+    free(AdbName);
 }
 
 void ExtractRel(char * RelName)
 {
     int state=0;
+    char *AdbName = 0;
 
-    strcpy(AdbName, RelName);
+    AdbName = strdup(RelName);
     ChangeExtension(AdbName, "adb");
 
     lib=fopen(LibName, "r");
     if(lib==NULL)
     {
         printf("ERROR: Couldn't open file '%s'\n", LibName);
+        free(AdbName);
         return;
     }
 
@@ -438,6 +468,7 @@ void ExtractRel(char * RelName)
     {
         printf("ERROR: Couldn't create file '%s'\n", RelName);
         fclose(lib);
+        free(AdbName);
         return;
     }
     GetNameFromPath(RelName, ModName);
@@ -448,6 +479,7 @@ void ExtractRel(char * RelName)
         printf("ERROR: Couldn't create file '%s'\n", AdbName);
         fclose(lib);
         fclose(rel);
+        free(AdbName);
         return;
     }
 
@@ -455,7 +487,8 @@ void ExtractRel(char * RelName)
     {
         if(state==5) break;
         FLine[0]=0;
-        fgets(FLine, MAXLINE, lib);
+        if (fgets(FLine, MAXLINE, lib) == NULL)
+          break;
         CleanLine(FLine);
 
         switch(state)
@@ -464,10 +497,11 @@ void ExtractRel(char * RelName)
                 if(EQ(FLine, "<FILE>"))
                 {
                     FLine[0]=0;
-                    fgets(FLine, MAXLINE, lib);
+                    if (fgets(FLine, MAXLINE, lib) == NULL)
+                      break;
                     CleanLine(FLine);
                     if(EQ(FLine, ModName)) state=1;
-                }                
+                }
             break;
             case 1:
                 if(EQ(FLine, "<REL>")) state=2;
@@ -486,13 +520,15 @@ void ExtractRel(char * RelName)
                     state=5;
                 else
                     fprintf(adb, "%s\n", FLine);
-            break; 
+            break;
         }
     }
-    
+
     fclose(rel);
     fclose(lib);
     fclose(adb);
+
+    free(AdbName);
 }
 
 void DumpSymbols(void)
@@ -507,19 +543,21 @@ void DumpSymbols(void)
     }
 
     FLine[0]=0;
-    fgets(FLine, MAXLINE, lib);
+    if (fgets(FLine, MAXLINE, lib) == NULL)
+      return;
     CleanLine(FLine);
     if(NEQ(FLine, "<SDCCLIB>"))
     {
-        printf("ERROR: File '%s' was not created with '%s'\n", LibName, ProgName); 
+        printf("ERROR: File '%s' was not created with '%s'\n", LibName, ProgName);
         return;
     }
-    
+
     while(!feof(lib))
     {
         if(state==3) break;
         FLine[0]=0;
-        fgets(FLine, MAXLINE, lib);
+        if (fgets(FLine, MAXLINE, lib) == NULL)
+          break;
         CleanLine(FLine);
 
         switch(state)
@@ -531,7 +569,8 @@ void DumpSymbols(void)
                 if(EQ(FLine, "<MODULE>"))
                 {
                     FLine[0]=0;
-                    fgets(FLine, MAXLINE, lib);
+                    if (fgets(FLine, MAXLINE, lib) == NULL)
+                      break;
                     sscanf(FLine, "%s", ModName);
                     if(action==OPT_DUMP_SYM)
                     {
@@ -559,14 +598,14 @@ void DumpSymbols(void)
             break;
         }
     }
-    
+
     fclose(lib);
 }
 
 int fileexist(char * fname)
 {
     FILE * fp;
-    
+
     fp=fopen(fname, "r");
     if(fp==NULL) return 0;
     fclose(fp);
@@ -579,8 +618,8 @@ void AddList(void)
     char *cc;
     char *as;
     char CmdLine[1024];
-    char SrcName[PATH_MAX];
-    char RelName[PATH_MAX];
+    char SrcName[FILENAME_MAX];
+    char RelName[FILENAME_MAX];
 
     list=fopen(ListName, "r");
     if(list==NULL)
@@ -596,7 +635,8 @@ void AddList(void)
     while(!feof(list))
     {
         RelName[0]=0;
-        fgets(RelName, PATH_MAX, list);
+        if (fgets(RelName, FILENAME_MAX, list) == NULL)
+          break;
         CleanLine(RelName);
         if(strlen(RelName)>0) //Skip empty lines
         {
@@ -614,7 +654,8 @@ void AddList(void)
                 {
                     sprintf(CmdLine, "%s %s", cc, SrcName);
                     printf("%s\n", CmdLine);
-                    system(CmdLine);
+                    if (system(CmdLine))
+                      continue;
                 }
             }
             else if(as!=NULL)
@@ -624,7 +665,8 @@ void AddList(void)
                 {
                     sprintf(CmdLine, "%s %s", as, SrcName);
                     printf("%s\n", CmdLine);
-                    system(CmdLine);
+                    if (system(CmdLine))
+                      continue;
                 }
             }
 
@@ -660,7 +702,7 @@ int main(int argc, char **argv)
             for(j=0; j<NumRelFiles; j++) free(RelName[j]);
             free(RelName);
         break;
-        
+
         case OPT_ADD_LIST:
             AddList();
         break;
@@ -671,7 +713,7 @@ int main(int argc, char **argv)
             for(j=0; j<NumRelFiles; j++) free(RelName[j]);
             free(RelName);
         break;
-        
+
         case OPT_DUMP_SYM:
         case OPT_DUMP_MOD:
             DumpSymbols();
@@ -679,3 +721,4 @@ int main(int argc, char **argv)
     }
     return 0; //Success!!!
 }
+
