@@ -25,10 +25,6 @@
 #define TOLERANCE (1e-5)
 
 /* now exceptions for targets/functions which would not pass */
-#if defined(__SDCC_hc08) || defined(__SDCC_s08)
-#   define EXPF_DISABLED  (1)
-#endif
-
 #if defined(__SDCC_pic16)
 #   define FLOORF_DISABLED  (1)
 #   define NEG_DISABLED  (1)
