@@ -8,9 +8,7 @@
 #pragma std_c99
 #endif
 
-// Todo: Enable when sdcc supports long long modulo!
-#if 0
-
+#if !defined (__SDCC_pic14)
 /* PR c/19606
    The C front end used to shorten the type of a division to a type
    that does not preserve the semantics of the original computation.
@@ -33,7 +31,8 @@ bar (void)
 void
 testTortureExecute (void)
 {
-#if 0
+#if !defined (__SDCC_pic14)
+#if !defined (PORT_HOST) // failed in test-host
   int r;
 
   r = foo ();
@@ -46,4 +45,5 @@ testTortureExecute (void)
 
   return;
 #endif
+#endif
 }
