@@ -6,6 +6,8 @@
 #if !defined(__SDCC_z80) && !defined(__SDCC_z180) && !defined(__SDCC_r2k) && !defined(__SDCC_r3ka) && !defined(__SDCC_tlcs90)
 #define __z88dk_fastcall
 #define __z88dk_callee
+#endif
+#if !defined(__SDCC_z80) && !defined(__SDCC_z180) && !defined(__SDCC_r2k) && !defined(__SDCC_r3ka) && !defined(__SDCC_tlcs90)
 #define __smallc
 #endif
 
@@ -21,7 +23,7 @@ unsigned int f2(unsigned int c) __z88dk_fastcall
 	return c + 1;
 }
 
-#if !defined(__SDCC_hc08) && !defined(__SDCC_s08) 
+#if !defined(__SDCC_hc08) && !defined(__SDCC_s08)
 unsigned long int f4(unsigned long int c) __z88dk_fastcall
 #else
 unsigned long int f4(unsigned long int c) __z88dk_fastcall __reentrant
@@ -32,13 +34,13 @@ unsigned long int f4(unsigned long int c) __z88dk_fastcall __reentrant
 
 unsigned char (*p1)(unsigned char) __z88dk_fastcall;
 unsigned int (*p2)(unsigned int) __z88dk_fastcall;
-#if !defined(__SDCC_hc08) && !defined(__SDCC_s08) 
+#if !defined(__SDCC_hc08) && !defined(__SDCC_s08)
 unsigned long int (*p4)(unsigned long int) __z88dk_fastcall;
 #else
 unsigned long int (*p4)(unsigned long int) __z88dk_fastcall __reentrant;
 #endif
 
-/* __smallc and __z88dk_callee are currently only supported on the caller side */
+/* __z88dk_callee is currently only supported on the caller side */
 
 void s1(int x, int y) __smallc;
 
